@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: $ */
+/* $Id: FopFactory.java 637791 2008-03-17 08:59:51Z jeremias $ */
 
 package org.apache.fop.apps;
 
@@ -36,15 +36,16 @@ import javax.xml.transform.URIResolver;
 import org.xml.sax.SAXException;
 
 import org.apache.avalon.framework.configuration.Configuration;
-
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.xmlgraphics.image.loader.ImageContext;
+import org.apache.xmlgraphics.image.loader.ImageManager;
+
 import org.apache.fop.fo.ElementMapping;
 import org.apache.fop.fo.ElementMappingRegistry;
 import org.apache.fop.fonts.FontCache;
 import org.apache.fop.hyphenation.HyphenationTreeResolver;
-import org.apache.fop.image.ImageFactory;
 import org.apache.fop.layoutmgr.LayoutManagerMaker;
 import org.apache.fop.render.RendererFactory;
 import org.apache.fop.render.XMLHandlerRegistry;
@@ -57,7 +58,7 @@ import org.apache.fop.util.ContentHandlerFactoryRegistry;
  * Information that may potentially be different for each rendering run can be
  * found and managed in the FOUserAgent.
  */
-public class FopFactory {
+public class FopFactory implements ImageContext {
     
     /** logger instance */
     private static Log log = LogFactory.getLog(FopFactory.class);
@@ -80,8 +81,8 @@ public class FopFactory {
 
     private ColorSpaceCache colorSpaceCache = null;
     
-    /** Image factory for creating fop image objects */
-    private ImageFactory imageFactory;
+    /** Image manager for loading and caching image objects */
+    private ImageManager imageManager;
 
     /** Configuration layer used to configure fop */
     private FopFactoryConfigurator config = null;
@@ -150,7 +151,7 @@ public class FopFactory {
         this.elementMappingRegistry = new ElementMappingRegistry(this);
         this.foURIResolver = new FOURIResolver(validateUserConfigStrictly());
         this.colorSpaceCache = new ColorSpaceCache(foURIResolver);
-        this.imageFactory = new ImageFactory();        
+        this.imageManager = new ImageManager(this);
         this.rendererFactory = new RendererFactory();
         this.xmlHandlers = new XMLHandlerRegistry();
         this.ignoredNamespaces = new java.util.HashSet();
@@ -188,7 +189,7 @@ public class FopFactory {
      * @throws FOPException when the constructor fails
      */
     public Fop newFop(String outputFormat) throws FOPException {
-        return new Fop(outputFormat, newFOUserAgent());
+        return newFop(outputFormat, newFOUserAgent());
     }
 
     /**
@@ -205,10 +206,7 @@ public class FopFactory {
      * @throws FOPException  when the constructor fails
      */
     public Fop newFop(String outputFormat, FOUserAgent userAgent) throws FOPException {
-        if (userAgent == null) {
-            throw new NullPointerException("The userAgent parameter must not be null!");
-        }
-        return new Fop(outputFormat, userAgent);
+        return newFop(outputFormat, userAgent, null);
     }
     
     /**
@@ -223,7 +221,7 @@ public class FopFactory {
      * @throws FOPException when the constructor fails
      */
     public Fop newFop(String outputFormat, OutputStream stream) throws FOPException {
-        return new Fop(outputFormat, newFOUserAgent(), stream);
+        return newFop(outputFormat, newFOUserAgent(), stream);
     }
 
     /**
@@ -287,9 +285,12 @@ public class FopFactory {
         return this.contentHandlerFactoryRegistry;
     }
 
-    /** @return the image factory */
-    public ImageFactory getImageFactory() {
-        return this.imageFactory;
+    /**
+     * Returns the image manager.
+     * @return the image manager
+     */
+    public ImageManager getImageManager() {
+        return this.imageManager;
     }
 
     /**
@@ -318,11 +319,10 @@ public class FopFactory {
     }
 
     /**
-     * cleans the base url
-     * @param base
-     * @return
-     * @throws MalformedURLException
-     * @throws URISyntaxException 
+     * Checks if the given base URL is acceptable. It also normalizes the URL.
+     * @param base the base URL to check
+     * @return the normalized URL
+     * @throws MalformedURLException if there's a problem with a file URL
      */
     private String checkBaseURL(String base) throws MalformedURLException {
         if (!base.endsWith("/")) {
@@ -349,8 +349,7 @@ public class FopFactory {
     /**
      * Sets the base URL.
      * @param base base URL
-     * @throws MalformedURLException 
-     * @throws URISyntaxException 
+     * @throws MalformedURLException if there's a problem with a file URL
      */
     public void setBaseURL(String base) throws MalformedURLException {
         this.base = checkBaseURL(base);
@@ -367,8 +366,7 @@ public class FopFactory {
     /**
      * Sets the font base URL.
      * @param fontBase font base URL
-     * @throws MalformedURLException 
-     * @throws URISyntaxException 
+     * @throws MalformedURLException if there's a problem with a file URL
      */
     public void setFontBaseURL(String fontBase) throws MalformedURLException {
         this.fontBase = checkBaseURL(fontBase);
@@ -387,8 +385,7 @@ public class FopFactory {
     /**
      * Sets the hyphen base URL.
      * @param hyphenBase hythen base URL
-     * @throws MalformedURLException 
-     * @throws URISyntaxException 
+     * @throws MalformedURLException if there's a problem with a file URL
      * */
     public void setHyphenBaseURL(final String hyphenBase) throws MalformedURLException {
         if (hyphenBase != null) {
@@ -405,7 +402,7 @@ public class FopFactory {
     /**
      * Sets the URI Resolver. It is used for resolving factory-level URIs like hyphenation
      * patterns and as backup for URI resolution performed during a rendering run. 
-     * @param resolver the new URI resolver
+     * @param uriResolver the new URI resolver
      */
     public void setURIResolver(URIResolver uriResolver) {
         foURIResolver.setCustomURIResolver(uriResolver);
@@ -425,8 +422,8 @@ public class FopFactory {
     }
     
     /**
-     * sets the HyphenationTreeResolver
-     * @param hyphResolver
+     * Sets the HyphenationTreeResolver to be used for resolving user-supplied hyphenation files.
+     * @param hyphResolver the HyphenationTreeResolver instance
      */
     public void setHyphenationTreeResolver(HyphenationTreeResolver hyphResolver) {
         this.hyphResolver = hyphResolver;
@@ -709,6 +706,10 @@ public class FopFactory {
         return (this.fontCache != null);
     }
 
+    /**
+     * Returns the font cache instance used by this factory.
+     * @return the font cache
+     */
     public FontCache getFontCache() {
         return this.fontCache;
     }
@@ -748,5 +749,6 @@ public class FopFactory {
      */
     public ColorSpace getColorSpace(String baseUri, String iccProfileSrc) {
         return colorSpaceCache.get(baseUri, iccProfileSrc);
-    }    
+    }
+
 }
