@@ -15,36 +15,37 @@
  * limitations under the License.
  */
 
-/* $Id: AreaTreeHandler.java 554094 2007-07-07 00:04:25Z adelmelle $ */
+/* $Id: AreaTreeHandler.java 627367 2008-02-13 12:03:30Z maxberger $ */
 
 package org.apache.fop.area;
 
 // Java
 import java.io.OutputStream;
-import java.util.List;
 import java.util.Iterator;
+import java.util.List;
 
-// XML
 import org.xml.sax.SAXException;
 
-// Apache
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.apps.FormattingResults;
 import org.apache.fop.datatypes.Numeric;
 import org.apache.fop.fo.FOEventHandler;
 import org.apache.fop.fo.extensions.ExtensionAttachment;
+import org.apache.fop.fo.extensions.ExternalDocument;
+import org.apache.fop.fo.extensions.destination.Destination;
+import org.apache.fop.fo.pagination.AbstractPageSequence;
 import org.apache.fop.fo.pagination.PageSequence;
 import org.apache.fop.fo.pagination.Root;
 import org.apache.fop.fo.pagination.bookmarks.BookmarkTree;
-import org.apache.fop.layoutmgr.PageSequenceLayoutManager;
+import org.apache.fop.layoutmgr.ExternalDocumentLayoutManager;
 import org.apache.fop.layoutmgr.LayoutManagerMaker;
 import org.apache.fop.layoutmgr.LayoutManagerMapping;
-
-import org.apache.fop.area.DestinationData;
-import org.apache.fop.fo.extensions.destination.Destination;
+import org.apache.fop.layoutmgr.PageSequenceLayoutManager;
+import org.apache.fop.layoutmgr.TopLevelLayoutManager;
 
 /**
  * Area tree handler for formatting objects.
@@ -81,7 +82,7 @@ public class AreaTreeHandler extends FOEventHandler {
     // The formatting results to be handed back to the caller.
     private FormattingResults results = new FormattingResults();
 
-    private PageSequenceLayoutManager prevPageSeqLM;
+    private TopLevelLayoutManager prevPageSeqLM;
 
     private int idGen = 0;
 
@@ -188,12 +189,12 @@ public class AreaTreeHandler extends FOEventHandler {
         }
     }
 
-    /**
-     * @see org.apache.fop.fo.FOEventHandler
-     * @param pageSequence
-     *            is the pageSequence being started
-     */
+    /** {@inheritDoc} */
     public void startPageSequence(PageSequence pageSequence) {
+        startAbstractPageSequence(pageSequence);
+    }
+    
+    private void startAbstractPageSequence(AbstractPageSequence pageSequence) {
         rootFObj = pageSequence.getRoot();
         finishPrevPageSequence(pageSequence.getInitialPageNumber());
         pageSequence.initPageNumber();
@@ -237,6 +238,26 @@ public class AreaTreeHandler extends FOEventHandler {
         }
     }
 
+    /** {@inheritDoc} */
+    public void startExternalDocument(ExternalDocument document) {
+        startAbstractPageSequence(document);
+    }
+
+    /** {@inheritDoc} */
+    public void endExternalDocument(ExternalDocument document) {
+        if (statistics != null) {
+            statistics.end();
+        }
+        
+        ExternalDocumentLayoutManager edLM;
+        edLM = getLayoutManagerMaker().makeExternalDocumentLayoutManager(this, document);
+        edLM.activateLayout();
+        // preserve the current PageSequenceLayoutManger for the
+        // force-page-count check at the beginning of the next PageSequence
+        prevPageSeqLM = edLM;
+        
+    }
+
     /**
      * Called by the PageSequenceLayoutManager when it is finished with a
      * page-sequence.
@@ -244,7 +265,7 @@ public class AreaTreeHandler extends FOEventHandler {
      * @param pageSequence the page-sequence just finished
      * @param pageCount The number of pages generated for the page-sequence
      */
-    public void notifyPageSequenceFinished(PageSequence pageSequence,
+    public void notifyPageSequenceFinished(AbstractPageSequence pageSequence,
             int pageCount) {
         this.results.haveFormattedPageSequence(pageSequence, pageCount);
         if (log.isDebugEnabled()) {
@@ -261,25 +282,26 @@ public class AreaTreeHandler extends FOEventHandler {
 
         finishPrevPageSequence(null);
         // process fox:destination elements
-        List destinationList = rootFObj.getDestinationList();
-        if (destinationList != null) {
-            while (destinationList.size() > 0) {
-                Destination destination = (Destination) destinationList.remove(0);
-                DestinationData destinationData = new DestinationData(destination);
-                addOffDocumentItem(destinationData);
+        if (rootFObj != null) {
+            List destinationList = rootFObj.getDestinationList();
+            if (destinationList != null) {
+                while (destinationList.size() > 0) {
+                    Destination destination = (Destination) destinationList.remove(0);
+                    DestinationData destinationData = new DestinationData(destination);
+                    addOffDocumentItem(destinationData);
+                }
+            }
+            // process fo:bookmark-tree
+            BookmarkTree bookmarkTree = rootFObj.getBookmarkTree();
+            if (bookmarkTree != null) {
+                BookmarkData data = new BookmarkData(bookmarkTree);
+                addOffDocumentItem(data);
+                if (!data.isResolved()) {
+                    // bookmarks did not fully resolve, add anyway. (hacky? yeah)
+                    model.handleOffDocumentItem(data);
+                }
             }
         }
-        // process fo:bookmark-tree
-        BookmarkTree bookmarkTree = rootFObj.getBookmarkTree();
-        if (bookmarkTree != null) {
-            BookmarkData data = new BookmarkData(bookmarkTree);
-            addOffDocumentItem(data);
-            if (!data.isResolved()) {
-                // bookmarks did not fully resolve, add anyway. (hacky? yeah)
-                model.handleOffDocumentItem(data);
-            }
-        }
-
         model.endDocument();
 
         if (statistics != null) {
