@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PageViewport.java 535866 2007-05-07 12:22:22Z jeremias $ */
+/* $Id: PageViewport.java 581906 2007-10-04 13:51:25Z cbowditch $ */
 
 package org.apache.fop.area;
 
@@ -304,7 +304,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
     }
 
     /**
-     * @see org.apache.fop.area.Resolvable#resolveIDRef(String, List)
+     * {@inheritDoc} 
      */
     public void resolveIDRef(String id, List pages) {
         if (page == null) {
@@ -565,7 +565,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
     }
 
     /**
-     * @see java.lang.Object#toString()
+     * {@inheritDoc}
      */
     public String toString() {
         StringBuffer sb = new StringBuffer(64);
@@ -666,4 +666,9 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
     public RegionReference getRegionReference(int id) {
         return getPage().getRegionViewport(id).getRegionReference();
     }
+    
+    /** @return whether this page viewport has any extension attachments */
+    public boolean hasExtensionAttachments() {
+        return this.extensionAttachments != null && !this.extensionAttachments.isEmpty();
+    }
 }
