@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: LengthBase.java 495371 2007-01-11 21:03:07Z adelmelle $ */
+/* $Id: LengthBase.java 619670 2008-02-07 22:32:27Z adelmelle $ */
 
 package org.apache.fop.datatypes;
 
@@ -25,10 +25,6 @@ import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.FObj;
 import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.expr.PropertyException;
-import org.apache.fop.fo.flow.Character;
-import org.apache.fop.fo.flow.ExternalGraphic;
-import org.apache.fop.fo.flow.InstreamForeignObject;
-import org.apache.fop.layoutmgr.inline.LeafNodeLayoutManager;
 
 /**
  * Models a length which can be used as a factor in a percentage length
@@ -83,7 +79,6 @@ public class LengthBase implements PercentBase {
     
     /**
      * Constructor
-     * @param parentFO parent FO for this
      * @param plist property list for this
      * @param baseType a constant defining the type of teh percent base
      * @throws PropertyException In case an problem occurs while evaluating values
@@ -120,7 +115,7 @@ public class LengthBase implements PercentBase {
     }
 
     /**
-     * @see org.apache.fop.datatypes.PercentBase#getBaseLength(PercentBaseContext)
+     * {@inheritDoc}
      */
     public int getBaseLength(PercentBaseContext context) throws PropertyException {
         int baseLen = 0;
@@ -135,5 +130,17 @@ public class LengthBase implements PercentBase {
         return baseLen;
     }
 
+    /** {@inheritDoc} */
+    public String toString() {
+        return super.toString()
+          + "[fo=" + fobj + "," 
+          + "baseType=" + baseType + ","
+          + "baseLength=" + baseLength + "]";
+    }
+    
+    /**@return the base length as a {@link Length} */
+    public Length getBaseLength() {
+        return baseLength;
+    }
 }
 
