@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FObj.java 442282 2006-09-11 18:24:35Z jeremias $ */
+/* $Id: FObj.java 668641 2008-06-17 12:59:25Z jeremias $ */
 
 package org.apache.fop.fo;
 
@@ -24,15 +24,17 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.ListIterator;
 import java.util.Map;
+import java.util.NoSuchElementException;
 import java.util.Set;
 
+import org.xml.sax.Attributes;
+import org.xml.sax.Locator;
+
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.fo.extensions.ExtensionAttachment;
 import org.apache.fop.fo.flow.Marker;
 import org.apache.fop.fo.properties.PropertyMaker;
 import org.apache.fop.util.QName;
-import org.xml.sax.Attributes;
-import org.xml.sax.Locator;
 
 /**
  * Base class for representation of formatting objects and their processing.
@@ -43,9 +45,11 @@ public abstract class FObj extends FONode implements Constants {
     private static PropertyMaker[] propertyListTable
                             = FOPropertyMapping.getGenericMappings();
     
-    /** The immediate child nodes of this node. */
-    protected List childNodes = null;
-
+    /** 
+     * pointer to the descendant subtree
+     */
+    protected FONode firstChild;
+    
     /** The list of extension attachments, null if none */
     private List extensionAttachments = null;
     
@@ -60,6 +64,10 @@ public abstract class FObj extends FONode implements Constants {
     /** Markers added to this element. */
     private Map markers = null;
     
+    // The value of properties relevant for all fo objects
+    private String id = null;
+    // End of property values
+
     /**
      * Create a new formatting object.
      * All formatting object classes extend this class.
@@ -84,13 +92,13 @@ public abstract class FObj extends FONode implements Constants {
     }
 
     /**
-     * @see org.apache.fop.fo.FONode#clone(FONode, boolean)
+     * {@inheritDoc} 
      */
     public FONode clone(FONode parent, boolean removeChildren)
         throws FOPException {
         FObj fobj = (FObj) super.clone(parent, removeChildren);
         if (removeChildren) {
-            fobj.childNodes = null;
+            fobj.firstChild = null;
         }
         return fobj;
     }
@@ -105,7 +113,7 @@ public abstract class FObj extends FONode implements Constants {
     }
 
     /**
-     * @see org.apache.fop.fo.FONode#processNode
+     * {@inheritDoc}
      */
     public void processNode(String elementName, Locator locator, 
                             Attributes attlist, PropertyList pList) 
@@ -121,7 +129,7 @@ public abstract class FObj extends FONode implements Constants {
 
     /**
      * Create a default property list for this element. 
-     * @see org.apache.fop.fo.FONode
+     * {@inheritDoc}
      */
     protected PropertyList createPropertyList(PropertyList parent, 
                     FOEventHandler foEventHandler) throws FOPException {
@@ -136,6 +144,17 @@ public abstract class FObj extends FONode implements Constants {
      * @throws FOPException if there is a problem binding the values
      */
     public void bind(PropertyList pList) throws FOPException {
+        id = pList.get(PR_ID).getString();
+    }
+
+    /**
+     * {@inheritDoc}
+     * @throws FOPException FOP Exception
+     */
+    protected void startOfNode() throws FOPException {
+        if (id != null) {
+            checkId(id);
+        }
     }
 
     /**
@@ -147,7 +166,7 @@ public abstract class FObj extends FONode implements Constants {
      * @throws ValidationException if the ID is already defined elsewhere
      *                              (strict validation only)
      */
-    protected void checkId(String id) throws ValidationException {
+    private void checkId(String id) throws ValidationException {
         if (!inMarker() && !id.equals("")) {
             Set idrefs = getFOEventHandler().getIDReferences();
             if (!idrefs.contains(id)) {
@@ -161,7 +180,7 @@ public abstract class FObj extends FONode implements Constants {
                     if (log.isWarnEnabled()) {
                         StringBuffer msg = new StringBuffer();
                         msg.append("Found non-unique id on ").append(getName());
-                        if (locator.getLineNumber() != -1) {
+                        if (locator != null && locator.getLineNumber() != -1) {
                             msg.append(" (at ").append(locator.getLineNumber())
                                 .append("/").append(locator.getColumnNumber())
                                 .append(")");
@@ -185,12 +204,12 @@ public abstract class FObj extends FONode implements Constants {
     }
 
     /**
-     * @see org.apache.fop.fo.FONode#addChildNode(FONode)
+     * {@inheritDoc}
      */
     protected void addChildNode(FONode child) throws FOPException {
         if (canHaveMarkers() && child.getNameId() == FO_MARKER) {
             addMarker((Marker) child);
-        } else {
+        } else { 
             ExtensionAttachment attachment = child.getExtensionAttachment();
             if (attachment != null) {
                 /* This removes the element from the normal children, 
@@ -199,10 +218,16 @@ public abstract class FObj extends FONode implements Constants {
                  */
                 addExtensionAttachment(attachment);
             } else {
-                if (childNodes == null) {
-                    childNodes = new java.util.ArrayList();
+                if (firstChild == null) {
+                    firstChild = child;
+                } else {
+                    FONode prevChild = firstChild;
+                    while (prevChild.siblings != null
+                            && prevChild.siblings[1] != null) {
+                        prevChild = prevChild.siblings[1];
+                    }
+                    FONode.attachSiblings(prevChild, child);
                 }
-                childNodes.add(child);
             }
         }
     }
@@ -218,10 +243,23 @@ public abstract class FObj extends FONode implements Constants {
         parent.addChildNode(child);
     }
     
-    /** @see org.apache.fop.fo.FONode#removeChild(org.apache.fop.fo.FONode) */
+    /** {@inheritDoc} */
     public void removeChild(FONode child) {
-        if (childNodes != null) {
-            childNodes.remove(child);
+        FONode nextChild = null;
+        if (child.siblings != null) {
+            nextChild = child.siblings[1];
+        }
+        if (child == firstChild) {
+            firstChild = nextChild;
+            if (firstChild != null) {
+                firstChild.siblings[0] = null;
+            }
+        } else {
+            FONode prevChild = child.siblings[0];
+            prevChild.siblings[1] = nextChild;
+            if (nextChild != null) {
+                nextChild.siblings[0] = prevChild;
+            }
         }
     }
     
@@ -247,41 +285,38 @@ public abstract class FObj extends FONode implements Constants {
     }
 
     /**
-     * @see org.apache.fop.fo.FONode#getChildNodes()
+     * {@inheritDoc}
      */
-    public ListIterator getChildNodes() {
-        if (childNodes != null) {
-            return childNodes.listIterator();
+    public FONodeIterator getChildNodes() {
+        if (firstChild != null) {
+            return new FObjIterator(this);
         }
         return null;
     }
 
     /**
      * Return an iterator over the object's childNodes starting
-     * at the passed-in node.
+     * at the passed-in node (= first call to iterator.next() will
+     * return childNode)
      * @param childNode First node in the iterator
      * @return A ListIterator or null if childNode isn't a child of
      * this FObj.
      */
-    public ListIterator getChildNodes(FONode childNode) {
-        if (childNodes != null) {
-            int i = childNodes.indexOf(childNode);
-            if (i >= 0) {
-                return childNodes.listIterator(i);
-            }
-        }
-        return null;
-    }
-
-    /**
-     * Return a FONode based on the index in the list of childNodes.
-     * @param nodeIndex index of the node to return
-     * @return the node or null if the index is invalid
-     */
-    public FONode getChildNodeAt(int nodeIndex) {
-        if (childNodes != null) {
-            if (nodeIndex >= 0 && nodeIndex < childNodes.size()) {
-                return (FONode) childNodes.get(nodeIndex);
+    public FONodeIterator getChildNodes(FONode childNode) {
+        FONodeIterator it = getChildNodes();
+        if (it != null) {
+            if (firstChild == childNode) {
+                return it;
+            } else {
+                while (it.hasNext()
+                        && it.nextNode().siblings[1] != childNode) {
+                    //nop
+                }
+                if (it.hasNext()) {
+                    return it;
+                } else {
+                    return null;
+                }
             }
         }
         return null;
@@ -307,14 +342,14 @@ public abstract class FObj extends FONode implements Constants {
      */
     protected void addMarker(Marker marker) {
         String mcname = marker.getMarkerClassName();
-        if (childNodes != null) {
+        if (firstChild != null) {
             // check for empty childNodes
-            for (Iterator iter = childNodes.iterator(); iter.hasNext();) {
+            for (Iterator iter = getChildNodes(); iter.hasNext();) {
                 FONode node = (FONode) iter.next();
                 if (node instanceof FObj
                         || (node instanceof FOText
                                 && ((FOText) node).willCreateArea())) {
-                    getLogger().error(
+                    log.error(
                             "fo:marker must be an initial child: " + mcname);
                     return;
                 } else if (node instanceof FOText) {
@@ -329,7 +364,7 @@ public abstract class FObj extends FONode implements Constants {
         if (!markers.containsKey(mcname)) {
             markers.put(mcname, marker);
         } else {
-            getLogger().error("fo:marker 'marker-class-name' "
+            log.error("fo:marker 'marker-class-name' "
                     + "must be unique for same parent: " + mcname);
         }
     }
@@ -348,7 +383,7 @@ public abstract class FObj extends FONode implements Constants {
         return markers;
     }
 
-    /** @see org.apache.fop.fo.FONode#gatherContextInfo() */
+    /** {@inheritDoc} */
     protected String gatherContextInfo() {
         if (getLocator() != null) {
             return super.gatherContextInfo();
@@ -469,13 +504,22 @@ public abstract class FObj extends FONode implements Constants {
         return -1;
     }
     
+    /** @return the "id" property. */
+    public String getId() {
+        return id;
+    }
     
-    /** @see org.apache.fop.fo.FONode#getNamespaceURI() */
+    /** @return whether this object has an id set */
+    public boolean hasId() {
+        return id != null && id.length() > 0;
+    }
+
+    /** {@inheritDoc} */
     public String getNamespaceURI() {
         return FOElementMapping.URI;
     }
 
-    /** @see org.apache.fop.fo.FONode#getNormalNamespacePrefix() */
+    /** {@inheritDoc} */
     public String getNormalNamespacePrefix() {
         return "fo";
     }
@@ -495,7 +539,7 @@ public abstract class FObj extends FONode implements Constants {
             extensionAttachments = new java.util.ArrayList();
         }
         if (log.isDebugEnabled()) {
-            getLogger().debug("ExtensionAttachment of category " 
+            log.debug("ExtensionAttachment of category " 
                     + attachment.getCategory() + " added to " 
                     + getName() + ": " + attachment);
         }
@@ -537,4 +581,206 @@ public abstract class FObj extends FONode implements Constants {
             return foreignAttributes;
         }
     }
+    
+    /** {@inheritDoc} */
+    public String toString() {
+        return (super.toString() + "[@id=" + this.id + "]");
+    }
+
+
+    public class FObjIterator implements FONodeIterator {
+        
+        private static final int F_NONE_ALLOWED = 0;
+        private static final int F_SET_ALLOWED = 1;
+        private static final int F_REMOVE_ALLOWED = 2;
+        
+        private FONode currentNode;
+        private FObj parentNode;
+        private int currentIndex;
+        private int flags = F_NONE_ALLOWED;
+        
+        protected FObjIterator(FObj parent) {
+            this.parentNode = parent;
+            this.currentNode = parent.firstChild;
+            this.currentIndex = 0;
+            this.flags = F_NONE_ALLOWED;
+        }
+        
+        /**
+         * {@inheritDoc}
+         */
+        public FObj parentNode() {
+            return parentNode;
+        }
+        
+        /**
+         * {@inheritDoc}
+         */
+        public Object next() {
+            if (currentNode != null) {
+                if (currentIndex != 0) {
+                    if (currentNode.siblings != null
+                        && currentNode.siblings[1] != null) {
+                        currentNode = currentNode.siblings[1];
+                    } else {
+                        throw new NoSuchElementException();
+                    }
+                }
+                currentIndex++;
+                flags |= (F_SET_ALLOWED | F_REMOVE_ALLOWED);
+                return currentNode;
+            } else {
+                throw new NoSuchElementException();
+            }
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public Object previous() {
+            if (currentNode.siblings != null
+                    && currentNode.siblings[0] != null) {
+                currentIndex--;
+                currentNode = currentNode.siblings[0];
+                flags |= (F_SET_ALLOWED | F_REMOVE_ALLOWED);
+                return currentNode;
+            } else {
+                throw new NoSuchElementException();
+            }
+        }
+        
+        /**
+         * {@inheritDoc}
+         */
+        public void set(Object o) {
+            if ((flags & F_SET_ALLOWED) == F_SET_ALLOWED) {
+                FONode newNode = (FONode) o;
+                if (currentNode == parentNode.firstChild) {
+                    parentNode.firstChild = newNode;
+                } else {
+                    FONode.attachSiblings(currentNode.siblings[0], newNode);
+                }
+                if (currentNode.siblings != null
+                        && currentNode.siblings[1] != null) {
+                    FONode.attachSiblings(newNode, currentNode.siblings[1]);
+                }
+            } else {
+                throw new IllegalStateException();
+            }
+        }
+        
+        /**
+         * {@inheritDoc}
+         */
+        public void add(Object o) {
+            FONode newNode = (FONode) o;
+            if (currentIndex == -1) {
+                if (currentNode != null) {
+                    FONode.attachSiblings(newNode, currentNode);
+                }
+                parentNode.firstChild = newNode;
+                currentIndex = 0;
+                currentNode = newNode;
+            } else {
+                if (currentNode.siblings != null
+                        && currentNode.siblings[1] != null) {
+                    FONode.attachSiblings((FONode) o, currentNode.siblings[1]);
+                }
+                FONode.attachSiblings(currentNode, (FONode) o);
+            }
+            flags &= F_NONE_ALLOWED;
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public boolean hasNext() {
+            return (currentNode != null)
+                && ((currentIndex == 0)
+                        || (currentNode.siblings != null
+                            && currentNode.siblings[1] != null));
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public boolean hasPrevious() {
+            return (currentIndex != 0)
+                || (currentNode.siblings != null
+                    && currentNode.siblings[0] != null);
+        }
+        
+        /**
+         * {@inheritDoc}
+         */
+        public int nextIndex() {
+            return currentIndex + 1;
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public int previousIndex() {
+            return currentIndex - 1;
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public void remove() {
+            if ((flags & F_REMOVE_ALLOWED) == F_REMOVE_ALLOWED) {
+                parentNode.removeChild(currentNode);
+                if (currentIndex == 0) {
+                    //first node removed
+                    currentNode = parentNode.firstChild;
+                } else if (currentNode.siblings != null
+                        && currentNode.siblings[0] != null) {
+                    currentNode = currentNode.siblings[0];
+                    currentIndex--;
+                } else {
+                    currentNode = null;
+                }
+                flags &= F_NONE_ALLOWED;
+            } else {
+                throw new IllegalStateException();
+            }
+        }
+
+        /**
+         * {@inheritDoc}
+         */
+        public FONode lastNode() {
+            while (currentNode != null
+                    && currentNode.siblings != null
+                    && currentNode.siblings[1] != null) {
+                currentNode = currentNode.siblings[1];
+                currentIndex++;
+            }
+            return currentNode;
+        }
+        
+        /**
+         * {@inheritDoc}
+         */
+        public FONode firstNode() {
+            currentNode = parentNode.firstChild;
+            currentIndex = 0;
+            return currentNode;
+        }
+        
+        /**
+         * {@inheritDoc}
+         */
+        public FONode nextNode() {
+            return (FONode) next();
+        }
+        
+        /**
+         * {@inheritDoc}
+         */
+        public FONode previousNode() {
+            return (FONode) previous();
+        }
+    }
+
 }
