@@ -15,11 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: PropertyList.java 474225 2006-11-13 10:08:19Z jeremias $ */
+/* $Id: PropertyList.java 617976 2008-02-03 12:05:49Z adelmelle $ */
 
 package org.apache.fop.fo;
 
 // Java
+import java.text.MessageFormat;
+
 import org.xml.sax.Attributes;
 
 import org.apache.commons.logging.Log;
@@ -183,21 +185,24 @@ public abstract class PropertyList {
      * @param propId The ID of the property whose value is desired.
      * @return The computed value if the property is explicitly set on some
      * ancestor of the current FO, else the initial value.
-     * @throws PropertyException ...
+     * @throws PropertyException if there an error occurred when getting the property
      */
     public Property getNearestSpecified(int propId) throws PropertyException {
         Property p = null;
-
-        for (PropertyList plist = this; p == null && plist != null;
-                plist = plist.parentPropertyList) {
-            p = plist.getExplicit(propId);
+        PropertyList pList = parentPropertyList;
+        
+        while (pList != null) {
+            p = pList.getExplicit(propId);
+            if (p != null) {
+                return p;
+            } else {
+                pList = pList.parentPropertyList;
+            }
         }
-
-        if (p == null) {
-            // If no explicit setting found, return initial (default) value.
-            p = makeProperty(propId);
-        }
-        return p;
+        
+        // If no explicit value found on any of the ancestor-nodes, 
+        // return initial (default) value.
+        return makeProperty(propId);
     }
 
     /**
@@ -305,11 +310,13 @@ public abstract class PropertyList {
         String attributeNS;
         FopFactory factory = getFObj().getUserAgent().getFactory(); 
         for (int i = 0; i < attributes.getLength(); i++) {
-            /* convert all attributes with the same namespace as the fo element for this fObj */
+            /* convert all attributes with the same namespace as the fo element
+             * the "xml:lang" property is a special case */
             attributeNS = attributes.getURI(i);
             attributeName = attributes.getQName(i);
             attributeValue = attributes.getValue(i);
-            if (attributeNS == null || attributeNS.length() == 0) {
+            if (attributeNS == null || attributeNS.length() == 0 
+                    || "xml:lang".equals(attributeName)) {
                 convertAttributeToProperty(attributes, attributeName, attributeValue);
             } else if (!factory.isNamespaceIgnored(attributeNS)) {
                 ElementMapping mapping = factory.getElementMappingRegistry().getElementMapping(
@@ -352,9 +359,9 @@ public abstract class PropertyList {
         if (propId == -1 
                 || (subpropId == -1 
                         && findSubPropertyName(propertyName) != null)) {
-            StringBuffer errorMessage = new StringBuffer().append(
-                        "Invalid property name \'").append(propertyName);
-            handleInvalidProperty(errorMessage.toString(), propertyName);
+            String errorMessage = MessageFormat.format(
+                    "Invalid property name ''{0}''.", new Object[] {propertyName});
+            handleInvalidProperty(errorMessage, propertyName);
             return false;
         }
         return true;
@@ -574,7 +581,7 @@ public abstract class PropertyList {
      */
     public CommonBorderPaddingBackground getBorderPaddingBackgroundProps() 
                 throws PropertyException {
-        return new CommonBorderPaddingBackground(this, getFObj());
+        return new CommonBorderPaddingBackground(this);
     }
     
     /**
@@ -583,7 +590,7 @@ public abstract class PropertyList {
      * @throws PropertyException if there's a problem while processing the properties
      */
     public CommonHyphenation getHyphenationProps() throws PropertyException {
-        return new CommonHyphenation(this);
+        return CommonHyphenation.getInstance(this);
     }
     
     /**
@@ -643,12 +650,13 @@ public abstract class PropertyList {
     
 
     /**
-     * Constructs a CommonFont object. 
+     * Constructs a CommonFont object.
+     *  
      * @return A CommonFont object
      * @throws PropertyException if there's a problem while processing the properties
      */
     public CommonFont getFontProps() throws PropertyException {
-        return new CommonFont(this);
+        return CommonFont.getInstance(this);
     }
     
     /**
