@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RecursiveCharIterator.java 567294 2007-08-18 17:00:50Z vhennebert $ */
+/* $Id: RecursiveCharIterator.java 557337 2007-07-18 17:37:14Z adelmelle $ */
 
 package org.apache.fop.fo;
 
@@ -113,7 +113,7 @@ public class RecursiveCharIterator extends CharIterator {
      */
     public boolean hasNext() {
         while (curCharIter != null) {
-            if (!curCharIter.hasNext()) {
+            if (curCharIter.hasNext() == false) {
                 getNextCharIter();
             } else {
                 return true;
@@ -134,7 +134,7 @@ public class RecursiveCharIterator extends CharIterator {
     }
 
     /**
-     * @see java.util.Iterator#remove
+     * {@inheritDoc}
      */
     public void remove() {
         if (curCharIter != null) {
