@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ExtensionElementMapping.java 525526 2007-04-04 15:05:30Z jeremias $ */
+/* $Id: ExtensionElementMapping.java 627367 2008-02-13 12:03:30Z maxberger $ */
 
 package org.apache.fop.fo.extensions;
 
@@ -62,6 +62,7 @@ public class ExtensionElementMapping extends ElementMapping {
             foObjs.put("outline", new UnknownXMLObj.Maker(URI));
             foObjs.put("label", new UnknownXMLObj.Maker(URI));
             foObjs.put("destination", new DestinationMaker());
+            foObjs.put("external-document", new ExternalDocumentMaker());
         }
     }
     
@@ -71,12 +72,18 @@ public class ExtensionElementMapping extends ElementMapping {
         }
     }
 
-    /** @see org.apache.fop.fo.ElementMapping#getStandardPrefix() */
+    static class ExternalDocumentMaker extends ElementMapping.Maker {
+        public FONode make(FONode parent) {
+            return new ExternalDocument(parent);
+        }
+    }
+
+    /** {@inheritDoc} */
     public String getStandardPrefix() {
         return "fox";
     }
     
-    /** @see org.apache.fop.fo.ElementMapping#isAttributeProperty(org.apache.fop.util.QName) */
+    /** {@inheritDoc} */
     public boolean isAttributeProperty(QName attributeName) {
         if (!URI.equals(attributeName.getNamespaceURI())) {
             throw new IllegalArgumentException("The namespace URIs don't match");
