@@ -15,11 +15,12 @@
  * limitations under the License.
  */
 
-/* $Id: Character.java 567294 2007-08-18 17:00:50Z vhennebert $ */
+/* $Id: Character.java 557337 2007-07-18 17:37:14Z adelmelle $ */
 
 package org.apache.fop.fo.flow;
 
 import java.awt.Color;
+import java.util.NoSuchElementException;
 
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.datatypes.Length;
@@ -37,16 +38,17 @@ import org.apache.fop.fo.properties.SpaceProperty;
 import org.xml.sax.Locator;
 
 /**
- * This class represents the flow object 'fo:character'. Its use is defined by
- * the spec: "The fo:character flow object represents a character that is mapped to
- * a glyph for presentation. It is an atomic unit to the formatter.
- * When the result tree is interpreted as a tree of formatting objects,
- * a character in the result tree is treated as if it were an empty
- * element of type fo:character with a character attribute
- * equal to the Unicode representation of the character.
- * The semantics of an "auto" value for character properties, which is
- * typically their initial value,  are based on the Unicode codepoint.
- * Overrides may be specified in an implementation-specific manner." (6.6.3)
+ * Class modelling the fo:character object. 
+ * Its use is defined by the spec: 
+ * "The fo:character flow object represents a character that is mapped to
+ *  a glyph for presentation. It is an atomic unit to the formatter.
+ *  When the result tree is interpreted as a tree of formatting objects,
+ *  a character in the result tree is treated as if it were an empty
+ *  element of type fo:character with a character attribute
+ *  equal to the Unicode representation of the character.
+ *  The semantics of an "auto" value for character properties, which is
+ *  typically their initial value,  are based on the Unicode codepoint.
+ *  Overrides may be specified in an implementation-specific manner." (6.6.3)
  *
  */
 public class Character extends FObj {
@@ -57,8 +59,7 @@ public class Character extends FObj {
     private Length alignmentAdjust;
     private int alignmentBaseline;
     private Length baselineShift;
-    /** Holds the character value */
-    protected char character;
+    private char character;
     private Color color;
     private int dominantBaseline;
     // private ToBeImplementedProperty glyphOrientationHorizontal;
@@ -130,6 +131,7 @@ public class Character extends FObj {
 
     /**
      * {@inheritDoc}
+     * XSL Content Model: empty
      */
     protected void validateChildNode(Locator loc, String nsURI, String localName) 
         throws ValidationException {
@@ -232,9 +234,7 @@ public class Character extends FObj {
         return wordSpacing; 
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public String getLocalName() {
         return "character";
     }
@@ -245,5 +245,37 @@ public class Character extends FObj {
     public int getNameId() {
         return FO_CHARACTER;
     }
+    
+    private class FOCharIterator extends CharIterator {
+
+        private boolean bFirst = true;
+        private Character foChar;
+        
+        FOCharIterator(Character foChar) {
+            this.foChar = foChar;
+        }
+        
+        public boolean hasNext() {
+            return bFirst;
+        }
+
+        public char nextChar() {
+            if (bFirst) {
+                bFirst = false;
+                return foChar.character;
+            } else {
+                throw new NoSuchElementException();
+            }
+        }
+
+        public void remove() {
+            foChar.parent.removeChild(foChar);
+        }
+
+        public void replaceChar(char c) {
+            foChar.character = c;
+        }
+
+    }
 
 }
