@@ -15,11 +15,10 @@
  * limitations under the License.
  */
 
-/* $Id: Wrapper.java 554094 2007-07-07 00:04:25Z adelmelle $ */
+/* $Id: Wrapper.java 604814 2007-12-17 10:21:04Z vhennebert $ */
 
 package org.apache.fop.fo.flow;
 
-// Java
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.FObjMixed;
@@ -28,9 +27,9 @@ import org.apache.fop.fo.ValidationException;
 import org.xml.sax.Locator;
 
 /**
- * Implementation for fo:wrapper formatting object.
- * The wrapper object serves as
- * a property holder for its child node objects.
+ * Class modelling the fo:wrapper object.
+ * The wrapper object serves as a property holder for 
+ * its child node objects.
  */
 public class Wrapper extends FObjMixed {
     // The value of properties relevant for fo:wrapper.
@@ -38,43 +37,68 @@ public class Wrapper extends FObjMixed {
     
     // used for FO validation
     private boolean blockOrInlineItemFound = false;
+    private boolean inlineChildrenAllowed = false;
 
     /**
      * @param parent FONode that is the parent of this object
      */
     public Wrapper(FONode parent) {
         super(parent);
+        /* Check if the fo:wrapper is a child of a FO that allows mixed content
+         * (or a descendant in nested fo:wrapper sequence, the first of which
+         *  is a child of a FO that allows mixed content) */
+        FONode ancestor = this.parent;
+        while (ancestor instanceof Wrapper) {
+            ancestor = ancestor.getParent();
+        }
+        if (ancestor instanceof FObjMixed ) {
+            inlineChildrenAllowed = true;
+        }
     }
 
     /**
-     * @see org.apache.fop.fo.FONode#validateChildNode(Locator, String, String)
-     * XSL Content Model: marker* (#PCDATA|%inline;|%block;)*
-     * Additionally (unimplemented): "An fo:wrapper that is a child of an 
+     * {@inheritDoc}
+     * <br>XSL Content Model: marker* (#PCDATA|%inline;|%block;)*
+     * <br><i>Additionally (unimplemented): "An fo:wrapper that is a child of an 
      * fo:multi-properties is only permitted to have children that would 
-     * be permitted in place of the fo:multi-properties."
+     * be permitted in place of the fo:multi-properties."</i>
+     * 
      */
     protected void validateChildNode(Locator loc, String nsURI, String localName) 
         throws ValidationException {
-        if (FO_URI.equals(nsURI) && localName.equals("marker")) {
+        if (FO_URI.equals(nsURI) && "marker".equals(localName)) {
             if (blockOrInlineItemFound) {
                nodesOutOfOrderError(loc, "fo:marker", 
                     "(#PCDATA|%inline;|%block;)");
             }
         } else if (isBlockOrInlineItem(nsURI, localName)) {
+            //delegate validation to parent
+            FONode.validateChildNode(this.parent, loc, nsURI, localName);
             blockOrInlineItemFound = true;
         } else {
             invalidChildError(loc, nsURI, localName);
         }
     }
 
-    /** @see org.apache.fop.fo.FONode#getLocalName() */
+    /** {@inheritDoc} */
+    protected void addCharacters(
+                char[] data, 
+                int start, 
+                int end, 
+                PropertyList pList, 
+                Locator locator) throws FOPException {
+        /* Only add text if the fo:wrapper's parent allows inline children */
+        if (this.inlineChildrenAllowed) {
+            super.addCharacters(data, start, end, pList, locator);
+        }
+    }
+
+    /** {@inheritDoc} */
     public String getLocalName() {
         return "wrapper";
     }
 
-    /**
-     * @see org.apache.fop.fo.FObj#getNameId()
-     */
+    /** {@inheritDoc} */
     public int getNameId() {
         return FO_WRAPPER;
     }
