@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RepeatablePageMasterAlternatives.java 554104 2007-07-07 01:07:10Z adelmelle $ */
+/* $Id: RepeatablePageMasterAlternatives.java 607032 2007-12-27 10:34:15Z jeremias $ */
 
 package org.apache.fop.fo.pagination;
 
@@ -49,6 +49,7 @@ public class RepeatablePageMasterAlternatives extends FObj
 
     private List conditionalPageMasterRefs;
     private boolean hasPagePositionLast = false;
+    private boolean hasPagePositionOnly = false;
 
     /**
      * @see org.apache.fop.fo.FONode#FONode(FONode)
@@ -58,14 +59,14 @@ public class RepeatablePageMasterAlternatives extends FObj
     }
 
     /**
-     * @see org.apache.fop.fo.FObj#bind(PropertyList)
+     * {@inheritDoc}
      */
     public void bind(PropertyList pList) throws FOPException {
         maximumRepeats = pList.get(PR_MAXIMUM_REPEATS);
     }
 
     /**
-     * @see org.apache.fop.fo.FONode#startOfNode
+     * {@inheritDoc}
      */
     protected void startOfNode() throws FOPException {
         conditionalPageMasterRefs = new java.util.ArrayList();
@@ -81,7 +82,7 @@ public class RepeatablePageMasterAlternatives extends FObj
     }
 
     /**
-     * @see org.apache.fop.fo.FONode#endOfNode
+     * {@inheritDoc}
      */
     protected void endOfNode() throws FOPException {
         if (firstChild == null) {
@@ -90,7 +91,7 @@ public class RepeatablePageMasterAlternatives extends FObj
     }
 
     /**
-     * @see org.apache.fop.fo.FONode#validateChildNode(Locator, String, String)
+     * {@inheritDoc}
         XSL/FOP: (conditional-page-master-reference+)
      */
     protected void validateChildNode(Locator loc, String nsURI, String localName) 
@@ -124,6 +125,7 @@ public class RepeatablePageMasterAlternatives extends FObj
     public String getNextPageMasterName(boolean isOddPage,
                                         boolean isFirstPage,
                                         boolean isLastPage,
+                                        boolean isOnlyPage,
                                         boolean isBlankPage) {
         if (getMaximumRepeats() != INFINITE) {
             if (numberConsumed < getMaximumRepeats()) {
@@ -138,7 +140,7 @@ public class RepeatablePageMasterAlternatives extends FObj
         for (int i = 0; i < conditionalPageMasterRefs.size(); i++) {
             ConditionalPageMasterReference cpmr
                 = (ConditionalPageMasterReference)conditionalPageMasterRefs.get(i);
-            if (cpmr.isValid(isOddPage, isFirstPage, isLastPage, isBlankPage)) {
+            if (cpmr.isValid(isOddPage, isFirstPage, isLastPage, isOnlyPage, isBlankPage)) {
                 return cpmr.getMasterReference();
             }
         }
@@ -155,14 +157,17 @@ public class RepeatablePageMasterAlternatives extends FObj
         if (cpmr.getPagePosition() == EN_LAST) {
             this.hasPagePositionLast = true;
         }
+        if (cpmr.getPagePosition() == EN_ONLY) {
+            this.hasPagePositionOnly = true;
+        }
     }
 
-    /** @see org.apache.fop.fo.pagination.SubSequenceSpecifier#reset() */
+    /** {@inheritDoc} */
     public void reset() {
         this.numberConsumed = 0;
     }
 
-    /** @see org.apache.fop.fo.pagination.SubSequenceSpecifier#goToPrevious() */
+    /** {@inheritDoc} */
     public boolean goToPrevious() {
         if (numberConsumed == 0) {
             return false;
@@ -172,17 +177,23 @@ public class RepeatablePageMasterAlternatives extends FObj
         }
     }
     
-    /** @see org.apache.fop.fo.pagination.SubSequenceSpecifier#hasPagePositionLast() */
+    /** {@inheritDoc} */
     public boolean hasPagePositionLast() {
         return this.hasPagePositionLast;
     }
     
+    /** {@inheritDoc} */
+    /** @see org.apache.fop.fo.pagination.SubSequenceSpecifier#hasPagePositionOnly() */
+    public boolean hasPagePositionOnly() {
+        return this.hasPagePositionOnly;
+    }
+    
     /** @see org.apache.fop.fo.FONode#getLocalName() */
     public String getLocalName() {
         return "repeatable-page-master-alternatives";
     }
 
-    /** @see org.apache.fop.fo.FObj#getNameId() */
+    /** {@inheritDoc} */
     public int getNameId() {
         return FO_REPEATABLE_PAGE_MASTER_ALTERNATIVES;
     }
