@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RepeatablePageMasterReference.java 495371 2007-01-11 21:03:07Z adelmelle $ */
+/* $Id: RepeatablePageMasterReference.java 607032 2007-12-27 10:34:15Z jeremias $ */
 
 package org.apache.fop.fo.pagination;
 
@@ -54,7 +54,7 @@ public class RepeatablePageMasterReference extends FObj
     }
 
     /**
-     * @see org.apache.fop.fo.FObj#bind(PropertyList)
+     * {@inheritDoc}
      */
     public void bind(PropertyList pList) throws FOPException {
         masterReference = pList.get(PR_MASTER_REFERENCE).getString();
@@ -66,7 +66,7 @@ public class RepeatablePageMasterReference extends FObj
     }
 
     /**
-     * @see org.apache.fop.fo.FONode#startOfNode
+     * {@inheritDoc}
      */
     protected void startOfNode() throws FOPException {
         PageSequenceMaster pageSequenceMaster = (PageSequenceMaster) parent;
@@ -79,7 +79,7 @@ public class RepeatablePageMasterReference extends FObj
     }
  
     /**
-     * @see org.apache.fop.fo.FONode#validateChildNode(Locator, String, String)
+     * {@inheritDoc}
      * XSL Content Model: empty
      */
     protected void validateChildNode(Locator loc, String nsURI, String localName) 
@@ -88,11 +88,12 @@ public class RepeatablePageMasterReference extends FObj
     }
 
     /**
-     * @see org.apache.fop.fo.pagination.SubSequenceSpecifier
+     * {@inheritDoc}
      */
     public String getNextPageMasterName(boolean isOddPage,
                                         boolean isFirstPage,
                                         boolean isLastPage,
+                                        boolean isOnlyPage,
                                         boolean isEmptyPage) {
         if (getMaximumRepeats() != INFINITE) {
             if (numberConsumed < getMaximumRepeats()) {
@@ -119,13 +120,13 @@ public class RepeatablePageMasterReference extends FObj
         }
     }
 
-    /** @see org.apache.fop.fo.pagination.SubSequenceSpecifier#reset() */
+    /** {@inheritDoc} */
     public void reset() {
         this.numberConsumed = 0;
     }
 
     
-    /** @see org.apache.fop.fo.pagination.SubSequenceSpecifier#goToPrevious() */
+    /** {@inheritDoc} */
     public boolean goToPrevious() {
         if (numberConsumed == 0) {
             return false;
@@ -135,19 +136,25 @@ public class RepeatablePageMasterReference extends FObj
         }
     }
     
-    /** @see org.apache.fop.fo.pagination.SubSequenceSpecifier#hasPagePositionLast() */
+    /** {@inheritDoc} */
     public boolean hasPagePositionLast() {
         return false;
     }
 
-    /** @see org.apache.fop.fo.FONode#getLocalName() */
+    /** {@inheritDoc} */
+    public boolean hasPagePositionOnly() {
+        return false;
+    }
+
+    /** {@inheritDoc} */
     public String getLocalName() {
         return "repeatable-page-master-reference";
     }
 
-    /** @see org.apache.fop.fo.FObj#getNameId() */
+    /** {@inheritDoc} */
     public int getNameId() {
         return FO_REPEATABLE_PAGE_MASTER_REFERENCE;
     }
 
+
 }
