@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Root.java 535883 2007-05-07 14:30:23Z jeremias $ */
+/* $Id: Root.java 611278 2008-01-11 19:50:53Z jeremias $ */
 
 package org.apache.fop.fo.pagination;
 
@@ -25,13 +25,14 @@ import java.util.List;
 import org.xml.sax.Locator;
 
 import org.apache.fop.apps.FOPException;
+import org.apache.fop.fo.ElementMapping;
 import org.apache.fop.fo.FOEventHandler;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.FObj;
 import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.ValidationException;
-import org.apache.fop.fo.pagination.bookmarks.BookmarkTree;
 import org.apache.fop.fo.extensions.destination.Destination;
+import org.apache.fop.fo.pagination.bookmarks.BookmarkTree;
 
 /**
  * The fo:root formatting object. Contains page masters, page-sequences.
@@ -72,25 +73,21 @@ public class Root extends FObj {
         }
     }
 
-    /**
-     * @see org.apache.fop.fo.FObj#bind(PropertyList)
-     */
+    /** {@inheritDoc} */
     public void bind(PropertyList pList) throws FOPException {
         mediaUsage = pList.get(PR_MEDIA_USAGE).getEnum();
     }
 
-    /**
-     * Signal end of this xml element.
-     */
+    /** {@inheritDoc} */
     protected void endOfNode() throws FOPException {
         if (!pageSequenceFound || layoutMasterSet == null) {
-            missingChildElementError("(layout-master-set, declarations?, " + 
-                "bookmark-tree?, page-sequence+)");
+            missingChildElementError("(layout-master-set, declarations?, " 
+                + "bookmark-tree?, (page-sequence|fox:external-document)+)");
         }
     }
 
     /**
-     * @see org.apache.fop.fo.FONode#validateChildNode(Locator, String, String)
+     * {@inheritDoc}
         XSL 1.0 Spec: (layout-master-set,declarations?,page-sequence+)
         FOP: (layout-master-set, declarations?, fox:bookmarks?, page-sequence+)
      */
@@ -129,7 +126,21 @@ public class Root extends FObj {
                 invalidChildError(loc, nsURI, localName);
             }
         } else {
-            invalidChildError(loc, nsURI, localName);
+            if (FOX_URI.equals(nsURI)) {
+                if ("external-document".equals(localName)) {
+                    pageSequenceFound = true;
+                }
+            }
+            //invalidChildError(loc, nsURI, localName);
+            //Ignore non-FO elements under root
+        }
+    }
+    
+
+    /** @inheritDoc */
+    protected void validateChildNode(Locator loc, FONode child) throws ValidationException {
+        if (child instanceof AbstractPageSequence) {
+            pageSequenceFound = true;
         }
     }
 
@@ -281,19 +292,19 @@ public class Root extends FObj {
     }
 
     /**
-     * @see org.apache.fop.fo.FONode#getRoot()
+     * {@inheritDoc}
      */
     public Root getRoot() {
         return this;
     }
 
-    /** @see org.apache.fop.fo.FONode#getLocalName() */
+    /** {@inheritDoc} */
     public String getLocalName() {
         return "root";
     }
 
     /**
-     * @see org.apache.fop.fo.FObj#getNameId()
+     * {@inheritDoc}
      */
     public int getNameId() {
         return FO_ROOT;
