@@ -15,21 +15,23 @@
  * limitations under the License.
  */
 
-/* $Id: CommonBorderPaddingBackground.java 495371 2007-01-11 21:03:07Z adelmelle $ */
+/* $Id: CommonBorderPaddingBackground.java 627367 2008-02-13 12:03:30Z maxberger $ */
 
 package org.apache.fop.fo.properties;
 
 import java.awt.Color;
 
+import org.apache.xmlgraphics.image.loader.ImageInfo;
+import org.apache.xmlgraphics.image.loader.ImageManager;
+import org.apache.xmlgraphics.image.loader.ImageSessionContext;
+
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.datatypes.Length;
 import org.apache.fop.datatypes.PercentBaseContext;
+import org.apache.fop.datatypes.URISpecification;
 import org.apache.fop.fo.Constants;
-import org.apache.fop.fo.FObj;
 import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.expr.PropertyException;
-import org.apache.fop.image.FopImage;
-import org.apache.fop.image.ImageFactory;
 
 /**
  * Stores all common border and padding properties.
@@ -65,20 +67,20 @@ public class CommonBorderPaddingBackground {
      * The "background-position-vertical" property.
      */
     public Length backgroundPositionVertical;
-    
-    
-    private FopImage fopimage;
-    
-    
-    /** the "before" edge */ 
+
+
+    private ImageInfo backgroundImageInfo;
+
+
+    /** the "before" edge */
     public static final int BEFORE = 0;
-    /** the "after" edge */ 
+    /** the "after" edge */
     public static final int AFTER = 1;
-    /** the "start" edge */ 
+    /** the "start" edge */
     public static final int START = 2;
-    /** the "end" edge */ 
+    /** the "end" edge */
     public static final int END = 3;
-    
+
     public static class BorderInfo {
         private int mStyle; // Enum for border style
         private Color mColor; // Border color
@@ -89,15 +91,15 @@ public class CommonBorderPaddingBackground {
             mWidth = width;
             mColor = color;
         }
-        
+
         public int getStyle() {
             return this.mStyle;
         }
-        
+
         public Color getColor() {
             return this.mColor;
         }
-        
+
         public CondLengthProperty getWidth() {
             return this.mWidth;
         }
@@ -110,8 +112,8 @@ public class CommonBorderPaddingBackground {
                 return mWidth.getLengthValue();
             }
         }
-        
-        /** @see java.lang.Object#toString() */
+
+        /** {@inheritDoc} */
         public String toString() {
             StringBuffer sb = new StringBuffer("BorderInfo");
             sb.append(" {");
@@ -125,6 +127,79 @@ public class CommonBorderPaddingBackground {
         }
     }
 
+    /**
+     * A border info with style none. Used as a singleton, in the collapsing-border model,
+     * for elements which don't specify any border on some of their sides.
+     */
+    private static BorderInfo defaultBorderInfo;
+
+    /**
+     * A conditional length of value 0. Returned by the
+     * {@link CommonBorderPaddingBackground#getBorderInfo(int)} method when the
+     * corresponding border isn't specified, to avoid to callers painful checks for null.
+     */
+    private static class ConditionalNullLength extends CondLengthProperty {
+
+        /** {@inheritDoc} */
+        public Property getComponent(int cmpId) {
+            throw new UnsupportedOperationException();
+        }
+
+        /** {@inheritDoc} */
+        public Property getConditionality() {
+            throw new UnsupportedOperationException();
+        }
+
+        /** {@inheritDoc} */
+        public Length getLength() {
+            throw new UnsupportedOperationException();
+        }
+
+        /** {@inheritDoc} */
+        public Property getLengthComponent() {
+            throw new UnsupportedOperationException();
+        }
+
+        /** {@inheritDoc} */
+        public int getLengthValue() {
+            return 0;
+        }
+
+        /** {@inheritDoc} */
+        public int getLengthValue(PercentBaseContext context) {
+            return 0;
+        }
+
+        /** {@inheritDoc} */
+        public boolean isDiscard() {
+            return true;
+        }
+
+        /** {@inheritDoc} */
+        public void setComponent(int cmpId, Property cmpnValue, boolean isDefault) {
+            throw new UnsupportedOperationException();
+        }
+
+        /** {@inheritDoc} */
+        public String toString() {
+            return "CondLength[0mpt, discard]";
+        }
+    }
+
+    /**
+     * Returns a default BorderInfo of style none.
+     * 
+     * @return a BorderInfo instance with style set to {@link Constants#EN_NONE}
+     */
+    public static synchronized BorderInfo getDefaultBorderInfo() {
+        if (defaultBorderInfo == null) {
+            /* It is enough to set color to null, as it should never be consulted */
+            defaultBorderInfo = new BorderInfo(Constants.EN_NONE,
+                    new ConditionalNullLength(), null);
+        }
+        return defaultBorderInfo;
+    }
+
     private BorderInfo[] borderInfo = new BorderInfo[4];
     private CondLengthProperty[] padding = new CondLengthProperty[4];
 
@@ -132,20 +207,19 @@ public class CommonBorderPaddingBackground {
      * Construct a CommonBorderPaddingBackground object.
      */
     public CommonBorderPaddingBackground() {
-        
     }
-    
+
     /**
      * Construct a CommonBorderPaddingBackground object.
+     * 
      * @param pList The PropertyList to get properties from.
-     * @param fobj The FO to create this instance for.
      * @throws PropertyException if there's an error while binding the properties
      */
-    public CommonBorderPaddingBackground(PropertyList pList, FObj fobj) throws PropertyException {
-        
+    public CommonBorderPaddingBackground(PropertyList pList) throws PropertyException {
+
         backgroundAttachment = pList.get(Constants.PR_BACKGROUND_ATTACHMENT).getEnum();
         backgroundColor = pList.get(Constants.PR_BACKGROUND_COLOR).getColor(
-                fobj == null ? null : fobj.getUserAgent());
+                                        pList.getFObj().getUserAgent());
         if (backgroundColor.getAlpha() == 0) {
             backgroundColor = null;
         }
@@ -159,63 +233,59 @@ public class CommonBorderPaddingBackground {
                     Constants.PR_BACKGROUND_POSITION_HORIZONTAL).getLength();
             backgroundPositionVertical = pList.get(
                     Constants.PR_BACKGROUND_POSITION_VERTICAL).getLength();
-            
+
             //Additional processing: preload image
-            String url = ImageFactory.getURL(backgroundImage);
-            FOUserAgent userAgent = fobj.getUserAgent();
-            ImageFactory fact = userAgent.getFactory().getImageFactory();
-            fopimage = fact.getImage(url, userAgent);
-            if (fopimage == null) {
-                Property.log.error("Background image not available: " + backgroundImage);
-            } else {
-                // load dimensions
-                if (!fopimage.load(FopImage.DIMENSIONS)) {
-                    Property.log.error("Cannot read background image dimensions: " 
-                            + backgroundImage);
-                }
+            String uri = URISpecification.getURL(backgroundImage);
+            FOUserAgent userAgent = pList.getFObj().getUserAgent();
+            ImageManager manager = userAgent.getFactory().getImageManager();
+            ImageSessionContext sessionContext = userAgent.getImageSessionContext();
+            ImageInfo info;
+            try {
+                info = manager.getImageInfo(uri, sessionContext);
+                this.backgroundImageInfo = info;
+            } catch (Exception e) {
+                Property.log.error("Background image not available: " + uri);
             }
             //TODO Report to caller so he can decide to throw an exception
         }
 
-        initBorderInfo(pList, BEFORE, 
-                Constants.PR_BORDER_BEFORE_COLOR, 
-                Constants.PR_BORDER_BEFORE_STYLE, 
-                Constants.PR_BORDER_BEFORE_WIDTH, 
+        initBorderInfo(pList, BEFORE,
+                Constants.PR_BORDER_BEFORE_COLOR,
+                Constants.PR_BORDER_BEFORE_STYLE,
+                Constants.PR_BORDER_BEFORE_WIDTH,
                 Constants.PR_PADDING_BEFORE);
-        initBorderInfo(pList, AFTER, 
-                Constants.PR_BORDER_AFTER_COLOR, 
-                Constants.PR_BORDER_AFTER_STYLE, 
-                Constants.PR_BORDER_AFTER_WIDTH, 
+        initBorderInfo(pList, AFTER,
+                Constants.PR_BORDER_AFTER_COLOR,
+                Constants.PR_BORDER_AFTER_STYLE,
+                Constants.PR_BORDER_AFTER_WIDTH,
                 Constants.PR_PADDING_AFTER);
-        initBorderInfo(pList, START, 
-                Constants.PR_BORDER_START_COLOR, 
-                Constants.PR_BORDER_START_STYLE, 
-                Constants.PR_BORDER_START_WIDTH, 
+        initBorderInfo(pList, START,
+                Constants.PR_BORDER_START_COLOR,
+                Constants.PR_BORDER_START_STYLE,
+                Constants.PR_BORDER_START_WIDTH,
                 Constants.PR_PADDING_START);
-        initBorderInfo(pList, END, 
-                Constants.PR_BORDER_END_COLOR, 
-                Constants.PR_BORDER_END_STYLE, 
-                Constants.PR_BORDER_END_WIDTH, 
+        initBorderInfo(pList, END,
+                Constants.PR_BORDER_END_COLOR,
+                Constants.PR_BORDER_END_STYLE,
+                Constants.PR_BORDER_END_WIDTH,
                 Constants.PR_PADDING_END);
 
     }
 
-    private void initBorderInfo(PropertyList pList, int side, 
+    private void initBorderInfo(PropertyList pList, int side,
                     int colorProp, int styleProp, int widthProp, int paddingProp)
                 throws PropertyException {
         padding[side] = pList.get(paddingProp).getCondLength();
         // If style = none, force width to 0, don't get Color (spec 7.7.20)
         int style = pList.get(styleProp).getEnum();
         if (style != Constants.EN_NONE) {
-            FOUserAgent ua = (pList == null) 
-                    ? null 
-                    : (pList.getFObj() == null ? null : pList.getFObj().getUserAgent());
+            FOUserAgent ua = pList.getFObj().getUserAgent();
             setBorderInfo(new BorderInfo(style,
                 pList.get(widthProp).getCondLength(),
                 pList.get(colorProp).getColor(ua)), side);
         }
     }
-    
+
     /**
      * Sets a border.
      * @param info the border information
@@ -224,15 +294,19 @@ public class CommonBorderPaddingBackground {
     public void setBorderInfo(BorderInfo info, int side) {
         this.borderInfo[side] = info;
     }
-    
+
     /**
      * @param side the side to retrieve
      * @return the border info for a side
      */
     public BorderInfo getBorderInfo(int side) {
-        return this.borderInfo[side];
+        if (this.borderInfo[side] == null) {
+            return getDefaultBorderInfo();
+        } else {
+            return this.borderInfo[side];
+        }
     }
-    
+
     /**
      * Set padding.
      * @param source the padding info to copy from
@@ -240,15 +314,15 @@ public class CommonBorderPaddingBackground {
     public void setPadding(CommonBorderPaddingBackground source) {
         this.padding = source.padding;
     }
-    
+
     /**
-     * @return the background image as a preloaded FopImage, null if there is
+     * @return the background image info object, null if there is
      *     no background image.
      */
-    public FopImage getFopImage() {
-        return this.fopimage;
+    public ImageInfo getImageInfo() {
+        return this.backgroundImageInfo;
     }
-    
+
     /**
      * @param bDiscard indicates whether the .conditionality component should be
      * considered (start of a reference-area)
@@ -331,7 +405,7 @@ public class CommonBorderPaddingBackground {
             return padding[side].getLengthValue(context);
         }
     }
-    
+
     /**
      * Returns the CondLengthProperty for the padding on one side.
      * @param side the side
@@ -342,21 +416,21 @@ public class CommonBorderPaddingBackground {
     }
 
     /**
-     * Return all the border and padding width in the inline progression 
+     * Return all the border and padding width in the inline progression
      * dimension.
      * @param bDiscard the discard flag.
      * @param context for percentage evaluation.
      * @return all the padding and border width.
      */
     public int getIPPaddingAndBorder(boolean bDiscard, PercentBaseContext context) {
-        return getPaddingStart(bDiscard, context) 
-            + getPaddingEnd(bDiscard, context) 
-            + getBorderStartWidth(bDiscard) 
-            + getBorderEndWidth(bDiscard);        
+        return getPaddingStart(bDiscard, context)
+            + getPaddingEnd(bDiscard, context)
+            + getBorderStartWidth(bDiscard)
+            + getBorderEndWidth(bDiscard);
     }
-    
+
     /**
-     * Return all the border and padding height in the block progression 
+     * Return all the border and padding height in the block progression
      * dimension.
      * @param bDiscard the discard flag.
      * @param context for percentage evaluation
@@ -364,17 +438,17 @@ public class CommonBorderPaddingBackground {
      */
     public int getBPPaddingAndBorder(boolean bDiscard, PercentBaseContext context) {
         return getPaddingBefore(bDiscard, context) + getPaddingAfter(bDiscard, context)
-               + getBorderBeforeWidth(bDiscard) + getBorderAfterWidth(bDiscard);        
+               + getBorderBeforeWidth(bDiscard) + getBorderAfterWidth(bDiscard);
     }
 
-    /** @see java.lang.Object#toString() */
+    /** {@inheritDoc} */
     public String toString() {
         return "CommonBordersAndPadding (Before, After, Start, End):\n"
             + "Borders: (" + getBorderBeforeWidth(false) + ", " + getBorderAfterWidth(false) + ", "
             + getBorderStartWidth(false) + ", " + getBorderEndWidth(false) + ")\n"
             + "Border Colors: (" + getBorderColor(BEFORE) + ", " + getBorderColor(AFTER) + ", "
             + getBorderColor(START) + ", " + getBorderColor(END) + ")\n"
-            + "Padding: (" + getPaddingBefore(false, null) + ", " + getPaddingAfter(false, null) 
+            + "Padding: (" + getPaddingBefore(false, null) + ", " + getPaddingAfter(false, null)
             + ", " + getPaddingStart(false, null) + ", " + getPaddingEnd(false, null) + ")\n";
     }
 
@@ -382,24 +456,24 @@ public class CommonBorderPaddingBackground {
      * @return true if there is any kind of background to be painted
      */
     public boolean hasBackground() {
-        return ((backgroundColor != null || getFopImage() != null));
+        return ((backgroundColor != null || getImageInfo() != null));
     }
 
     /** @return true if border is non-zero. */
     public boolean hasBorder() {
-        return ((getBorderBeforeWidth(false) + getBorderAfterWidth(false) 
+        return ((getBorderBeforeWidth(false) + getBorderAfterWidth(false)
                 + getBorderStartWidth(false) + getBorderEndWidth(false)) > 0);
     }
 
     /** 
      * @param context for percentage based evaluation.
-     * @return true if padding is non-zero. 
+     * @return true if padding is non-zero.
      */
     public boolean hasPadding(PercentBaseContext context) {
-        return ((getPaddingBefore(false, context) + getPaddingAfter(false, context) 
+        return ((getPaddingBefore(false, context) + getPaddingAfter(false, context)
                 + getPaddingStart(false, context) + getPaddingEnd(false, context)) > 0);
     }
-    
+
     /** @return true if there are any borders defined. */
     public boolean hasBorderInfo() {
         return (borderInfo[BEFORE] != null || borderInfo[AFTER] != null
