@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: EnumProperty.java 554251 2007-07-07 20:13:41Z adelmelle $ */
+/* $Id: EnumProperty.java 627367 2008-02-13 12:03:30Z maxberger $ */
 
 package org.apache.fop.fo.properties;
 
@@ -26,7 +26,7 @@ import org.apache.fop.fo.expr.PropertyException;
 /**
  * Superclass for properties that wrap an enumeration value
  */
-public class EnumProperty extends Property {
+public final class EnumProperty extends Property {
     
     /** cache holding all canonical EnumProperty instances */
     private static final PropertyCache cache = new PropertyCache();
@@ -97,7 +97,7 @@ public class EnumProperty extends Property {
     }
 
     /**
-     * @see java.lang.Object#equals(Object)
+     * {@inheritDoc}
      */
     public boolean equals(Object obj) {
         if (obj instanceof EnumProperty) {
@@ -112,7 +112,7 @@ public class EnumProperty extends Property {
     }
 
     /**
-     * @see java.lang.Object#hashCode()
+     * {@inheritDoc}
      */
     public int hashCode() {
         return value + text.hashCode();
