@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: NumberProperty.java 563951 2007-08-08 17:37:33Z vhennebert $ */
+/* $Id: NumberProperty.java 627712 2008-02-14 11:36:01Z vhennebert $ */
 
 package org.apache.fop.fo.properties;
 
@@ -32,7 +32,7 @@ import org.apache.fop.fo.expr.PropertyException;
 /**
  * Class for handling numeric properties
  */
-public class NumberProperty extends Property implements Numeric {
+public final class NumberProperty extends Property implements Numeric {
 
     /**
      * Inner class for making NumberProperty objects
@@ -48,8 +48,7 @@ public class NumberProperty extends Property implements Numeric {
         }
 
         /**
-         * @throws PropertyException 
-         * @see PropertyMaker#convertProperty
+         * {@inheritDoc}
          */
         public Property convertProperty(Property p,
                                         PropertyList propertyList, FObj fo) 
@@ -62,7 +61,7 @@ public class NumberProperty extends Property implements Numeric {
             }
             Number val = p.getNumber();
             if (val != null) {
-                return new NumberProperty(val);
+                return getInstance(val.doubleValue());
             }
             return convertPropertyDatatype(p, propertyList, fo);
         }
@@ -74,41 +73,54 @@ public class NumberProperty extends Property implements Numeric {
     
     private final Number number;
 
-    /**
-     * Constructor for Number input
-     * @param num Number object value for property
-     */
-    protected NumberProperty(Number num) {
-        this.number = num;
-    }
-
     /**
      * Constructor for double input
      * @param num double numeric value for property
      */
-    protected NumberProperty(double num) {
-        this.number = new Double(num);
+    private NumberProperty(double num) {
+        //Store the number as an int or a long,
+        //if possible
+        if (num == Math.floor(num)) {
+            if (num < Integer.MAX_VALUE) {
+                this.number = new Integer((int)num);
+            } else {
+                this.number = new Long((long)num);
+            }
+        } else {
+            this.number = new Double(num);
+        }
     }
 
     /**
      * Constructor for integer input
      * @param num integer numeric value for property
      */
-    protected NumberProperty(int num) {
+    private NumberProperty(int num) {
         this.number = new Integer(num);
     }
     
     /**
      * Returns the canonical NumberProperty instance
      * corresponding to the given Number
-     * @param num   the base Number
+     * @param num   the base Double
      * @return  the canonical NumberProperty
      */
-    public static NumberProperty getInstance(Number num) {
+    public static NumberProperty getInstance(Double num) {
         return (NumberProperty)cache.fetch(
-                    new NumberProperty(num));
+                    new NumberProperty(num.doubleValue()));
     }
     
+    /**
+     * Returns the canonical NumberProperty instance
+     * corresponding to the given Integer
+     * @param num   the base Integer
+     * @return  the canonical NumberProperty
+     */
+    public static NumberProperty getInstance(Integer num) {
+        return (NumberProperty)cache.fetch(
+                    new NumberProperty(num.intValue()));
+    }
+
     /**
      * Returns the canonical NumberProperty instance
      * corresponding to the given double
@@ -134,7 +146,6 @@ public class NumberProperty extends Property implements Numeric {
     /**
      * Plain number always has a dimension of 0.
      * @return a dimension of 0.
-     * @see Numeric#getDimension()
      */
     public int getDimension() {
         return 0;
@@ -143,7 +154,6 @@ public class NumberProperty extends Property implements Numeric {
     /**
      * Return the value of this Numeric.
      * @return The value as a double.
-     * @see Numeric#getNumericValue()
      */
     public double getNumericValue() {
         return number.doubleValue();
@@ -153,13 +163,12 @@ public class NumberProperty extends Property implements Numeric {
      * Return the value of this Numeric.
      * @param context Evaluation context
      * @return The value as a double.
-     * @see Numeric#getNumericValue(PercentBaseContext)
      */
     public double getNumericValue(PercentBaseContext context) {
         return getNumericValue();
     }
 
-    /** @see org.apache.fop.datatypes.Numeric#getValue() */
+    /** {@inheritDoc} */
     public int getValue() {
         return number.intValue();
     }
@@ -168,7 +177,6 @@ public class NumberProperty extends Property implements Numeric {
      * Return the value
      * @param context Evaluation context
      * @return The value as an int.
-     * @see Numeric#getValue(PercentBaseContext)
      */
     public int getValue(PercentBaseContext context) {
         return getValue();
@@ -177,7 +185,6 @@ public class NumberProperty extends Property implements Numeric {
     /**
      * Return true because all numbers are absolute.
      * @return true.
-     * @see Numeric#isAbsolute()
      */
     public boolean isAbsolute() {
         return true;
@@ -205,7 +212,7 @@ public class NumberProperty extends Property implements Numeric {
         return this;
     }
 
-    /** @see org.apache.fop.fo.properties.Property#getLength() */
+    /** {@inheritDoc} */
     public Length getLength() {
         //Assume pixels (like in HTML) when there's no unit
         return FixedLength.getInstance(getNumericValue(), "px");
@@ -230,13 +237,15 @@ public class NumberProperty extends Property implements Numeric {
     
     /** {@inheritDoc} */
     public boolean equals(Object o) {
-        if (o != null && o instanceof NumberProperty) {
+        if (o == this) {
+            return true;
+        }
+        if (o instanceof NumberProperty) {
             NumberProperty np = (NumberProperty) o;
             return (np.number == this.number
                     || (this.number != null
                         && this.number.equals(np.number)));
-        } else {
-            return false;
         }
+        return false;
     }
 }
