@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PercentLength.java 434497 2006-08-24 20:08:26Z adelmelle $ */
+/* $Id: PercentLength.java 627367 2008-02-13 12:03:30Z maxberger $ */
 
 package org.apache.fop.fo.properties;
 
@@ -74,22 +74,18 @@ public class PercentLength extends LengthProperty {
 
     /**
      * Return false because percent-length are always relative.
-     * @see org.apache.fop.datatypes.Numeric#isAbsolute()
+     * {@inheritDoc}
      */
     public boolean isAbsolute() {
         return false;
     }
 
-    /**
-     * @see org.apache.fop.datatypes.Numeric#getNumericValue()
-     */
+    /** {@inheritDoc} */
     public double getNumericValue() {
         return getNumericValue(null);
     }
 
-    /**
-     * @see org.apache.fop.datatypes.Numeric#getNumericValue(PercentBaseContext)
-     */
+    /** {@inheritDoc} */
     public double getNumericValue(PercentBaseContext context) {
         try {
             resolvedValue = factor * lbase.getBaseLength(context);
@@ -99,18 +95,21 @@ public class PercentLength extends LengthProperty {
             return 0;
         }
     }
+    
+    /** {@inheritDoc} */
+    public String getString() {
+        return (factor * 100.0) + "%";
+    }
 
     /**
      * Return the length of this PercentLength.
-     * @see org.apache.fop.datatypes.Length#getValue() 
+     * {@inheritDoc} 
      */
     public int getValue() {
         return (int) getNumericValue();
     }
 
-    /**
-     * @see org.apache.fop.datatypes.Numeric#getValue(PercentBaseContext)
-     */
+    /** {@inheritDoc} */
     public int getValue(PercentBaseContext context) {
         return (int) getNumericValue(context);
     }
@@ -119,8 +118,11 @@ public class PercentLength extends LengthProperty {
      * @return the String equivalent of this
      */
     public String toString() {
-        // TODO: What about the base value?
-        return (new Double(factor * 100.0).toString()) + "%";
+        StringBuffer sb = 
+            new StringBuffer(PercentLength.class.getName())
+                .append("[factor=").append(factor)
+                .append(",lbase=").append(lbase).append("]");
+        return sb.toString();
     }
 
 }
