@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: StringProperty.java 554251 2007-07-07 20:13:41Z adelmelle $ */
+/* $Id: StringProperty.java 627367 2008-02-13 12:03:30Z maxberger $ */
 
 package org.apache.fop.fo.properties;
 
@@ -26,7 +26,7 @@ import org.apache.fop.fo.PropertyList;
  * Exists primarily as a container for its Maker inner class, which is
  * extended by many string-based FO property classes.
  */
-public class StringProperty extends Property {
+public final class StringProperty extends Property {
 
     /**
      * Inner class for making instances of StringProperty
@@ -116,7 +116,7 @@ public class StringProperty extends Property {
     }
 
     /**
-     * @see java.lang.Object#hashCode()
+     * {@inheritDoc}
      */
     public boolean equals(Object obj) {
         if (obj instanceof StringProperty) {
@@ -129,7 +129,7 @@ public class StringProperty extends Property {
     }
     
     /**
-     * @see java.lang.Object#hashCode()
+     * {@inheritDoc}
      */
     public int hashCode() {
         return str.hashCode();
