@@ -15,12 +15,15 @@
  * limitations under the License.
  */
 
-/* $Id: CustomFont.java 542237 2007-05-28 14:31:24Z jeremias $ */
+/* $Id: CustomFont.java 649006 2008-04-17 08:42:52Z jeremias $ */
 
 package org.apache.fop.fonts;
 
 import java.io.IOException;
+import java.util.Collections;
 import java.util.Map;
+import java.util.Set;
+
 import javax.xml.transform.Source;
 
 /**
@@ -30,6 +33,8 @@ public abstract class CustomFont extends Typeface
             implements FontDescriptor, MutableFont {
 
     private String fontName = null;
+    private String fullName = null;
+    private Set familyNames = null; //Set<String>
     private String fontSubName = null;
     private String embedFileName = null;
     private String embedResourceName = null;
@@ -41,6 +46,7 @@ public abstract class CustomFont extends Typeface
     private int descender = 0;
     private int[] fontBBox = {0, 0, 0, 0};
     private int flags = 4;
+    private int weight = 0; //0 means unknown weight
     private int stemV = 0;
     private int italicAngle = 0;
     private int missingWidth = 0;
@@ -52,18 +58,36 @@ public abstract class CustomFont extends Typeface
 
     private boolean useKerning = true;
 
-    /**
-     * @see org.apache.fop.fonts.FontMetrics#getFontName()
-     */
+    /** {@inheritDoc} */
     public String getFontName() {
         return fontName;
     }
 
+    /** {@inheritDoc} */
+    public String getEmbedFontName() {
+        return getFontName();
+    }
+
+    /** {@inheritDoc} */
+    public String getFullName() {
+        return fullName;
+    }
+    
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getStrippedFontName()
+     * Returns the font family names.
+     * @return the font family names (a Set of Strings)
+     */
+    public Set getFamilyNames() {
+        return Collections.unmodifiableSet(this.familyNames);
+    }
+
+    /**
+     * Returns the font family name stripped of whitespace.
+     * @return the stripped font family
+     * @see FontUtil#stripWhiteSpace(String)
      */
     public String getStrippedFontName() {
-        return FontUtil.stripWhiteSpace(fontName);
+        return FontUtil.stripWhiteSpace(getFontName());
     }
 
     /**
@@ -92,7 +116,10 @@ public abstract class CustomFont extends Typeface
         Source result = null;
         if (resolver != null && embedFileName != null) {
             result = resolver.resolve(embedFileName);
-            if(result == null) throw new IOException("Unable to resolve Source '" + embedFileName + "' for embedded font");
+            if (result == null) {
+                throw new IOException("Unable to resolve Source '" 
+                        + embedFileName + "' for embedded font");
+            }
         }
         return result;
     }
@@ -108,77 +135,91 @@ public abstract class CustomFont extends Typeface
     }
 
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#getAscender()
+     * {@inheritDoc}
      */
     public int getAscender() {
         return ascender;
     }
 
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#getDescender()
+     * {@inheritDoc}
      */
     public int getDescender() {
         return descender;
     }
 
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#getCapHeight()
+     * {@inheritDoc}
      */
     public int getCapHeight() {
         return capHeight;
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getAscender(int)
+     * {@inheritDoc}
      */
     public int getAscender(int size) {
         return size * ascender;
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getDescender(int)
+     * {@inheritDoc}
      */
     public int getDescender(int size) {
         return size * descender;
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getCapHeight(int)
+     * {@inheritDoc}
      */
     public int getCapHeight(int size) {
         return size * capHeight;
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getXHeight(int)
+     * {@inheritDoc}
      */
     public int getXHeight(int size) {
         return size * xHeight;
     }
 
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#getFontBBox()
+     * {@inheritDoc}
      */
     public int[] getFontBBox() {
         return fontBBox;
     }
 
-    /**
-     * @see org.apache.fop.fonts.FontDescriptor#getFlags()
-     */
+    /** {@inheritDoc} */
     public int getFlags() {
         return flags;
     }
+    
+    /** {@inheritDoc} */
+    public boolean isSymbolicFont() {
+        return ((getFlags() & 4) != 0) || "ZapfDingbatsEncoding".equals(getEncoding());
+        //Note: The check for ZapfDingbats is necessary as the PFM does not reliably indicate
+        //if a font is symbolic.
+    }
 
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#getStemV()
+     * Returns the font weight (100, 200...800, 900). This value may be different from the
+     * one that was actually used to register the font.
+     * @return the font weight (or 0 if the font weight is unknown)
+     */
+    public int getWeight() {
+        return this.weight;
+    }
+    
+    /**
+     * {@inheritDoc}
      */
     public int getStemV() {
         return stemV;
     }
 
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#getItalicAngle()
+     * {@inheritDoc}
      */
     public int getItalicAngle() {
         return italicAngle;
@@ -193,7 +234,7 @@ public abstract class CustomFont extends Typeface
     }
 
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#getFontType()
+     * {@inheritDoc}
      */
     public FontType getFontType() {
         return fontType;
@@ -204,9 +245,7 @@ public abstract class CustomFont extends Typeface
      * @return the index of the first character
      */
     public int getFirstChar() {
-        return 0;
-        // return firstChar;
-        /**(todo) Why is this hardcoded??? This code was in SingleByteFont.java */
+        return firstChar;
     }
 
     /**
@@ -226,14 +265,14 @@ public abstract class CustomFont extends Typeface
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#hasKerningInfo()
+     * {@inheritDoc}
      */
     public final boolean hasKerningInfo() {
         return (isKerningEnabled() && (kerning != null) && !kerning.isEmpty());
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getKerningInfo()
+     * {@inheritDoc}
      */
     public final Map getKerningInfo() {
         if (hasKerningInfo()) {
@@ -243,16 +282,23 @@ public abstract class CustomFont extends Typeface
         }
     }
 
-
     /* ---- MutableFont interface ---- */
 
-    /**
-     * @see org.apache.fop.fonts.MutableFont#setFontName(String)
-     */
+    /** {@inheritDoc} */
     public void setFontName(String name) {
         this.fontName = name;
     }
 
+    /** {@inheritDoc} */
+    public void setFullName(String name) {
+        this.fullName = name;
+    }
+    
+    /** {@inheritDoc} */
+    public void setFamilyNames(Set names) {
+        this.familyNames = new java.util.HashSet(names);
+    }
+    
     /**
      * Sets the font's subfamily name.
      * @param subFamilyName the subfamily name of the font
@@ -262,21 +308,21 @@ public abstract class CustomFont extends Typeface
     }
     
     /**
-     * @see org.apache.fop.fonts.MutableFont#setEmbedFileName(String)
+     * {@inheritDoc}
      */
     public void setEmbedFileName(String path) {
         this.embedFileName = path;
     }
 
     /**
-     * @see org.apache.fop.fonts.MutableFont#setEmbedResourceName(String)
+     * {@inheritDoc}
      */
     public void setEmbedResourceName(String name) {
         this.embedResourceName = name;
     }
 
     /**
-     * @see org.apache.fop.fonts.MutableFont#setCapHeight(int)
+     * {@inheritDoc}
      */
     public void setCapHeight(int capHeight) {
         this.capHeight = capHeight;
@@ -291,77 +337,88 @@ public abstract class CustomFont extends Typeface
     }
 
     /**
-     * @see org.apache.fop.fonts.MutableFont#setAscender(int)
+     * {@inheritDoc}
      */
     public void setAscender(int ascender) {
         this.ascender = ascender;
     }
 
     /**
-     * @see org.apache.fop.fonts.MutableFont#setDescender(int)
+     * {@inheritDoc}
      */
     public void setDescender(int descender) {
         this.descender = descender;
     }
 
     /**
-     * @see org.apache.fop.fonts.MutableFont#setFontBBox(int[])
+     * {@inheritDoc}
      */
     public void setFontBBox(int[] bbox) {
         this.fontBBox = bbox;
     }
 
     /**
-     * @see org.apache.fop.fonts.MutableFont#setFlags(int)
+     * {@inheritDoc}
      */
     public void setFlags(int flags) {
         this.flags = flags;
     }
 
     /**
-     * @see org.apache.fop.fonts.MutableFont#setStemV(int)
+     * Sets the font weight. Valid values are 100, 200...800, 900.
+     * @param weight the font weight
+     */
+    public void setWeight(int weight) {
+        weight = (weight / 100) * 100;
+        weight = Math.max(100, weight);
+        weight = Math.min(900, weight);
+        this.weight = weight;
+    }
+    
+    /**
+     * {@inheritDoc}
      */
     public void setStemV(int stemV) {
         this.stemV = stemV;
     }
 
     /**
-     * @see org.apache.fop.fonts.MutableFont#setItalicAngle(int)
+     * {@inheritDoc}
      */
     public void setItalicAngle(int italicAngle) {
         this.italicAngle = italicAngle;
     }
 
     /**
-     * @see org.apache.fop.fonts.MutableFont#setMissingWidth(int)
+     * {@inheritDoc}
      */
     public void setMissingWidth(int width) {
         this.missingWidth = width;
     }
 
     /**
-     * @see org.apache.fop.fonts.MutableFont#setFontType(FontType)
+     * {@inheritDoc}
      */
     public void setFontType(FontType fontType) {
         this.fontType = fontType;
     }
 
     /**
-     * @see org.apache.fop.fonts.MutableFont#setFirstChar(int)
+     * {@inheritDoc}
      */
     public void setFirstChar(int index) {
         this.firstChar = index;
     }
 
     /**
-     * @see org.apache.fop.fonts.MutableFont#setLastChar(int)
+     * {@inheritDoc}
      */
     public void setLastChar(int index) {
         this.lastChar = index;
     }
 
     /**
-     * @see org.apache.fop.fonts.MutableFont#setKerningEnabled(boolean)
+     * {@inheritDoc}
      */
     public void setKerningEnabled(boolean enabled) {
         this.useKerning = enabled;
@@ -375,14 +432,25 @@ public abstract class CustomFont extends Typeface
         this.resolver = resolver;
     }
 
-    /**
-     * @see org.apache.fop.fonts.MutableFont#putKerningEntry(Integer, Map)
-     */
+    /** {@inheritDoc} */
     public void putKerningEntry(Integer key, Map value) {
         if (kerning == null) {
             kerning = new java.util.HashMap();
         }
         this.kerning.put(key, value);
     }
+    
+    /**
+     * Replaces the existing kerning map with a new one.
+     * @param kerningMap the kerning map (Map<Integer, Map<Integer, Integer>, the integers are
+     *                          character codes)
+     */
+    public void replaceKerningMap(Map kerningMap) {
+        if (kerningMap == null) {
+            this.kerning = Collections.EMPTY_MAP;
+        } else {
+            this.kerning = kerningMap;
+        }
+    }
 
 }
