@@ -15,11 +15,10 @@
  * limitations under the License.
  */
 
-/* $Id: Font.java 563951 2007-08-08 17:37:33Z vhennebert $ */
+/* $Id: Font.java 594067 2007-11-12 09:40:16Z jeremias $ */
 
 package org.apache.fop.fonts;
 
-import java.util.BitSet;
 import java.util.Map;
 
 import org.apache.commons.logging.Log;
@@ -31,12 +30,18 @@ import org.apache.commons.logging.LogFactory;
  */
 public class Font {
 
+    /** Extra Bold font weight */
+    public static final int WEIGHT_EXTRA_BOLD = 800;
+    
     /** Bold font weight */
     public static final int WEIGHT_BOLD = 700;
-
+    
     /** Normal font weight */
     public static final int WEIGHT_NORMAL = 400;
 
+    /** Light font weight */
+    public static final int WEIGHT_LIGHT = 200;
+
     /** Normal font style */
     public static final String STYLE_NORMAL = "normal";
 
@@ -75,6 +80,14 @@ public class Font {
         this.fontSize = fontSize;
     }
 
+    /**
+     * Returns the associated font metrics object.
+     * @return the font metrics
+     */
+    public FontMetrics getFontMetrics() {
+        return this.metric;
+    }
+    
     /**
      * Returns the font's ascender.
      * @return the ascender
@@ -211,7 +224,7 @@ public class Font {
     }
 
     /**
-     * @see java.lang.Object#toString()
+     * {@inheritDoc}
      */
     public String toString() {
         StringBuffer sbuf = new StringBuffer();
