@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FontInfo.java 438923 2006-08-31 15:56:32 +0200 (Thu, 31 Aug 2006) adelmelle $ */
+/* $Id: FontInfo.java 629103 2008-02-19 14:16:08Z jeremias $ */
 
 package org.apache.fop.fonts;
 
@@ -41,17 +41,23 @@ import org.apache.commons.logging.LogFactory;
  */
 public class FontInfo {
     
+    private static final FontTriplet[] TRIPLETS_TYPE = new FontTriplet[1];
+    
     /** logging instance */
     protected static Log log = LogFactory.getLog(FontInfo.class);
 
     /** Map containing fonts that have been used */
-    private Map usedFonts;
+    private Map usedFonts; //Map<String,FontMetrics> (String = font key)
     
     /** look up a font-triplet to find a font-name */
-    private Map triplets;
+    private Map triplets; //Map<FontTriplet,String> (String = font key)
     
+    /** look up a font-triplet to find its priority
+     *  (only used inside addFontProperties()) */
+    private Map tripletPriorities; //Map<FontTriplet,Integer>
+
     /** look up a font-name to get a font (that implements FontMetrics at least) */
-    private Map fonts;
+    private Map fonts; //Map<String,FontMetrics> (String = font key)
     
     /** collection of missing fonts; used to make sure the user gets 
      *  a warning for a missing font only once (not every time the font is used)
@@ -66,6 +72,7 @@ public class FontInfo {
      */
     public FontInfo() {
         this.triplets = new java.util.HashMap();
+        this.tripletPriorities = new java.util.HashMap();
         this.fonts = new java.util.HashMap();
         this.usedFonts = new java.util.HashMap();
     }
@@ -76,6 +83,8 @@ public class FontInfo {
      * @return True if valid
      */
     public boolean isSetupValid() {
+        //We're only called when font setup is done:
+        tripletPriorities = null; // candidate for garbage collection
         return triplets.containsKey(Font.DEFAULT_FONT);
     }
 
@@ -100,7 +109,45 @@ public class FontInfo {
          * add the given family, style and weight as a lookup for the font
          * with the given name
          */
+        if (log.isDebugEnabled()) {
+            log.debug("Registering: " + triplet + " under " + name);
+        }
+        String oldName = (String)triplets.get(triplet);
+        int newPriority = triplet.getPriority();
+        if (oldName != null) {
+            int oldPriority = ((Integer)tripletPriorities.get(triplet)).intValue();
+            if (oldPriority < newPriority) {
+                logDuplicateFont(triplet, false, oldName, oldPriority,
+                            name, newPriority);
+                return;
+            } else {
+                logDuplicateFont(triplet, true, oldName, oldPriority,
+                            name, newPriority);
+            }
+        }
         this.triplets.put(triplet, name);
+        this.tripletPriorities.put(triplet, new Integer(newPriority));
+    }
+
+    /** Log warning about duplicate font triplets.
+     * @param triplet the duplicate font triplet
+     * @param replacing true iff the new font will replace the old one
+     * @param oldKey the old internal font name
+     * @param oldPriority the priority of the existing font mapping
+     * @param newKey the new internal font name
+     * @param newPriority the priority of the duplicate font mapping
+     */
+    private void logDuplicateFont(FontTriplet triplet, boolean replacing,
+                                  String oldKey, int oldPriority,
+                                  String newKey, int newPriority) {
+        if (log.isDebugEnabled()) {
+            log.debug(triplet
+                    + (replacing ? ": Replacing " : ": Not replacing ")
+                    + ((FontMetrics)fonts.get(triplets.get(triplet))).getFullName()
+                    + " (" + oldPriority + ") by "
+                    + ((FontMetrics)fonts.get(newKey)).getFullName()
+                    + " (" + newPriority + ")");
+        }
     }
 
     /**
@@ -130,46 +177,18 @@ public class FontInfo {
      */
     private FontTriplet fontLookup(String family, String style,
                              int weight, boolean substFont) {
-        FontTriplet startKey = createFontKey(family, style, weight); 
+        if (log.isTraceEnabled()) {
+            log.trace("Font lookup: " + family + " " + style + " " + weight);
+        }
+        FontTriplet startKey = createFontKey(family, style, weight);
         FontTriplet key = startKey;
         // first try given parameters
         String f = getInternalFontKey(key);
         if (f == null) {
-            // then adjust weight, favouring normal or bold
-            key = findAdjustWeight(family, style, weight);
-            f = getInternalFontKey(key);
-
-            if (!substFont && f == null) {
-                return null;
-            }
-            
-            // only if the font may be substituted
-            // fallback 1: try the same font-family and weight with default style
-            if (f == null) {
-                key = createFontKey(family, "normal", weight);
-                f = getInternalFontKey(key);
-            }
-            
-            // fallback 2: try the same font-family with default style and weight
-            if (f == null) {
-                key = createFontKey(family, "normal", 400);
-                f = getInternalFontKey(key);
-            }
-            
-            // fallback 3: try any family with orig style/weight
-            if (f == null) {
-                key = createFontKey("any", style, weight);
-                f = getInternalFontKey(key);
-            }
-
-            // last resort: use default
-            if (f == null) {
-                key = Font.DEFAULT_FONT;
-                f = getInternalFontKey(key);
-            }
+            key = doAdjustedLookup(family, style, weight, startKey, substFont);
         }
 
-        if (f != null) {
+        if (key != null) {
             if (key != startKey) {
                 notifyFontReplacement(startKey, key);
             }
@@ -179,6 +198,76 @@ public class FontInfo {
         }
     }
 
+    private FontTriplet doAdjustedLookup(String family, String style,
+            int weight, FontTriplet startKey, boolean substFont) {
+        FontTriplet key;
+        String f;
+        if (!family.equals(startKey.getName())) {
+            key = createFontKey(family, style, weight);
+            f = getInternalFontKey(key);
+            if (f != null) {
+                return key;
+            }
+        }
+        
+        // adjust weight, favouring normal or bold
+        key = findAdjustWeight(family, style, weight);
+        f = getInternalFontKey(key);
+
+        if (!substFont && f == null) {
+            return null;
+        }
+        
+        // only if the font may be substituted
+        // fallback 1: try the same font-family and weight with default style
+        if (f == null && style != Font.STYLE_NORMAL) {
+            key = createFontKey(family, Font.STYLE_NORMAL, weight);
+            f = getInternalFontKey(key);
+        }
+
+        if (f == null && weight != Font.WEIGHT_NORMAL) {
+            int diffWeight = (Font.WEIGHT_NORMAL - weight) / 100;
+            int direction = diffWeight > 0 ? 1 : -1;
+            int tryWeight = weight;
+            while (tryWeight != Font.WEIGHT_NORMAL) {
+                tryWeight += 100 * direction;
+                key = createFontKey(family, style, weight);
+                f = getInternalFontKey(key);
+                if (f == null) {
+                    key = createFontKey(family, Font.STYLE_NORMAL, weight);
+                    f = getInternalFontKey(key);
+                }
+                if (f != null) {
+                    break;
+                }
+            }
+        }
+        
+        // fallback 2: try the same font-family with default style and weight
+        /* obsolete: replaced by the loop above
+        if (f == null) {
+            key = createFontKey(family, Font.STYLE_NORMAL, Font.WEIGHT_NORMAL);
+            f = getInternalFontKey(key);
+        }*/
+        
+        // fallback 3: try any family with orig style/weight
+        if (f == null) {
+            return doAdjustedLookup("any", style, weight, startKey, false);
+        }
+
+        // last resort: use default
+        if (f == null) {
+            key = Font.DEFAULT_FONT;
+            f = getInternalFontKey(key);
+        }
+
+        if (f != null) {
+            return key;
+        } else {
+            return null;
+        }
+    }
+    
     /**
      * Tells this class that the font with the given internal name has been used.
      * @param internalName the internal font name (F1, F2 etc.)
@@ -229,26 +318,44 @@ public class FontInfo {
     }
     
     /**
-     * Lookup a font.
+     * Looks up a set of fonts.
      * <br>
-     * Locate the font name for a given family, style and weight.
-     * The font name can then be used as a key as it is unique for
+     * Locate the font name(s) for the given families, style and weight.
+     * The font name(s) can then be used as a key as they are unique for
      * the associated document.
-     * This also adds the font to the list of used fonts.
-     * @param family font family (priority list)
-     * @param style font style
-     * @param weight font weight
-     * @return font triplet of the font chosen
+     * This also adds the fonts to the list of used fonts.
+     * @param families  font families (priority list)
+     * @param style     font style
+     * @param weight    font weight
+     * @return the set of font triplets of all supported and chosen font-families
+     *          in the specified style and weight.
      */
-    public FontTriplet fontLookup(String[] family, String style,
+    public FontTriplet[] fontLookup(String[] families, String style,
                              int weight) {
-        for (int i = 0; i < family.length; i++) {
-            FontTriplet triplet = fontLookup(family[i], style, weight, (i >= family.length - 1));
+        if (families.length == 0) {
+            throw new IllegalArgumentException("Specify at least one font family");
+        }
+        FontTriplet triplet;
+        List tmpTriplets = new ArrayList();
+        for (int i = 0, c = families.length; i < c; i++) {
+            triplet = fontLookup(families[i], style, weight, (i >= families.length - 1));
             if (triplet != null) {
-                return triplet;
+                tmpTriplets.add(triplet);
             }
         }
-        throw new IllegalStateException("fontLookup must return a key on the last call");
+        if (tmpTriplets.size() != 0) {
+            return (FontTriplet[]) tmpTriplets.toArray(TRIPLETS_TYPE);
+        }
+        StringBuffer sb = new StringBuffer();
+        for (int i = 0, c = families.length; i < c; i++) {
+            if (i > 0) {
+                sb.append(", ");
+            }
+            sb.append(families[i]);
+        }
+        throw new IllegalStateException(
+                    "fontLookup must return an array with at least one "
+                    + "FontTriplet on the last call. Lookup: " + sb.toString());
     }
     
     private void notifyFontReplacement(FontTriplet replacedKey, FontTriplet newKey) {
@@ -276,11 +383,17 @@ public class FontInfo {
         String f = null;
         int newWeight = weight;
         if (newWeight < 400) {
-            while (f == null && newWeight > 0) {
+            while (f == null && newWeight > 100) {
                 newWeight -= 100;
                 key = createFontKey(family, style, newWeight);
                 f = getInternalFontKey(key);
             }
+            newWeight = weight;
+            while (f == null && newWeight < 400) {
+                newWeight += 100;
+                key = createFontKey(family, style, newWeight);
+                f = getInternalFontKey(key);
+            }
         } else if (newWeight == 500) {
             key = createFontKey(family, style, 400);
             f = getInternalFontKey(key);
@@ -343,13 +456,21 @@ public class FontInfo {
     }
 
     /**
-     * Gets a Map of all registred fonts.
+     * Gets a Map of all registered fonts.
      * @return a read-only Map with font key/FontMetrics pairs
      */
     public Map getFonts() {
         return java.util.Collections.unmodifiableMap(this.fonts);
     }
 
+    /**
+     * Gets a Map of all registered font triplets.
+     * @return a read-only Map with FontTriplet/font key pairs
+     */
+    public Map getFontTriplets() {
+        return java.util.Collections.unmodifiableMap(this.triplets);
+    }
+    
     /**
      * This is used by the renderers to retrieve all the
      * fonts used in the document.
@@ -427,7 +548,21 @@ public class FontInfo {
         }
     }
     
+    /**
+     * Diagnostic method for logging all registered fonts to System.out. 
+     */
+    public void dumpAllTripletsToSystemOut() {
+        Collection entries = new java.util.TreeSet();
+        Iterator iter = this.triplets.keySet().iterator();
+        while (iter.hasNext()) {
+            FontTriplet triplet = (FontTriplet)iter.next();
+            String key = getInternalFontKey(triplet);
+            FontMetrics metrics = getMetricsFor(key); 
+            entries.add(triplet.toString() + " -> " + key + " -> " + metrics.getFontName());
+        }
+        iter = entries.iterator();
+        while (iter.hasNext()) {
+            System.out.println(iter.next());
+        }
+    }
 }
-
-
-
