@@ -15,20 +15,24 @@
  * limitations under the License.
  */
 
-/* $Id: LazyFont.java 542237 2007-05-28 14:31:24Z jeremias $ */
+/* $Id: LazyFont.java 649006 2008-04-17 08:42:52Z jeremias $ */
 
 package org.apache.fop.fonts;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.URL;
 import java.util.Map;
+import java.util.Set;
+
 import javax.xml.transform.Source;
 import javax.xml.transform.stream.StreamSource;
 
+import org.xml.sax.InputSource;
+
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.apps.FOPException;
-import org.xml.sax.InputSource;
 
 /**
  * This class is used to defer the loading of a font until it is really used.
@@ -49,9 +53,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     
     /**
      * Main constructor
-     * @param fontEmbedPath path to embeddable file (may be null)
-     * @param metricsFileName path to the metrics XML file
-     * @param useKerning True, if kerning should be enabled
+     * @param fontInfo  the font info to embed
      * @param resolver the font resolver to handle font URIs
      */
     public LazyFont(EmbedFontInfo fontInfo, FontResolver resolver) {
@@ -95,8 +97,10 @@ public class LazyFont extends Typeface implements FontDescriptor {
                             in = new java.net.URL(source.getSystemId()).openStream();
                         }
                         if (in == null) {
-                            String err = "Cannot load font: failed to create InputStream from"
-                                + " Source for metrics file " + metricsFileName; 
+                            String err = "Cannot load font: After URI resolution, the returned"
+                                + " Source object does not contain an InputStream"
+                                + " or a valid URL (system identifier) for metrics file: "
+                                + metricsFileName; 
                             if (fail) {
                                 throw new RuntimeException(err);
                             } else {
@@ -150,7 +154,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
 
     // ---- Font ----
     /**
-     * @see org.apache.fop.fonts.Typeface#getEncoding()
+     * {@inheritDoc}
      */
     public String getEncoding() {
         load(true);
@@ -158,7 +162,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.Typeface#mapChar(char)
+     * {@inheritDoc}
      */
     public char mapChar(char c) {
         load(true);
@@ -166,7 +170,15 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.Typeface#hasChar(char)
+     * {@inheritDoc}
+     */
+    public boolean hadMappingOperations() {
+        load(true);
+        return realFont.hadMappingOperations();
+    }
+    
+    /**
+     * {@inheritDoc}
      */
     public boolean hasChar(char c) {
         load(true);
@@ -174,7 +186,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.Typeface#isMultiByte()
+     * {@inheritDoc}
      */
     public boolean isMultiByte() {
         load(true);
@@ -182,16 +194,32 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     // ---- FontMetrics interface ----
-    /**
-     * @see org.apache.fop.fonts.FontMetrics#getFontName()
-     */
+    /** {@inheritDoc} */
     public String getFontName() {
         load(true);
         return realFont.getFontName();
     }
 
+    /** {@inheritDoc} */
+    public String getEmbedFontName() {
+        load(true);
+        return realFont.getEmbedFontName();
+    }
+
+    /** {@inheritDoc} */
+    public String getFullName() {
+        load(true);
+        return realFont.getFullName();
+    }
+
+    /** {@inheritDoc} */
+    public Set getFamilyNames() {
+        load(true);
+        return realFont.getFamilyNames();
+    }
+
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getMaxAscent(int)
+     * {@inheritDoc}
      */
     public int getMaxAscent(int size) {
         load(true);
@@ -199,7 +227,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getAscender(int)
+     * {@inheritDoc}
      */
     public int getAscender(int size) {
         load(true);
@@ -207,7 +235,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getCapHeight(int)
+     * {@inheritDoc}
      */
     public int getCapHeight(int size) {
         load(true);
@@ -215,7 +243,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getDescender(int)
+     * {@inheritDoc}
      */
     public int getDescender(int size) {
         load(true);
@@ -223,7 +251,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getXHeight(int)
+     * {@inheritDoc}
      */
     public int getXHeight(int size) {
         load(true);
@@ -231,7 +259,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getWidth(int, int)
+     * {@inheritDoc} 
      */
     public int getWidth(int i, int size) {
         load(true);
@@ -239,7 +267,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getWidths()
+     * {@inheritDoc}
      */
     public int[] getWidths() {
         load(true);
@@ -247,7 +275,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#hasKerningInfo()
+     * {@inheritDoc}
      */
     public boolean hasKerningInfo() {
         load(true);
@@ -255,7 +283,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getKerningInfo()
+     * {@inheritDoc}
      */
     public Map getKerningInfo() {
         load(true);
@@ -264,7 +292,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
 
     // ---- FontDescriptor interface ----
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#getCapHeight()
+     * {@inheritDoc}
      */
     public int getCapHeight() {
         load(true);
@@ -272,7 +300,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#getDescender()
+     * {@inheritDoc}
      */
     public int getDescender() {
         load(true);
@@ -280,23 +308,27 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#getAscender()
+     * {@inheritDoc}
      */
     public int getAscender() {
         load(true);
         return realFontDescriptor.getAscender();
     }
 
-    /**
-     * @see org.apache.fop.fonts.FontDescriptor#getFlags()
-     */
+    /** {@inheritDoc} */
     public int getFlags() {
         load(true);
         return realFontDescriptor.getFlags();
     }
 
+    /** {@inheritDoc} */
+    public boolean isSymbolicFont() {
+        load(true);
+        return realFontDescriptor.isSymbolicFont();
+    }
+
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#getFontBBox()
+     * {@inheritDoc}
      */
     public int[] getFontBBox() {
         load(true);
@@ -304,7 +336,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#getItalicAngle()
+     * {@inheritDoc}
      */
     public int getItalicAngle() {
         load(true);
@@ -312,7 +344,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#getStemV()
+     * {@inheritDoc}
      */
     public int getStemV() {
         load(true);
@@ -320,7 +352,7 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#getFontType()
+     * {@inheritDoc}
      */
     public FontType getFontType() {
         load(true);
@@ -328,11 +360,12 @@ public class LazyFont extends Typeface implements FontDescriptor {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#isEmbeddable()
+     * {@inheritDoc}
      */
     public boolean isEmbeddable() {
         load(true);
         return realFontDescriptor.isEmbeddable();
     }
+
 }
 
