@@ -15,11 +15,12 @@
  * limitations under the License.
  */
 
-/* $Id: MultiByteFont.java 542237 2007-05-28 14:31:24Z jeremias $ */
+/* $Id: MultiByteFont.java 669173 2008-06-18 14:07:27Z jeremias $ */
 
 package org.apache.fop.fonts;
 
 //Java
+import java.text.DecimalFormat;
 import java.util.Map;
 
 /**
@@ -27,8 +28,7 @@ import java.util.Map;
  */
 public class MultiByteFont extends CIDFont {
 
-    private static int uniqueCounter = 1;
-
+    private static int uniqueCounter = -1;
 
     private String ttcName = null;
     private String encoding = "Identity-H";
@@ -54,48 +54,59 @@ public class MultiByteFont extends CIDFont {
         usedGlyphs.put(new Integer(2), new Integer(2));
         usedGlyphsIndex.put(new Integer(2), new Integer(2));
         usedGlyphsCount++;
-
+        
         // Create a quasiunique prefix for fontname
-        int cnt = 0;
         synchronized (this.getClass()) {
-            cnt = uniqueCounter++;
+            uniqueCounter++;
+            if (uniqueCounter > 99999 || uniqueCounter < 0) {
+                uniqueCounter = 0; //We need maximum 5 character then we start again
+            }
         }
-        int ctm = (int)(System.currentTimeMillis() & 0xffff);
-        namePrefix = new String(cnt + "E" + Integer.toHexString(ctm));
+        DecimalFormat counterFormat = new DecimalFormat("00000");
+        String cntString = counterFormat.format(uniqueCounter);
+        
+        //Subset prefix as described in chapter 5.5.3 of PDF 1.4
+        StringBuffer sb = new StringBuffer("E");
+        for (int i = 0, c = cntString.length(); i < c; i++) {
+            //translate numbers to uppercase characters
+            sb.append((char)(cntString.charAt(i) + (65 - 48)));
+        }
+        sb.append("+");
+        namePrefix = sb.toString();
 
         setFontType(FontType.TYPE0);
     }
 
     /**
-     * @see org.apache.fop.fonts.CIDFont#getDefaultWidth()
+     * {@inheritDoc}
      */
     public int getDefaultWidth() {
         return defaultWidth;
     }
 
     /**
-     * @see org.apache.fop.fonts.CIDFont#getRegistry()
+     * {@inheritDoc}
      */
     public String getRegistry() {
         return "Adobe";
     }
 
     /**
-     * @see org.apache.fop.fonts.CIDFont#getOrdering()
+     * {@inheritDoc}
      */
     public String getOrdering() {
         return "UCS";
     }
 
     /**
-     * @see org.apache.fop.fonts.CIDFont#getSupplement()
+     * {@inheritDoc}
      */
     public int getSupplement() {
         return 0;
     }
 
     /**
-     * @see org.apache.fop.fonts.CIDFont#getCIDType()
+     * {@inheritDoc}
      */
     public CIDFontType getCIDType() {
         return cidType;
@@ -113,10 +124,8 @@ public class MultiByteFont extends CIDFont {
         return namePrefix + FontUtil.stripWhiteSpace(super.getFontName());
     }
     
-    /**
-     * @see org.apache.fop.fonts.CIDFont#getCidBaseFont()
-     */
-    public String getCidBaseFont() {
+    /** {@inheritDoc} */
+    public String getEmbedFontName() {
         if (isEmbeddable()) {
             return getPrefixedFontName();
         } else {
@@ -125,32 +134,21 @@ public class MultiByteFont extends CIDFont {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontDescriptor#isEmbeddable()
+     * {@inheritDoc}
      */
     public boolean isEmbeddable() {
         return !(getEmbedFileName() == null && getEmbedResourceName() == null);
     }
 
     /**
-     * @see org.apache.fop.fonts.Typeface#getEncoding()
+     * {@inheritDoc}
      */
     public String getEncoding() {
         return encoding;
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getFontName()
-     */
-    public String getFontName() {
-        if (isEmbeddable()) {
-            return getPrefixedFontName();
-        } else {
-            return super.getFontName();
-        }
-    }
-
-    /**
-     * @see org.apache.fop.fonts.FontMetrics#getWidth(int, int)
+     * {@inheritDoc} 
      */
     public int getWidth(int i, int size) {
         if (isEmbeddable()) {
@@ -162,7 +160,7 @@ public class MultiByteFont extends CIDFont {
     }
 
     /**
-     * @see org.apache.fop.fonts.FontMetrics#getWidths()
+     * {@inheritDoc}
      */
     public int[] getWidths() {
         int[] arr = new int[width.length];
@@ -211,9 +209,10 @@ public class MultiByteFont extends CIDFont {
     }
 
     /**
-     * @see org.apache.fop.fonts.Typeface#mapChar(char)
+     * {@inheritDoc}
      */
     public char mapChar(char c) {
+        notifyMapOperation();
         int retIdx = findGlyphIndex(c);
 
         if (isEmbeddable()) {
@@ -238,7 +237,7 @@ public class MultiByteFont extends CIDFont {
     }
 
     /**
-     * @see org.apache.fop.fonts.Typeface#hasChar(char)
+     * {@inheritDoc}
      */
     public boolean hasChar(char c) {
         return (findGlyphIndex(c) > 0);
@@ -308,7 +307,7 @@ public class MultiByteFont extends CIDFont {
      * that need to return an invalid character. */
     public static final char INVALID_UNICODE_CHAR = 0xFFFF;
 
-    /** @see org.apache.fop.fonts.CIDFont#getCharsUsed() */
+    /** {@inheritDoc} */
     public char[] getCharsUsed() {
         if (!isEmbeddable()) {
             return null;
