@@ -15,13 +15,14 @@
  * limitations under the License.
  */
 
-/* $Id: TTFReader.java 542237 2007-05-28 14:31:24Z jeremias $ */
+/* $Id: TTFReader.java 593245 2007-11-08 18:36:01Z jeremias $ */
  
 package org.apache.fop.fonts.apps;
 
 import java.io.IOException;
 import java.util.Iterator;
 import java.util.Map;
+import java.util.Set;
 
 import javax.xml.parsers.DocumentBuilderFactory;
 
@@ -209,7 +210,7 @@ public class TTFReader extends AbstractFontReader {
         if (!supported) {
             return null;
         }
-        log.info("Font Family: " + ttfFile.getFamilyName());
+        log.info("Font Family: " + ttfFile.getFamilyNames());
         if (ttfFile.isCFF()) {
             throw new UnsupportedOperationException(
                     "OpenType fonts with CFF data are not supported, yet");
@@ -266,6 +267,18 @@ public class TTFReader extends AbstractFontReader {
         } else {
             el.appendChild(doc.createTextNode(s));
         }
+        if (ttf.getFullName() != null) {
+            el = doc.createElement("full-name");
+            root.appendChild(el);
+            el.appendChild(doc.createTextNode(ttf.getFullName()));
+        }
+        Set familyNames = ttf.getFamilyNames();
+        if (familyNames.size() > 0) {
+            String familyName = (String)familyNames.iterator().next(); 
+            el = doc.createElement("family-name");
+            root.appendChild(el);
+            el.appendChild(doc.createTextNode(familyName));
+        }
 
         el = doc.createElement("embed");
         root.appendChild(el);
