@@ -15,18 +15,22 @@
  * limitations under the License.
  */
 
-/* $Id: TTFFile.java 563951 2007-08-08 17:37:33Z vhennebert $ */
+/* $Id: TTFFile.java 629131 2008-02-19 15:47:48Z jeremias $ */
 
 package org.apache.fop.fonts.truetype;
 
 import java.io.IOException;
 import java.util.Iterator;
-import java.util.Map;
 import java.util.List;
+import java.util.Map;
+import java.util.Set;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
-import org.apache.fop.fonts.Glyphs;
+
+import org.apache.xmlgraphics.fonts.Glyphs;
+
+import org.apache.fop.fonts.FontUtil;
 
 /**
  * Reads a TrueType file or a TrueType Collection.
@@ -74,10 +78,10 @@ public class TTFFile {
     protected TTFMtxEntry[] mtxTab;                  // Contains glyph data
     private int[] mtxEncoded = null;
 
-    private String fontName = "";
+    private String postScriptName = "";
     private String fullName = "";
     private String notice = "";
-    private String familyName = "";
+    private Set familyNames = new java.util.HashSet(); //Set<String>
     private String subFamilyName = "";
 
     private long italicAngle = 0;
@@ -101,6 +105,7 @@ public class TTFFile {
     //Ascender/descender from OS/2 table
     private int os2Ascender = 0;
     private int os2Descender = 0;
+    private int usWeightClass = 0;
 
     private short lastChar = 0;
 
@@ -218,7 +223,9 @@ public class TTFFile {
             int cmapEID = in.readTTFUShort();
             long cmapOffset = in.readTTFULong();
 
-            log.debug("Platform ID: " + cmapPID + " Encoding: " + cmapEID);
+            if (log.isDebugEnabled()) {
+                log.debug("Platform ID: " + cmapPID + " Encoding: " + cmapEID);
+            }
 
             if (cmapPID == 3 && cmapEID == 1) {
                 cmapUniOffset = cmapOffset;
@@ -226,8 +233,7 @@ public class TTFFile {
         }
 
         if (cmapUniOffset <= 0) {
-            log.fatal("Unicode cmap table not present");
-            log.fatal("Unsupported format: Aborting");
+            log.fatal("Unsupported TrueType font: Unicode cmap table not present. Aborting");
             return false;
         }
 
@@ -537,32 +543,24 @@ public class TTFFile {
         cmaps.add(tce);
     }
 
-    /**
-     * Returns the Windows name of the font.
-     * @return String The Windows name
-     */
-    public String getWindowsName() {
-        return familyName + "," + subFamilyName;
-    }
-
     /**
      * Returns the PostScript name of the font.
      * @return String The PostScript name
      */
     public String getPostScriptName() {
-        if ("Regular".equals(subFamilyName) || "Roman".equals(subFamilyName)) {
-            return familyName;
+        if (postScriptName.length() == 0) {
+            return FontUtil.stripWhiteSpace(getFullName());
         } else {
-            return familyName + "," + subFamilyName;
+            return postScriptName;
         }
     }
 
     /**
-     * Returns the font family name of the font.
-     * @return String The family name
+     * Returns the font family names of the font.
+     * @return Set The family names (a Set of Strings)
      */
-    public String getFamilyName() {
-        return familyName;
+    public Set getFamilyNames() {
+        return familyNames;
     }
 
     /**
@@ -573,6 +571,14 @@ public class TTFFile {
         return subFamilyName;
     }
 
+    /**
+     * Returns the full name of the font.
+     * @return String The full name
+     */
+    public String getFullName() {
+        return fullName;
+    }
+
     /**
      * Returns the name of the character set used.
      * @return String The caracter set
@@ -615,6 +621,13 @@ public class TTFFile {
         return flags;
     }
 
+    /**
+     * Returns the weight class of this font. Valid values are 100, 200....,800, 900.
+     * @return the weight class value (or 0 if there was no OS/2 table in the font)
+     */
+    public int getWeightClass() {
+        return this.usWeightClass;
+    }
 
     /**
      * Returns the StemV attribute of the font.
@@ -973,7 +986,9 @@ public class TTFFile {
     private final void readOS2(FontFileReader in) throws IOException {
         // Check if font is embeddable
         if (dirTabs.get("OS/2") != null) {
-            seekTab(in, "OS/2", 2 * 4);
+            seekTab(in, "OS/2", 2 * 2);
+            this.usWeightClass = in.readTTFUShort();
+            in.skip(2);
             int fsType = in.readTTFUShort();
             if (fsType == 2) {
                 isEmbeddable = false;
@@ -1106,37 +1121,42 @@ public class TTFFile {
             int l = in.readTTFUShort();
 
             if (((platformID == 1 || platformID == 3) 
-                    && (encodingID == 0 || encodingID == 1))
-                    && (k == 1 || k == 2 || k == 0 || k == 4 || k == 6)) {
+                    && (encodingID == 0 || encodingID == 1))) {
                 in.seekSet(j + in.readTTFUShort());
                 String txt = in.readTTFString(l);
                 
-                log.debug(platformID + " " 
-                    + encodingID + " "
-                    + languageID + " "
-                    + k + " " + txt);
+                if (log.isDebugEnabled()) {
+                    log.debug(platformID + " " 
+                            + encodingID + " "
+                            + languageID + " "
+                            + k + " " + txt);
+                }
                 switch (k) {
                 case 0:
-                    notice = txt;
+                    if (notice.length() == 0) {
+                        notice = txt;
+                    }
                     break;
-                case 1:
-                    familyName = txt;
+                case 1: //Font Family Name
+                case 16: //Preferred Family
+                    familyNames.add(txt);
                     break;
                 case 2:
-                    subFamilyName = txt;
+                    if (subFamilyName.length() == 0) {
+                        subFamilyName = txt;
+                    }
                     break;
                 case 4:
-                    fullName = txt;
+                    if (fullName.length() == 0) {
+                        fullName = txt;
+                    }
                     break;
                 case 6:
-                    fontName = txt;
+                    if (postScriptName.length() == 0) {
+                        postScriptName = txt;
+                    }
                     break;
-                }
-                if (!notice.equals("")
-                        && !fullName.equals("")
-                        && !fontName.equals("")
-                        && !familyName.equals("")
-                        && !subFamilyName.equals("")) {
+                default:
                     break;
                 }
             }
@@ -1324,9 +1344,11 @@ public class TTFFile {
                         if (iObj == null) {
                             // happens for many fonts (Ubuntu font set),
                             // stray entries in the kerning table?? 
-                            log.warn("Unicode index (1) not found for glyph " + i);
+                            log.debug("Ignoring kerning pair because no Unicode index was"
+                                    + " found for the first glyph " + i);
                         } else if (u2 == null) {
-                            log.warn("Unicode index (2) not found for glyph " + i);
+                            log.debug("Ignoring kerning pair because Unicode index was"
+                                    + " found for the second glyph " + i);
                         } else {
                             Map adjTab = (Map)kerningTab.get(iObj);
                             if (adjTab == null) {
@@ -1439,8 +1461,8 @@ public class TTFFile {
                 // Reset names
                 notice = "";
                 fullName = "";
-                familyName = "";
-                fontName = "";
+                familyNames.clear();
+                postScriptName = "";
                 subFamilyName = "";
             }
 
@@ -1470,9 +1492,9 @@ public class TTFFile {
      * Dumps a few informational values to System.out.
      */
     public void printStuff() {
-        System.out.println("Font name:   " + fontName);
+        System.out.println("Font name:   " + postScriptName);
         System.out.println("Full name:   " + fullName);
-        System.out.println("Family name: " + familyName);
+        System.out.println("Family name: " + familyNames);
         System.out.println("Subfamily name: " + subFamilyName);
         System.out.println("Notice:      " + notice);
         System.out.println("xHeight:     " + (int)convertTTFUnit2PDFUnit(xHeight));
