@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PFMFile.java 540042 2007-05-21 06:57:03Z jeremias $ */
+/* $Id: PFMFile.java 628280 2008-02-16 12:28:48Z jeremias $ */
 
 package org.apache.fop.fonts.type1;
 
@@ -29,9 +29,7 @@ import org.apache.commons.io.IOUtils;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
-
-// FOP
-import org.apache.fop.fonts.Glyphs;
+import org.apache.xmlgraphics.fonts.Glyphs;
 
 /**
  * This class represents a PFM file (or parts of it) as a Java object.
@@ -190,8 +188,8 @@ public class PFMFile {
         int i = inStream.readShort();
 
 
-        if (log.isDebugEnabled()) {
-            log.debug(i + " kerning pairs");
+        if (log.isTraceEnabled()) {
+            log.trace(i + " kerning pairs");
         }
         while (i > 0) {
             int g1 = (int)inStream.readByte();
@@ -204,11 +202,11 @@ public class PFMFile {
                 adj = -(0x10000 - adj);
             }
 
-            if (log.isDebugEnabled()) {
-                log.debug("Char no: (" + g1 + ", " + g2 + ") kern: " + adj);
+            if (log.isTraceEnabled()) {
+                log.trace("Char no: (" + g1 + ", " + g2 + ") kern: " + adj);
                 final String glyph1 = Glyphs.TEX8R_GLYPH_NAMES[g1];
                 final String glyph2 = Glyphs.TEX8R_GLYPH_NAMES[g2];
-                log.debug("glyphs: " + glyph1 + ", " + glyph2);
+                log.trace("glyphs: " + glyph1 + ", " + glyph2);
             }
 
             Map adjTab = (Map)kerningTab.get(new Integer(g1));
@@ -236,7 +234,7 @@ public class PFMFile {
         etmCapHeight = inStream.readShort();
         etmXHeight = inStream.readShort();
         etmLowerCaseAscent = inStream.readShort();
-        etmLowerCaseDescent = inStream.readShort();
+        etmLowerCaseDescent = -(inStream.readShort());
         //Ignore the rest of the values
     }
 
@@ -300,14 +298,24 @@ public class PFMFile {
      * @return The name of the charset.
      */
     public String getCharSetName() {
+        //TODO Had to remove the detection for Expert(Subset) encoding. The PFM is not suitable
+        //for detecting these character sets. We have to parse the AFM for that.
         switch (dfCharSet) {
         case 0:
-            return "WinAnsi";
+            return "WinAnsi"; // AKA ISOAdobe
+        case 2:
+            if ("Symbol".equals(getPostscriptName())) {
+                return "Symbol";
+            }
+            break;
         case 128:
             return "Shift-JIS (Japanese)";
         default:
-            return "Unknown (" + dfCharSet + ", 0x" + Integer.toHexString(dfCharSet) + ")";
+            log.warn("Unknown charset detected (" + dfCharSet
+                    + ", 0x" + Integer.toHexString(dfCharSet)
+                    + "). Trying fallback to WinAnsi.");
         }
+        return "WinAnsi"; 
     }
 
     /**
@@ -391,12 +399,21 @@ public class PFMFile {
         } else {
             bbox[0] = -100;
         }
-        bbox[1] = -(getLowerCaseDescent() + 5);
+        bbox[1] = getLowerCaseDescent() - 5;
         bbox[2] = dfMaxWidth + 10;
         bbox[3] = getLowerCaseAscent() + 5;
         return bbox;
     }
 
+    /**
+     * Indicates whether the font is non-symbolic (Font uses the Adobe standard Latin character 
+     * set or a subset of it).
+     * @return true if the font is non-symbolic
+     */
+    public boolean isNonSymbolic() {
+        return (dfCharSet != 2); //!= Symbol fonts
+    }
+    
     /**
      * Returns the characteristics flags for the font as
      * needed for a PDF font descriptor (See PDF specs).
@@ -406,19 +423,22 @@ public class PFMFile {
     public int getFlags() {
         int flags = 0;
         if (!getIsProportional()) {
-            flags |= 1;
+            flags |= 1; //bit 1: FixedPitch
         }
-        if ((dfPitchAndFamily & 16) == 16) {
-            flags |= 2;
+        if (isNonSymbolic()) {
+            flags |= 32; //bit 6: Nonsymbolic
+        } else {
+            flags |= 4; //bit 3: Symbolic
         }
-        if ((dfPitchAndFamily & 64) == 64) {
-            flags |= 4;
+        //int serif = dfPitchAndFamily & 0xFFFE;
+        if ((dfPitchAndFamily & 16) != 0) {
+            flags |= 2; //bit 2: Serif
         }
-        if (dfCharSet == 0) {
-            flags |= 6;
+        if ((dfPitchAndFamily & 64) != 0) {
+            flags |= 8; //bit 4: Script
         }
         if (dfItalic != 0) {
-            flags |= 7;
+            flags |= 64; //bit 7: Italic
         }
         return flags;
     }
