@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AreaAdditionUtil.java 530724 2007-04-20 10:00:05Z vhennebert $ */
+/* $Id: AreaAdditionUtil.java 635508 2008-03-10 10:06:37Z jeremias $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -57,7 +57,7 @@ public class AreaAdditionUtil {
         LayoutManager lastLM = null;
         Position firstPos = null;
         Position lastPos = null;
-
+        
         // "unwrap" the NonLeafPositions stored in parentIter
         // and put them in a new list; 
         LinkedList positionList = new LinkedList();
@@ -94,12 +94,15 @@ public class AreaAdditionUtil {
             //doesn't give us that info.
         }
         
-        if (bslm != null && bslm.markers != null) {
-            bslm.getCurrentPV().addMarkers(bslm.markers, true, 
-                    bslm.isFirst(firstPos), bslm.isLast(lastPos));
+        if (bslm != null) {
+            bslm.addMarkersToPage(
+                    true, 
+                    bslm.isFirst(firstPos), 
+                    bslm.isLast(lastPos));
         }
 
         StackingIter childPosIter = new StackingIter(positionList.listIterator());
+
         while ((childLM = childPosIter.getNextChildLM()) != null) {
             // TODO vh: the test above might be problematic in some cases. See comment in
             // the TableCellLM.getNextKnuthElements method
@@ -114,13 +117,17 @@ public class AreaAdditionUtil {
             // set space after for each LM, in order to implement
             // display-align = distribute
             lc.setSpaceAfter(layoutContext.getSpaceAfter());
-            lc.setStackLimit(layoutContext.getStackLimit());
+            lc.setStackLimitsFrom(layoutContext);
             childLM.addAreas(childPosIter, lc);
         }
-        if (bslm != null && bslm.markers != null) {
-            bslm.getCurrentPV().addMarkers(bslm.markers, false, 
-                    bslm.isFirst(firstPos), bslm.isLast(lastPos));
+        
+        if (bslm != null) {
+            bslm.addMarkersToPage(
+                    false, 
+                    bslm.isFirst(firstPos), 
+                    bslm.isLast(lastPos));
         }
+
         
     }
     
