@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BlockLayoutManager.java 453310 2006-10-05 18:44:15Z spepping $ */
+/* $Id: BlockLayoutManager.java 635508 2008-03-10 10:06:37Z jeremias $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -25,11 +25,14 @@ import java.util.ListIterator;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.area.Area;
 import org.apache.fop.area.Block;
 import org.apache.fop.area.LineArea;
 import org.apache.fop.datatypes.Length;
 import org.apache.fop.fonts.Font;
+import org.apache.fop.fonts.FontInfo;
+import org.apache.fop.fonts.FontTriplet;
 import org.apache.fop.layoutmgr.inline.InlineLayoutManager;
 import org.apache.fop.layoutmgr.inline.InlineLevelLayoutManager;
 import org.apache.fop.layoutmgr.inline.LineLayoutManager;
@@ -55,7 +58,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     private int lead = 12000;
     private Length lineHeight;
     private int follow = 2000;
-    private int middleShift = 0;
+    //private int middleShift = 0;
     
     private boolean discardBorderBefore;
     private boolean discardBorderAfter;
@@ -78,12 +81,12 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
 
     public void initialize() {
         super.initialize();
-        Font fs = getBlockFO().getCommonFont().getFontState(
-                  getBlockFO().getFOEventHandler().getFontInfo(), this);
-        
-        lead = fs.getAscender();
-        follow = -fs.getDescender();
-        middleShift = -fs.getXHeight() / 2;
+        FontInfo fi = getBlockFO().getFOEventHandler().getFontInfo();
+        FontTriplet[] fontkeys = getBlockFO().getCommonFont().getFontState(fi);
+        Font initFont = fi.getFontInstance(fontkeys[0], getBlockFO().getCommonFont().fontSize.getValue(this));
+        lead = initFont.getAscender();
+        follow = -initFont.getDescender();
+        //middleShift = -fs.getXHeight() / 2;
         lineHeight = getBlockFO().getLineHeight().getOptimum(this).getLength();
         startIndent = getBlockFO().getCommonMarginBlock().startIndent.getValue(this);
         endIndent = getBlockFO().getCommonMarginBlock().endIndent.getValue(this); 
@@ -107,7 +110,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
         }
     }
 
-    /** @see org.apache.fop.layoutmgr.BlockStackingLayoutManager */
+    /** {@inheritDoc} */
     public LinkedList getNextKnuthElements(LayoutContext context, int alignment) {
         resetSpaces(); 
         return super.getNextKnuthElements(context, alignment);
@@ -160,7 +163,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     }
 
     /**
-     * @see org.apache.fop.layoutmgr.LayoutManager#createNextChildLMs
+     * {@inheritDoc}
      */
     public boolean createNextChildLMs(int pos) {
 
@@ -204,7 +207,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     }
 
     /**
-     * @see org.apache.fop.layoutmgr.BlockLevelLayoutManager#mustKeepTogether()
+     * {@inheritDoc}
      */
     public boolean mustKeepTogether() {
         // TODO Keeps will have to be more sophisticated sooner or later
@@ -219,7 +222,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     }
 
     /**
-     * @see org.apache.fop.layoutmgr.BlockLevelLayoutManager#mustKeepWithPrevious()
+     * {@inheritDoc}
      */
     public boolean mustKeepWithPrevious() {
         return !getBlockFO().getKeepWithPrevious().getWithinPage().isAuto()
@@ -227,7 +230,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     }
 
     /**
-     * @see org.apache.fop.layoutmgr.BlockLevelLayoutManager#mustKeepWithNext()
+     * {@inheritDoc}
      */
     public boolean mustKeepWithNext() {
         return !getBlockFO().getKeepWithNext().getWithinPage().isAuto()
@@ -235,7 +238,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     }
 
     /**
-     * @see org.apache.fop.layoutmgr.LayoutManager#addAreas(org.apache.fop.layoutmgr.PositionIterator, org.apache.fop.layoutmgr.LayoutContext)
+     * {@inheritDoc} 
      */
     public void addAreas(PositionIterator parentIter,
             LayoutContext layoutContext) {
@@ -307,9 +310,8 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
         }
 
         getPSLM().addIDToPage(getBlockFO().getId());
-        if (markers != null) {
-            getCurrentPV().addMarkers(markers, true, isFirst(firstPos), isLast(lastPos));
-        }
+        
+        addMarkersToPage(true, isFirst(firstPos), isLast(lastPos));
 
         if (bpUnit == 0) {
             // the Positions in positionList were inside the elements
@@ -388,14 +390,12 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
             // set last area flag
             lc.setFlags(LayoutContext.LAST_AREA,
                     (layoutContext.isLastArea() && childLM == lastLM));
-            lc.setStackLimit(layoutContext.getStackLimit());
+            lc.setStackLimitBP(layoutContext.getStackLimitBP());
             // Add the line areas to Area
             childLM.addAreas(childPosIter, lc);
         }
 
-        if (markers != null) {
-            getCurrentPV().addMarkers(markers, false, isFirst(firstPos), isLast(lastPos));
-        }
+        addMarkersToPage(false, isFirst(firstPos), isLast(lastPos));
 
         TraitSetter.addSpaceBeforeAfter(curBlockArea, layoutContext.getSpaceAdjust(), 
                 effSpaceBefore, effSpaceAfter);
@@ -452,7 +452,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     }
 
     /**
-     * @see org.apache.fop.layoutmgr.LayoutManager#addChildArea(Area)
+     * {@inheritDoc}
      */
     public void addChildArea(Area childArea) {
         if (curBlockArea != null) {
@@ -466,7 +466,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
 
     /**
      * Force current area to be added to parent area.
-     * @see org.apache.fop.layoutmgr.BlockStackingLayoutManager#flush()
+     * {@inheritDoc}
      */
     protected void flush() {
         if (curBlockArea != null) {
@@ -477,19 +477,6 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
         }
     }
 
-    /**
-     * @see org.apache.fop.layoutmgr.LayoutManager#resetPosition(org.apache.fop.layoutmgr.Position)
-     */
-    public void resetPosition(Position resetPos) {
-        if (resetPos == null) {
-            reset(null);
-            childBreaks.clear();
-        } else {
-            //reset(resetPos);
-            LayoutManager lm = resetPos.getLM();
-        }
-    }
-
     /**
      * convenience method that returns the Block node
      * @return the block node
@@ -524,13 +511,13 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
     }
    
     /**
-     * @see org.apache.fop.layoutmgr.LayoutManager#getGeneratesBlockArea
+     * {@inheritDoc}
      */
     public boolean getGeneratesBlockArea() {
         return true;
     }
 
-    /** @see org.apache.fop.layoutmgr.ConditionalElementListener */
+    /** {@inheritDoc} */
     public void notifySpace(RelSide side, MinOptMax effectiveLength) {
         if (RelSide.BEFORE == side) {
             if (log.isDebugEnabled()) {
@@ -547,7 +534,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
         }
     }
 
-    /** @see org.apache.fop.layoutmgr.ConditionalElementListener */
+    /** {@inheritDoc} */
     public void notifyBorder(RelSide side, MinOptMax effectiveLength) {
         if (effectiveLength == null) {
             if (RelSide.BEFORE == side) {
@@ -561,7 +548,7 @@ public class BlockLayoutManager extends BlockStackingLayoutManager
         }
     }
 
-    /** @see org.apache.fop.layoutmgr.ConditionalElementListener */
+    /** {@inheritDoc} */
     public void notifyPadding(RelSide side, MinOptMax effectiveLength) {
         if (effectiveLength == null) {
             if (RelSide.BEFORE == side) {
