@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BreakElement.java 554689 2007-07-09 15:50:21Z vhennebert $ */
+/* $Id: BreakElement.java 620565 2008-02-11 18:18:13Z vhennebert $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -46,10 +46,13 @@ public class BreakElement extends UnresolvedListElement {
     
     /**
      * Constructor for hard breaks.
+     * 
      * @param position the Position instance needed by the addAreas stage of the LMs.
      * @param penaltyWidth the penalty width
      * @param penaltyValue the penalty value for the penalty element to be constructed
-     * @param breakClass the break class of this penalty (one of the break-* constants)
+     * @param breakClass the break class of this penalty (one of {@link Constants#EN_AUTO},
+     * {@link Constants#EN_COLUMN}, {@link Constants#EN_PAGE},
+     * {@link Constants#EN_EVEN_PAGE}, {@link Constants#EN_ODD_PAGE})
      * @param context the layout context which contains the pending conditional elements
      */
     public BreakElement(Position position, int penaltyWidth, int penaltyValue, 
@@ -62,12 +65,12 @@ public class BreakElement extends UnresolvedListElement {
         this.pendingAfterMarks = context.getPendingAfterMarks();
     }
     
-    /** @see org.apache.fop.layoutmgr.UnresolvedListElement#isConditional() */
+    /** {@inheritDoc} */
     public boolean isConditional() {
         return false; //Does not really apply here
     }
 
-    /** @see org.apache.fop.layoutmgr.ListElement#isPenalty() */
+    /** {@inheritDoc} */
     /*
     public boolean isPenalty() {
         return true; //not entirely true but a BreakElement will generate a penalty later
@@ -91,19 +94,28 @@ public class BreakElement extends UnresolvedListElement {
         this.penaltyValue = p;
     }
     
-    /** @see org.apache.fop.layoutmgr.ListElement#isForcedBreak() */
+    /** {@inheritDoc} */
     public boolean isForcedBreak() {
         return penaltyValue == -KnuthElement.INFINITE;
     }
     
-    /** @return the break class of this penalty (one of the break-* constants) */
+    /**
+     * Returns the break class of this penalty.
+     * 
+     * @return one of {@link Constants#EN_AUTO}, {@link Constants#EN_COLUMN},
+     * {@link Constants#EN_PAGE}, {@link Constants#EN_EVEN_PAGE},
+     * {@link Constants#EN_ODD_PAGE}
+     */
     public int getBreakClass() {
         return breakClass;
     }
     
     /**
      * Sets the break class.
-     * @param breakClass the new break class
+     * 
+     * @param breakClass one of {@link Constants#EN_AUTO}, {@link Constants#EN_COLUMN},
+     * {@link Constants#EN_PAGE}, {@link Constants#EN_EVEN_PAGE},
+     * {@link Constants#EN_ODD_PAGE}
      */
     public void setBreakClass(int breakClass) {
         this.breakClass = breakClass;
@@ -119,7 +131,17 @@ public class BreakElement extends UnresolvedListElement {
         return this.pendingAfterMarks;
     }
     
-    /** @see java.lang.Object#toString() */
+    /**
+     * Clears all pending marks associated with this break element. This is used in break
+     * cases where we only know very late if the break is actually after all the content
+     * of an FO has been generated.
+     */
+    public void clearPendingMarks() {
+        this.pendingBeforeMarks = null;
+        this.pendingAfterMarks = null;
+    }
+    
+    /** {@inheritDoc} */
     public String toString() {
         StringBuffer sb = new StringBuffer();
         sb.append("BreakPossibility[p:");
