@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BreakingAlgorithm.java 501453 2007-01-30 16:37:11Z lfurini $ */
+/* $Id: BreakingAlgorithm.java 627367 2008-02-13 12:03:30Z maxberger $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -151,7 +151,7 @@ public abstract class BreakingAlgorithm {
 
     protected BestRecords best;
 
-    /** @see #isPartOverflowRecoveryActivated() */
+    /** {@inheritDoc} */
     private boolean partOverflowRecoveryActivated = true;
     private KnuthNode lastRecovered;
 
@@ -397,9 +397,10 @@ public abstract class BreakingAlgorithm {
     }
 
     /** @see #findBreakingPoints(KnuthSequence, int, double, boolean, int) */
-    public int findBreakingPoints(KnuthSequence par, /*int lineWidth,*/
-            double threshold, boolean force,
-            int allowedBreaks) {
+    public int findBreakingPoints(KnuthSequence par, 
+                                  double threshold, 
+                                  boolean force,
+                                  int allowedBreaks) {
         return findBreakingPoints(par, 0, threshold, force, allowedBreaks);
     }
     
@@ -412,7 +413,6 @@ public abstract class BreakingAlgorithm {
      * @param allowedBreaks one of ONLY_FORCED_BREAKS, NO_FLAGGED_PENALTIES, ALL_BREAKS
      */
     public int findBreakingPoints(KnuthSequence par, int startIndex,
-                                  /*int lineWidth,*/
                                   double threshold, boolean force,
                                   int allowedBreaks) {
         this.par = par;
@@ -634,7 +634,7 @@ public abstract class BreakingAlgorithm {
 
     /** Creates a new active node for a break from the best active node of the given
      * fitness class to the element at the given position.
-     * @see #createNode(int, int, int, int, int, int, double, int, int, int, double, KnuthNode)
+     * @see #createNode(int, int, int, int, int, int, double, int, int, int, double, org.apache.fop.layoutmgr.BreakingAlgorithm.KnuthNode)
      * @see BreakingAlgorithm.BestRecords
      */
     protected KnuthNode createNode(int position, int line, int fitness,
