@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: LayoutManagerMapping.java 525526 2007-04-04 15:05:30Z jeremias $ */
+/* $Id: LayoutManagerMapping.java 627367 2008-02-13 12:03:30Z maxberger $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -32,6 +32,7 @@ import org.apache.fop.fo.FOElementMapping;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.FOText;
 import org.apache.fop.fo.FObjMixed;
+import org.apache.fop.fo.extensions.ExternalDocument;
 import org.apache.fop.fo.flow.BasicLink;
 import org.apache.fop.fo.flow.BidiOverride;
 import org.apache.fop.fo.flow.Block;
@@ -50,14 +51,14 @@ import org.apache.fop.fo.flow.PageNumber;
 import org.apache.fop.fo.flow.PageNumberCitation;
 import org.apache.fop.fo.flow.PageNumberCitationLast;
 import org.apache.fop.fo.flow.RetrieveMarker;
-import org.apache.fop.fo.flow.Table;
-import org.apache.fop.fo.flow.TableBody;
-import org.apache.fop.fo.flow.TableCell;
-import org.apache.fop.fo.flow.TableColumn;
-import org.apache.fop.fo.flow.TableFooter;
-import org.apache.fop.fo.flow.TableHeader;
-import org.apache.fop.fo.flow.TableRow;
 import org.apache.fop.fo.flow.Wrapper;
+import org.apache.fop.fo.flow.table.Table;
+import org.apache.fop.fo.flow.table.TableBody;
+import org.apache.fop.fo.flow.table.TableCell;
+import org.apache.fop.fo.flow.table.TableColumn;
+import org.apache.fop.fo.flow.table.TableFooter;
+import org.apache.fop.fo.flow.table.TableHeader;
+import org.apache.fop.fo.flow.table.TableRow;
 import org.apache.fop.fo.pagination.Flow;
 import org.apache.fop.fo.pagination.PageSequence;
 import org.apache.fop.fo.pagination.SideRegion;
@@ -86,8 +87,6 @@ import org.apache.fop.layoutmgr.list.ListItemLayoutManager;
 import org.apache.fop.layoutmgr.table.TableLayoutManager;
 import org.apache.fop.util.CharUtilities;
 
-import org.apache.fop.fo.extensions.destination.Destination;
-
 /**
  * The default LayoutManager maker class
  */
@@ -144,7 +143,7 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
     }
 
     /**
-     * @see org.apache.fop.layoutmgr.LayoutManagerMaker#makeLayoutManagers(FONode, List)
+     * {@inheritDoc} 
      */
     public void makeLayoutManagers(FONode node, List lms) {
         Maker maker = (Maker) makers.get(node.getClass());
@@ -162,7 +161,7 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
     }
 
     /**
-     * @see org.apache.fop.layoutmgr.LayoutManagerMaker#makeLayoutManager(FONode)
+     * {@inheritDoc}
      */
     public LayoutManager makeLayoutManager(FONode node) {
         List lms = new ArrayList();
@@ -185,7 +184,7 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
     }
 
     /*
-     * @see org.apache.fop.layoutmgr.LayoutManagerMaker#makeFlowLayoutManager(PageSequenceLayoutManager, Flow)
+     * {@inheritDoc} 
      */
     public FlowLayoutManager makeFlowLayoutManager(
             PageSequenceLayoutManager pslm, Flow flow) {
@@ -193,7 +192,7 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
     }
 
     /*
-     * @see org.apache.fop.layoutmgr.LayoutManagerMaker#makeContentLayoutManager(PageSequenceLayoutManager, Title)
+     * {@inheritDoc} 
      */
     public ContentLayoutManager makeContentLayoutManager(PageSequenceLayoutManager pslm,
                                                          Title title) {
@@ -201,7 +200,7 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
     }
     
     /*
-     * @see org.apache.fop.layoutmgr.LayoutManagerMaker#makeStaticContentLayoutManager(PageSequenceLayoutManager, StaticContent, Region)
+     * {@inheritDoc} 
      */
     public StaticContentLayoutManager makeStaticContentLayoutManager(
             PageSequenceLayoutManager pslm, StaticContent sc, SideRegion reg) {
@@ -209,7 +208,7 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
     }
     
     /*
-     * @see org.apache.fop.layoutmgr.LayoutManagerMaker#makeStaticContentLayoutManager(PageSequenceLayoutManager, StaticContent, Block)
+     * {@inheritDoc} 
      */
     public StaticContentLayoutManager makeStaticContentLayoutManager(
         PageSequenceLayoutManager pslm, StaticContent sc, org.apache.fop.area.Block block) {
@@ -272,9 +271,7 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
 
     public static class InlineLayoutManagerMaker extends Maker {
          public void make(FONode node, List lms) {
-             if (node.getChildNodes() != null) {
-                 lms.add(new InlineLayoutManager((InlineLevel) node));
-             }
+             lms.add(new InlineLayoutManager((InlineLevel) node));
          }
     }
 
@@ -409,4 +406,9 @@ public class LayoutManagerMapping implements LayoutManagerMaker {
         }       
     }
 
+    public ExternalDocumentLayoutManager makeExternalDocumentLayoutManager(
+        AreaTreeHandler ath, ExternalDocument ed) {
+        return new ExternalDocumentLayoutManager(ath, ed);
+    }
+
 }
