@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PageBreakingAlgorithm.java 554094 2007-07-07 00:04:25Z adelmelle $ */
+/* $Id: PageBreakingAlgorithm.java 633557 2008-03-04 17:30:40Z jeremias $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -25,11 +25,11 @@ import java.util.ListIterator;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.FObj;
 import org.apache.fop.layoutmgr.AbstractBreaker.PageBreakPosition;
-
 import org.apache.fop.traits.MinOptMax;
 
 class PageBreakingAlgorithm extends BreakingAlgorithm {
@@ -364,7 +364,13 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
         } else {
             // there are no footnotes
         }
-        return getLineWidth(activeNode.line) - actualWidth;
+        int diff = getLineWidth(activeNode.line) - actualWidth;
+        if (autoHeight && diff < 0) {
+            //getLineWidth() for auto-height parts return 0 so the diff will be negative
+            return 0; //...but we don't want to shrink in this case. Stick to optimum.
+        } else {
+            return diff;
+        }
     }
 
     /** Checks whether footnotes from preceding pages may be deferred to the page after
@@ -641,10 +647,14 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
                 demerits += (footnotesList.size() - 1 - footnoteListIndex) 
                                 * deferredFootnoteDemerits;
             }
-            if (footnoteElementIndex 
-                    < ((LinkedList) footnotesList.get(footnoteListIndex)).size() - 1) {
-                // add demerits for the footnote split between pages
-                demerits += splitFootnoteDemerits;
+            if (footnoteListIndex < footnotesList.size()) {
+                if (footnoteElementIndex 
+                        < ((LinkedList) footnotesList.get(footnoteListIndex)).size() - 1) {
+                    // add demerits for the footnote split between pages
+                    demerits += splitFootnoteDemerits;
+                }
+            } else {
+                //TODO Why can this happen in the first place? Does anybody know? See #44160
             }
         }
         demerits += activeNode.totalDemerits;
@@ -730,6 +740,20 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
         pageBreaks.addFirst(pageBreak);
     }
     
+    /**
+     * Removes all page breaks from the result list. This is used by block-containers and
+     * static-content when it is only desired to know where there is an overflow but later the
+     * whole content should be painted as one part.
+     */
+    public void removeAllPageBreaks() {
+        if (pageBreaks == null) {
+            return;
+        }
+        while (pageBreaks.size() > 1) {
+            pageBreaks.removeFirst();
+        }
+    }
+    
     private int getPartCount() {
         if (pageBreaks == null) {
             return 0;
@@ -844,7 +868,7 @@ class PageBreakingAlgorithm extends BreakingAlgorithm {
         return topLevelLM.getFObj();
     }
     
-    /** @see org.apache.fop.layoutmgr.BreakingAlgorithm#getLineWidth(int) */
+    /** {@inheritDoc} */
     protected int getLineWidth(int line) {
         int bpd;
         if (pageProvider != null) {
