@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: SpaceResolver.java 521185 2007-03-22 10:13:25Z vhennebert $ */
+/* $Id: SpaceResolver.java 598558 2007-11-27 08:36:23Z jeremias $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -25,6 +25,7 @@ import java.util.ListIterator;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.traits.MinOptMax;
 
 /**
@@ -441,7 +442,6 @@ public class SpaceResolver {
         glue2shrink -= glue3.opt - glue3.min;
         
         boolean hasPrecedingNonBlock = false;
-        boolean forcedBreak = false;
         if (log.isDebugEnabled()) {
             log.debug("noBreakLength=" + noBreakLength 
                     + ", glue1=" + glue1 
@@ -449,11 +449,16 @@ public class SpaceResolver {
                     + ", glue3=" + glue3);
         }
         if (breakPoss != null) {
+            boolean forcedBreak = breakPoss.isForcedBreak();
             if (glue1.isNonZero()) {
                 iter.add(new KnuthPenalty(0, KnuthPenalty.INFINITE, 
                         false, (Position)null, true));
                 iter.add(new KnuthGlue(glue1.opt, glue1.max - glue1.opt, glue1.opt - glue1.min, 
                         (Position)null, true));
+                if (forcedBreak) {
+                    //Otherwise, the preceding penalty and glue will be cut off
+                    iter.add(new KnuthBox(0, (Position)null, true));
+                }
             }
             iter.add(new KnuthPenalty(breakPoss.getPenaltyWidth(), breakPoss.getPenaltyValue(), 
                     false, breakPoss.getBreakClass(), 
@@ -544,7 +549,7 @@ public class SpaceResolver {
             }
         }
         
-        /** @see java.lang.Object#toString() */
+        /** {@inheritDoc} */
         public String toString() {
             StringBuffer sb = new StringBuffer();
             sb.append("SpaceHandlingBreakPosition(");
@@ -597,7 +602,7 @@ public class SpaceResolver {
             }
         }
         
-        /** @see java.lang.Object#toString() */
+        /** {@inheritDoc} */
         public String toString() {
             StringBuffer sb = new StringBuffer();
             sb.append("SpaceHandlingPosition");
