@@ -15,16 +15,17 @@
  * limitations under the License.
  */
 
-/* $Id: InlineLayoutManager.java 453310 2006-10-05 18:44:15Z spepping $ */
+/* $Id: InlineLayoutManager.java 635508 2008-03-10 10:06:37Z jeremias $ */
 
 package org.apache.fop.layoutmgr.inline;
 
-import java.util.ListIterator;
 import java.util.LinkedList;
 import java.util.List;
+import java.util.ListIterator;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.area.Area;
 import org.apache.fop.area.inline.InlineArea;
 import org.apache.fop.area.inline.InlineBlockParent;
@@ -38,18 +39,21 @@ import org.apache.fop.fo.properties.CommonBorderPaddingBackground;
 import org.apache.fop.fo.properties.CommonMarginInline;
 import org.apache.fop.fo.properties.SpaceProperty;
 import org.apache.fop.fonts.Font;
+import org.apache.fop.fonts.FontInfo;
+import org.apache.fop.fonts.FontTriplet;
 import org.apache.fop.layoutmgr.BlockKnuthSequence;
 import org.apache.fop.layoutmgr.BlockLevelLayoutManager;
 import org.apache.fop.layoutmgr.BreakElement;
+import org.apache.fop.layoutmgr.InlineKnuthSequence;
 import org.apache.fop.layoutmgr.KnuthBox;
 import org.apache.fop.layoutmgr.KnuthSequence;
 import org.apache.fop.layoutmgr.LayoutContext;
-import org.apache.fop.layoutmgr.NonLeafPosition;
-import org.apache.fop.layoutmgr.SpaceSpecifier;
-import org.apache.fop.layoutmgr.TraitSetter;
 import org.apache.fop.layoutmgr.LayoutManager;
+import org.apache.fop.layoutmgr.NonLeafPosition;
 import org.apache.fop.layoutmgr.Position;
 import org.apache.fop.layoutmgr.PositionIterator;
+import org.apache.fop.layoutmgr.SpaceSpecifier;
+import org.apache.fop.layoutmgr.TraitSetter;
 import org.apache.fop.traits.MinOptMax;
 import org.apache.fop.traits.SpaceVal;
 
@@ -86,6 +90,8 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
     protected int dominantBaseline;
     /** The line height property */
     protected SpaceProperty lineHeight;
+    /** The keep-together property */
+    //private KeepProperty keepTogether;
     
     private AlignmentContext alignmentContext = null;
 
@@ -103,13 +109,15 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
     }
     
     private Inline getInlineFO() {
-        return (Inline)fobj;
+        return (Inline) fobj;
     }
     
-    /** @see LayoutManager#initialize */
+    /** {@inheritDoc} */
     public void initialize() {
         int padding = 0;
-        font = fobj.getCommonFont().getFontState(fobj.getFOEventHandler().getFontInfo(), this);
+        FontInfo fi = fobj.getFOEventHandler().getFontInfo();
+        FontTriplet[] fontkeys = fobj.getCommonFont().getFontState(fi);
+        font = fi.getFontInstance(fontkeys[0], fobj.getCommonFont().fontSize.getValue(this));
         lineHeight = fobj.getLineHeight();
         borderProps = fobj.getCommonBorderPaddingBackground();
         inlineProps = fobj.getCommonMarginInline();
@@ -136,7 +144,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
 
     }
 
-    /** @see InlineStackingLayoutManager#getExtraIPD(boolean, boolean) */
+    /** {@inheritDoc} */
     protected MinOptMax getExtraIPD(boolean isNotFirst, boolean isNotLast) {
         int borderAndPadding = 0;
         if (borderProps != null) {
@@ -153,7 +161,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
     }
 
 
-    /** @see InlineStackingLayoutManager#hasLeadingFence(boolean) */
+    /** {@inheritDoc} */
     protected boolean hasLeadingFence(boolean isNotFirst) {
         return borderProps != null
             && (borderProps.getPadding(CommonBorderPaddingBackground.START, isNotFirst, this) > 0
@@ -161,7 +169,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
                );
     }
 
-    /** @see InlineStackingLayoutManager#hasTrailingFence(boolean) */
+    /** {@inheritDoc} */
     protected boolean hasTrailingFence(boolean isNotLast) {
         return borderProps != null
             && (borderProps.getPadding(CommonBorderPaddingBackground.END, isNotLast, this) > 0
@@ -169,16 +177,21 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
                );
     }
 
-    /** @see InlineStackingLayoutManager#getSpaceStart */
+    /** {@inheritDoc} */
     protected SpaceProperty getSpaceStart() {
         return inlineProps != null ? inlineProps.spaceStart : null;
     }
-    /** @see InlineStackingLayoutManager#getSpaceEnd */
+    /** {@inheritDoc} */
     protected SpaceProperty getSpaceEnd() {
         return inlineProps != null ? inlineProps.spaceEnd : null;
     }
     
-    /** @see org.apache.fop.layoutmgr.inline.InlineLayoutManager#createArea(boolean) */
+    /** 
+     * Create and initialize an <code>InlineArea</code> 
+     * 
+     * @param hasInlineParent   true if the parent is an inline 
+     * @return the area
+     */
     protected InlineArea createArea(boolean hasInlineParent) {
         InlineArea area;
         if (hasInlineParent) {
@@ -193,9 +206,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         return area;
     }
     
-    /**
-     * @see org.apache.fop.layoutmgr.inline.InlineStackingLayoutManager#setTraits(boolean, boolean)
-     */
+    /** {@inheritDoc} */
     protected void setTraits(boolean isNotFirst, boolean isNotLast) {
         if (borderProps != null) {
             // Add border and padding to current area and set flags (FIRST, LAST ...)
@@ -208,7 +219,6 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
     /**
      * @return true if this element must be kept together
      */
-    // TODO Use the keep-together property on Inline as well
     public boolean mustKeepTogether() {
         return mustKeepTogether(this.getParent());
     }
@@ -223,7 +233,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         }
     }
 
-    /** @see org.apache.fop.layoutmgr.LayoutManager */
+    /** {@inheritDoc} */
     public LinkedList getNextKnuthElements(LayoutContext context, int alignment) {
         LayoutManager curLM;
 
@@ -288,6 +298,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         }
         
         while ((curLM = (LayoutManager) getChildLM()) != null) {
+            
             if (!(curLM instanceof InlineLevelLayoutManager)) {
                 // A block LM
                 // Leave room for start/end border and padding
@@ -299,19 +310,19 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
                             - borderProps.getBorderEndWidth(hasNextChildLM()));
                 }
             }
+            
             // get KnuthElements from curLM
             returnedList = curLM.getNextKnuthElements(childLC, alignment);
             if (returnList.size() == 0 && childLC.isKeepWithPreviousPending()) {
                 childLC.setFlags(LayoutContext.KEEP_WITH_PREVIOUS_PENDING, false);
             }
-            if (returnedList == null) {
-                // curLM returned null because it finished;
+            if (returnedList == null
+                    || returnedList.size() == 0) {
+                // curLM returned null or an empty list, because it finished;
                 // just iterate once more to see if there is another child
                 continue;
             }
-            if (returnedList.size() == 0) {
-                continue;
-            }
+            
             if (curLM instanceof InlineLevelLayoutManager) {
                 context.setFlags(LayoutContext.KEEP_WITH_NEXT_PENDING, false);
                 // "wrap" the Position stored in each element of returnedList
@@ -369,6 +380,24 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
 
         setFinished(true);
         log.trace(trace);
+        
+        if (returnList.size() == 0) {
+            /*
+             * if the FO itself is empty, but has an id specified 
+             * or associated fo:markers, then we still need a dummy
+             * sequence to register its position in the area tree
+             */
+            if (fobj.hasId() || fobj.hasMarkers()) {
+                InlineKnuthSequence emptySeq = new InlineKnuthSequence();
+                emptySeq.add(new KnuthInlineBox(
+                                0, 
+                                alignmentContext, 
+                                notifyPos(getAuxiliaryPosition()), 
+                                true));
+                returnList.add(emptySeq);
+            }
+        }
+        
         return returnList.size() == 0 ? null : returnList;
     }
 
@@ -384,8 +413,6 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
     public void addAreas(PositionIterator parentIter,
                          LayoutContext context) {
         
-        Position lastPos = null;
-        
         addId();
 
         setChildContext(new LayoutContext(context)); // Store current value
@@ -410,7 +437,8 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         // layout context given to the other LMs.
         LinkedList positionList = new LinkedList();
         NonLeafPosition pos = null;
-        LayoutManager lastLM = null; // last child LM in this iterator
+        LayoutManager lastLM = null;// last child LM in this iterator
+        Position lastPos = null;
         while (parentIter.hasNext()) {
             pos = (NonLeafPosition) parentIter.next();
             if (pos != null && pos.getPosition() != null) {
@@ -423,6 +451,11 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
             lastLM = pos.getPosition().getLM();
         }*/
 
+        addMarkersToPage(
+                true, 
+                !areaCreated, 
+                lastPos == null || isLast(lastPos));
+        
         InlineArea parent = createArea(lastLM == null 
                                         || lastLM instanceof InlineLevelLayoutManager);
         parent.setBPD(alignmentContext.getHeight());
@@ -452,6 +485,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
             prevLM = childLM;
         }
 
+        
         /* If this LM has a trailing fence, resolve trailing space
          * specs from descendants.  Otherwise, propagate any trailing
          * space specs to the parent LM via the layout context.  If
@@ -460,6 +494,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
          * must be the last area for the current LM too.
          */
         boolean isLast = (getContext().isLastArea() && prevLM == lastChildLM);
+        
         if (hasTrailingFence(isLast)) {
             addSpace(getCurrentArea(),
                      getContext().getTrailingSpace().resolve(false),
@@ -479,11 +514,16 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         setTraits(areaCreated, lastPos == null || !isLast(lastPos));
         parentLM.addChildArea(getCurrentArea());
 
+        addMarkersToPage(
+                false, 
+                !areaCreated, 
+                lastPos == null || isLast(lastPos));
+        
         context.setFlags(LayoutContext.LAST_AREA, isLast);
         areaCreated = true;
     }
 
-    /** @see LayoutManager#addChildArea(Area) */
+    /** {@inheritDoc} */
     public void addChildArea(Area childArea) {
         Area parent = getCurrentArea();
         if (getContext().resolveLeadingSpace()) {
@@ -494,7 +534,7 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         parent.addChildArea(childArea);
     }
 
-    /** @see LayoutManager#getChangedKnuthElements(List, int) */
+    /** {@inheritDoc} */
     public LinkedList getChangedKnuthElements(List oldList, int alignment) {
         LinkedList returnedList = new LinkedList();
         addKnuthElementsForBorderPaddingStart(returnedList);
@@ -509,6 +549,14 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
      */
     protected void addKnuthElementsForBorderPaddingStart(List returnList) {
         //Border and Padding (start)
+        /**
+         * If the returnlist is a BlockKnuthSequence, the border and padding should be added
+         * to the first paragraph inside it, but it is too late to do that now.
+         * At least, avoid adding it to the bpd sequence.
+         */
+        if (returnList instanceof BlockKnuthSequence) {
+            return;
+        }
         CommonBorderPaddingBackground borderAndPadding = fobj.getCommonBorderPaddingBackground();
         if (borderAndPadding != null) {
             int ipStart = borderAndPadding.getBorderStartWidth(false)
@@ -525,6 +573,14 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
      */
     protected void addKnuthElementsForBorderPaddingEnd(List returnList) {
         //Border and Padding (after)
+        /**
+         * If the returnlist is a BlockKnuthSequence, the border and padding should be added
+         * to the last paragraph inside it, but it is too late to do that now.
+         * At least, avoid adding it to the bpd sequence.
+         */
+        if (returnList instanceof BlockKnuthSequence) {
+            return;
+        }
         CommonBorderPaddingBackground borderAndPadding = fobj.getCommonBorderPaddingBackground();
         if (borderAndPadding != null) {
             int ipEnd = borderAndPadding.getBorderEndWidth(false)
@@ -544,12 +600,9 @@ public class InlineLayoutManager extends InlineStackingLayoutManager {
         return this.auxiliaryPosition;
     }
     
-    /** @see org.apache.fop.layoutmgr.inline.LeafNodeLayoutManager#addId() */
+    /** {@inheritDoc} */
     protected void addId() {
-        if (fobj instanceof Inline) {
-            getPSLM().addIDToPage(getInlineFO().getId());
-        }
+        getPSLM().addIDToPage(fobj.getId());
     }
     
 }
-
