@@ -15,13 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: TextLayoutManager.java 563951 2007-08-08 17:37:33Z vhennebert $ */
+/* $Id: TextLayoutManager.java 604171 2007-12-14 11:32:51Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
 import java.util.ArrayList;
-import java.util.List;
 import java.util.LinkedList;
+import java.util.List;
 import java.util.ListIterator;
 
 import org.apache.commons.logging.Log;
@@ -31,6 +31,8 @@ import org.apache.fop.area.inline.TextArea;
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.FOText;
 import org.apache.fop.fonts.Font;
+import org.apache.fop.fonts.FontInfo;
+import org.apache.fop.fonts.FontTriplet;
 import org.apache.fop.layoutmgr.InlineKnuthSequence;
 import org.apache.fop.layoutmgr.KnuthBox;
 import org.apache.fop.layoutmgr.KnuthElement;
@@ -184,14 +186,16 @@ public class TextLayoutManager extends LeafNodeLayoutManager {
         vecAreaInfo = new java.util.ArrayList();
     }
     
-    /** @see org.apache.fop.layoutmgr.LayoutManager#initialize */
+    /** {@inheritDoc} */
     public void initialize() {
-        font = foText.getCommonFont().getFontState(foText.getFOEventHandler().getFontInfo(), this);
+        FontInfo fi = foText.getFOEventHandler().getFontInfo();
+        FontTriplet[] fontkeys = foText.getCommonFont().getFontState(fi);
+        font = fi.getFontInstance(fontkeys[0], foText.getCommonFont().fontSize.getValue(this));
         
         // With CID fonts, space isn't neccesary currentFontState.width(32)
         spaceCharIPD = font.getCharWidth(' ');
         // Use hyphenationChar property
-        hyphIPD = font.getCharWidth(foText.getCommonHyphenation().hyphenationCharacter);
+        hyphIPD = foText.getCommonHyphenation().getHyphIPD(font);
         
         SpaceVal ls = SpaceVal.makeLetterSpacing(foText.getLetterSpacing());
         halfLS = new SpaceVal(MinOptMax.multiply(ls.getSpace(), 0.5),
@@ -220,36 +224,6 @@ public class TextLayoutManager extends LeafNodeLayoutManager {
 
     }
 
-    /**
-     * Reset position for returning next BreakPossibility.
-     *
-     * @param prevPos the position to reset to
-     */
-    public void resetPosition(Position prevPos) {
-        if (prevPos != null) {
-            // ASSERT (prevPos.getLM() == this)
-            if (prevPos.getLM() != this) {
-                log.error("TextLayoutManager.resetPosition: "
-                          + "LM mismatch!!!");
-            }
-            LeafPosition tbp = (LeafPosition) prevPos;
-            AreaInfo ai = (AreaInfo) vecAreaInfo.get(tbp.getLeafPos());
-            if (ai.iBreakIndex != iNextStart) {
-                iNextStart = ai.iBreakIndex;
-                vecAreaInfo.ensureCapacity(tbp.getLeafPos() + 1);
-                // TODO: reset or recalculate total IPD = sum of all word IPD
-                // up to the break position
-                ipdTotal = ai.ipdArea;
-                setFinished(false);
-            }
-        } else {
-            // Reset to beginning!
-            vecAreaInfo.clear();
-            iNextStart = 0;
-            setFinished(false);
-        }
-    }
-
     // TODO: see if we can use normal getNextBreakPoss for this with
     // extra hyphenation information in LayoutContext
     private boolean getHyphenIPD(HyphContext hc, MinOptMax hyphIPD) {
@@ -504,7 +478,7 @@ public class TextLayoutManager extends LeafNodeLayoutManager {
                         && i == lastIndex 
                         && areaInfo.bHyphenated) {
                         // add the hyphenation character
-                        wordChars.append(foText.getCommonHyphenation().hyphenationCharacter);
+                        wordChars.append(foText.getCommonHyphenation().getHyphChar(font));
                     }
                     textArea.addWord(wordChars.toString(), 0, letterAdjust);
                     wordStartIndex = -1;
@@ -546,7 +520,7 @@ public class TextLayoutManager extends LeafNodeLayoutManager {
             || CharUtilities.isFixedWidthSpace(ch);
     }
     
-    /** @see org.apache.fop.layoutmgr.LayoutManager#getNextKnuthElements(LayoutContext, int) */
+    /** {@inheritDoc} */
     public LinkedList getNextKnuthElements(LayoutContext context, int alignment) {
         lineStartBAP = context.getLineStartBorderAndPaddingWidth();
         lineEndBAP = context.getLineEndBorderAndPaddingWidth();
@@ -796,7 +770,7 @@ public class TextLayoutManager extends LeafNodeLayoutManager {
         }
     }
 
-    /** @see InlineLevelLayoutManager#addALetterSpaceTo(List) */
+    /** {@inheritDoc} */
     public List addALetterSpaceTo(List oldList) {
         // old list contains only a box, or the sequence: box penalty glue box;
         // look at the Position stored in the first element in oldList
@@ -804,30 +778,34 @@ public class TextLayoutManager extends LeafNodeLayoutManager {
         ListIterator oldListIterator = oldList.listIterator();
         KnuthElement el = (KnuthElement)oldListIterator.next();
         LeafPosition pos = (LeafPosition) ((KnuthBox) el).getPosition();
-        AreaInfo ai = (AreaInfo) vecAreaInfo.get(pos.getLeafPos());
-        ai.iLScount++;
-        ai.ipdArea.add(letterSpaceIPD);
-        if (BREAK_CHARS.indexOf(textArray[iTempStart - 1]) >= 0) {
-            // the last character could be used as a line break
-            // append new elements to oldList
-            oldListIterator = oldList.listIterator(oldList.size());
-            oldListIterator.add(new KnuthPenalty(0, KnuthPenalty.FLAGGED_PENALTY, true,
-                                                 new LeafPosition(this, -1), false));
-            oldListIterator.add(new KnuthGlue(letterSpaceIPD.opt,
-                                       letterSpaceIPD.max - letterSpaceIPD.opt,
-                                       letterSpaceIPD.opt - letterSpaceIPD.min,
-                                       new LeafPosition(this, -1), false));
-        } else if (letterSpaceIPD.min == letterSpaceIPD.max) {
-            // constant letter space: replace the box
-            oldListIterator.set(new KnuthInlineBox(ai.ipdArea.opt, alignmentContext, pos, false));
-        } else {
-            // adjustable letter space: replace the glue
-            oldListIterator.next(); // this would return the penalty element
-            oldListIterator.next(); // this would return the glue element
-            oldListIterator.set(new KnuthGlue(ai.iLScount * letterSpaceIPD.opt,
-                                              ai.iLScount * (letterSpaceIPD.max - letterSpaceIPD.opt),
-                                              ai.iLScount * (letterSpaceIPD.opt - letterSpaceIPD.min),
-                                              new LeafPosition(this, -1), true));
+        int idx = pos.getLeafPos();
+        //element could refer to '-1' position, for non-collapsed spaces (?)
+        if (idx > -1) {
+            AreaInfo ai = (AreaInfo) vecAreaInfo.get(idx);
+            ai.iLScount++;
+            ai.ipdArea.add(letterSpaceIPD);
+            if (BREAK_CHARS.indexOf(textArray[iTempStart - 1]) >= 0) {
+                // the last character could be used as a line break
+                // append new elements to oldList
+                oldListIterator = oldList.listIterator(oldList.size());
+                oldListIterator.add(new KnuthPenalty(0, KnuthPenalty.FLAGGED_PENALTY, true,
+                                                     new LeafPosition(this, -1), false));
+                oldListIterator.add(new KnuthGlue(letterSpaceIPD.opt,
+                                           letterSpaceIPD.max - letterSpaceIPD.opt,
+                                           letterSpaceIPD.opt - letterSpaceIPD.min,
+                                           new LeafPosition(this, -1), false));
+            } else if (letterSpaceIPD.min == letterSpaceIPD.max) {
+                // constant letter space: replace the box
+                oldListIterator.set(new KnuthInlineBox(ai.ipdArea.opt, alignmentContext, pos, false));
+            } else {
+                // adjustable letter space: replace the glue
+                oldListIterator.next(); // this would return the penalty element
+                oldListIterator.next(); // this would return the glue element
+                oldListIterator.set(new KnuthGlue(ai.iLScount * letterSpaceIPD.opt,
+                                                  ai.iLScount * (letterSpaceIPD.max - letterSpaceIPD.opt),
+                                                  ai.iLScount * (letterSpaceIPD.opt - letterSpaceIPD.min),
+                                                  new LeafPosition(this, -1), true));
+            }
         }
         return oldList;
     }
@@ -862,7 +840,7 @@ public class TextLayoutManager extends LeafNodeLayoutManager {
         }
     }
 
-    /** @see InlineLevelLayoutManager#hyphenate(Position, HyphContext) */
+    /** {@inheritDoc} */
     public void hyphenate(Position pos, HyphContext hc) {
         AreaInfo ai
             = (AreaInfo) vecAreaInfo.get(((LeafPosition) pos).getLeafPos());
@@ -940,7 +918,7 @@ public class TextLayoutManager extends LeafNodeLayoutManager {
         }
     }
 
-    /** @see InlineLevelLayoutManager#applyChanges(List) */
+    /** {@inheritDoc} */
     public boolean applyChanges(List oldList) {
         setFinished(false);
 
@@ -972,7 +950,7 @@ public class TextLayoutManager extends LeafNodeLayoutManager {
         return bChanged;
     }
 
-    /** @see org.apache.fop.layoutmgr.LayoutManager#getChangedKnuthElements(List, int) */
+    /** {@inheritDoc} */
     public LinkedList getChangedKnuthElements(List oldList,
                                               int alignment) {
         if (isFinished()) {
@@ -999,7 +977,7 @@ public class TextLayoutManager extends LeafNodeLayoutManager {
         return returnList;
     }
 
-    /** @see InlineLevelLayoutManager#getWordChars(StringBuffer, Position) */
+    /** {@inheritDoc} */
     public void getWordChars(StringBuffer sbChars, Position pos) {
         int iLeafValue = ((LeafPosition) pos).getLeafPos();
         if (iLeafValue != -1) {
