@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: WrapperLayoutManager.java 535866 2007-05-07 12:22:22Z jeremias $ */
+/* $Id: WrapperLayoutManager.java 627367 2008-02-13 12:03:30Z maxberger $ */
 
 package org.apache.fop.layoutmgr.inline;
 
@@ -41,15 +41,14 @@ public class WrapperLayoutManager extends LeafNodeLayoutManager {
         fobj = node;
     }
 
-    /** @see org.apache.fop.layoutmgr.inline.LeafNodeLayoutManager */
+    /** {@inheritDoc} */
     public InlineArea get(LayoutContext context) {
         // Create a zero-width, zero-height dummy area so this node can
         // participate in the ID handling. Otherwise, addId() wouldn't
         // be called. The area must also be added to the tree, because
         // determination of the X,Y position is done in the renderer.
         InlineArea area = new InlineArea();
-        String id = fobj.getId();
-        if (id != null && id.length() > 0) {
+        if (fobj.hasId()) {
             TraitSetter.setProducerID(area, fobj.getId());
         }
         return area;
@@ -64,8 +63,7 @@ public class WrapperLayoutManager extends LeafNodeLayoutManager {
      * @param context the layout context for adding the area
      */
     public void addAreas(PositionIterator posIter, LayoutContext context) {
-        String id = fobj.getId();
-        if (id != null && id.length() > 0) {
+        if (fobj.hasId()) {
             addId();
             InlineArea area = getEffectiveArea();
             parentLM.addChildArea(area);
@@ -75,7 +73,7 @@ public class WrapperLayoutManager extends LeafNodeLayoutManager {
         }
     }
 
-    /** @see org.apache.fop.layoutmgr.inline.LeafNodeLayoutManager#addId() */
+    /** {@inheritDoc} */
     protected void addId() {
         getPSLM().addIDToPage(fobj.getId());
     }
