@@ -15,34 +15,39 @@
  * limitations under the License.
  */
 
-/* $Id: TableLayoutManager.java 478928 2006-11-24 17:32:48Z vhennebert $ */
+/* $Id: TableLayoutManager.java 669118 2008-06-18 09:02:45Z jeremias $ */
 
 package org.apache.fop.layoutmgr.table;
 
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.LinkedList;
+import java.util.List;
+
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
-import org.apache.fop.fo.flow.Table;
-import org.apache.fop.fo.flow.TableColumn;
+
+import org.apache.fop.area.Area;
+import org.apache.fop.area.Block;
+import org.apache.fop.datatypes.LengthBase;
+import org.apache.fop.fo.Constants;
+import org.apache.fop.fo.FONode;
+import org.apache.fop.fo.FObj;
+import org.apache.fop.fo.flow.table.Table;
+import org.apache.fop.fo.flow.table.TableColumn;
 import org.apache.fop.layoutmgr.BlockStackingLayoutManager;
+import org.apache.fop.layoutmgr.BreakElement;
 import org.apache.fop.layoutmgr.ConditionalElementListener;
 import org.apache.fop.layoutmgr.KnuthElement;
 import org.apache.fop.layoutmgr.KnuthGlue;
 import org.apache.fop.layoutmgr.LayoutContext;
 import org.apache.fop.layoutmgr.ListElement;
 import org.apache.fop.layoutmgr.PositionIterator;
-import org.apache.fop.layoutmgr.Position;
 import org.apache.fop.layoutmgr.RelSide;
 import org.apache.fop.layoutmgr.TraitSetter;
-import org.apache.fop.area.Area;
-import org.apache.fop.area.Block;
 import org.apache.fop.traits.MinOptMax;
 import org.apache.fop.traits.SpaceVal;
-
-import java.util.Iterator;
-import java.util.LinkedList;
-import org.apache.fop.datatypes.LengthBase;
-import org.apache.fop.fo.FONode;
-import org.apache.fop.fo.FObj;
+import org.apache.fop.util.BreakUtil;
 
 /**
  * LayoutManager for a table FO.
@@ -77,7 +82,27 @@ public class TableLayoutManager extends BlockStackingLayoutManager
     
     private int halfBorderSeparationBPD;
     private int halfBorderSeparationIPD;
-    
+
+    /** See {@link TableLayoutManager#registerColumnBackgroundArea(TableColumn, Block, int)}. */
+    private List columnBackgroundAreas;
+
+    /**
+     * Temporary holder of column background informations for a table-cell's area.
+     * 
+     * @see TableLayoutManager#registerColumnBackgroundArea(TableColumn, Block, int)
+     */
+    private static final class ColumnBackgroundInfo {
+        private TableColumn column;
+        private Block backgroundArea;
+        private int xShift;
+
+        private ColumnBackgroundInfo(TableColumn column, Block backgroundArea, int xShift) {
+            this.column = column;
+            this.backgroundArea = backgroundArea;
+            this.xShift = xShift;
+        }
+    }
+
     /**
      * Create a new table layout manager.
      * @param node the table FO
@@ -99,7 +124,7 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         return this.columns;
     }
     
-    /** @see org.apache.fop.layoutmgr.LayoutManager#initialize() */
+    /** {@inheritDoc} */
     public void initialize() {
         foSpaceBefore = new SpaceVal(
                 getTable().getCommonMarginBlock().spaceBefore, this).getSpace();
@@ -134,33 +159,31 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         this.effSpaceAfter = null;
     }
     
-    /** @return half the value of border-separation.block-progression-dimension. */
+    /**
+     * @return half the value of border-separation.block-progression-dimension, or 0 if
+     * border-collapse="collapse".
+     */
     public int getHalfBorderSeparationBPD() {
         return halfBorderSeparationBPD;
     }
 
-    /** @return half the value of border-separation.inline-progression-dimension. */
+    /**
+     * @return half the value of border-separation.inline-progression-dimension, or 0 if
+     * border-collapse="collapse".
+     */
     public int getHalfBorderSeparationIPD() {
         return halfBorderSeparationIPD;
     }
-    
-    /** @see org.apache.fop.layoutmgr.LayoutManager */
+
+    /** {@inheritDoc} */
     public LinkedList getNextKnuthElements(LayoutContext context, int alignment) {
         
         LinkedList returnList = new LinkedList();
-        
-        if (!breakBeforeServed) {
-            try {
-                if (addKnuthElementsForBreakBefore(returnList, context)) {
-                    return returnList;
-                }
-            } finally {
-                breakBeforeServed = true;
-            }
-        }
 
+        /*
+         * Compute the IPD and adjust it if necessary (overconstrained)
+         */
         referenceIPD = context.getRefIPD();
-
         if (getTable().getInlineProgressionDimension().getOptimum(this).getEnum() != EN_AUTO) {
             int contentIPD = getTable().getInlineProgressionDimension().getOptimum(this)
                     .getLength().getValue(this);
@@ -173,7 +196,6 @@ public class TableLayoutManager extends BlockStackingLayoutManager
             }
             updateContentAreaIPDwithOverconstrainedAdjust();
         }
-
         int sumOfColumns = columns.getSumOfColumnWidths(this);
         if (!autoLayout && sumOfColumns > getContentAreaIPD()) {
             log.debug(FONode.decorateWithContextInfo(
@@ -181,7 +203,6 @@ public class TableLayoutManager extends BlockStackingLayoutManager
                     getTable()));
             updateContentAreaIPDwithOverconstrainedAdjust(sumOfColumns);
         }
-
         int availableIPD = referenceIPD - getIPIndents();
         if (getContentAreaIPD() > availableIPD) {
             log.warn(FONode.decorateWithContextInfo(
@@ -205,16 +226,16 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         if (getTable().isSeparateBorderModel()) {
             addKnuthElementsForBorderPaddingBefore(returnList, !firstVisibleMarkServed);
             firstVisibleMarkServed = true;
+            // Border and padding to be repeated at each break
+            // This must be done only in the separate-border model, as in collapsing
+            // tables have no padding and borders are determined at the cell level
+            addPendingMarks(context);
         }
 
-        //Spaces, border and padding to be repeated at each break
-        addPendingMarks(context);
 
+        // Elements for the table-header/footer/body
         LinkedList contentKnuthElements = null;
-        LinkedList contentList = new LinkedList();
-        //Position returnPosition = new NonLeafPosition(this, null);
-        //Body prevLM = null;
-
+        contentLM = new TableContentLayoutManager(this);
         LayoutContext childLC = new LayoutContext(0);
         /*
         childLC.setStackLimit(
@@ -223,19 +244,7 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         childLC.setRefIPD(context.getRefIPD());
         childLC.copyPendingMarksFrom(context);
 
-        if (contentLM == null) {
-            contentLM = new TableContentLayoutManager(this);
-        }
         contentKnuthElements = contentLM.getNextKnuthElements(childLC, alignment);
-        if (childLC.isKeepWithNextPending()) {
-            log.debug("TableContentLM signals pending keep-with-next");
-            context.setFlags(LayoutContext.KEEP_WITH_NEXT_PENDING);
-        }
-        if (childLC.isKeepWithPreviousPending()) {
-            log.debug("TableContentLM signals pending keep-with-previous");
-            context.setFlags(LayoutContext.KEEP_WITH_PREVIOUS_PENDING);
-        }
-        
         //Set index values on elements coming from the content LM
         Iterator iter = contentKnuthElements.iterator();
         while (iter.hasNext()) {
@@ -243,78 +252,66 @@ public class TableLayoutManager extends BlockStackingLayoutManager
             notifyPos(el.getPosition());
         }
         log.debug(contentKnuthElements);
-        
-        if (contentKnuthElements.size() == 1
-                && ((ListElement)contentKnuthElements.getFirst()).isForcedBreak()) {
-            // a descendant of this block has break-before
-            if (returnList.size() == 0) {
-                // the first child (or its first child ...) has
-                // break-before;
-                // all this block, including space before, will be put in
-                // the
-                // following page
-                //FIX ME
-                //bSpaceBeforeServed = false;
-            }
-            contentList.addAll(contentKnuthElements);
+        wrapPositionElements(contentKnuthElements, returnList);
 
-            // "wrap" the Position inside each element
-            // moving the elements from contentList to returnList
-            contentKnuthElements = new LinkedList();
-            wrapPositionElements(contentList, returnList);
-
-            return returnList;
-        } else {
-            /*
-            if (prevLM != null) {
-                // there is a block handled by prevLM
-                // before the one handled by curLM
-                if (mustKeepTogether() 
-                        || prevLM.mustKeepWithNext()
-                        || curLM.mustKeepWithPrevious()) {
-                    // add an infinite penalty to forbid a break between
-                    // blocks
-                    contentList.add(new KnuthPenalty(0,
-                            KnuthElement.INFINITE, false,
-                            new Position(this), false));
-                } else if (!((KnuthElement) contentList.getLast()).isGlue()) {
-                    // add a null penalty to allow a break between blocks
-                    contentList.add(new KnuthPenalty(0, 0, false,
-                            new Position(this), false));
-                } else {
-                    // the last element in contentList is a glue;
-                    // it is a feasible breakpoint, there is no need to add
-                    // a penalty
-                }
-            }*/
-            contentList.addAll(contentKnuthElements);
-            if (contentKnuthElements.size() > 0) {
-                if (((ListElement)contentKnuthElements.getLast()).isForcedBreak()) {
-                    // a descendant of this block has break-after
-                    if (false /*curLM.isFinished()*/) {
-                        // there is no other content in this block;
-                        // it's useless to add space after before a page break
-                        setFinished(true);
-                    }
-
-                    contentKnuthElements = new LinkedList();
-                    wrapPositionElements(contentList, returnList);
-
-                    return returnList;
-                }
-            }
+        if (mustKeepWithPrevious() || childLC.isKeepWithPreviousPending()) {
+            context.setFlags(LayoutContext.KEEP_WITH_PREVIOUS_PENDING);
         }
-        wrapPositionElements(contentList, returnList);
+        if (mustKeepWithNext() || childLC.isKeepWithNextPending()) {
+            context.setFlags(LayoutContext.KEEP_WITH_NEXT_PENDING);
+        }
+
         if (getTable().isSeparateBorderModel()) {
             addKnuthElementsForBorderPaddingAfter(returnList, true);
         }
         addKnuthElementsForSpaceAfter(returnList, alignment);
-        addKnuthElementsForBreakAfter(returnList, context);
+
+        if (!context.suppressBreakBefore()) {
+            //addKnuthElementsForBreakBefore(returnList, context);
+            int breakBefore = BreakUtil.compareBreakClasses(getTable().getBreakBefore(),
+                    childLC.getBreakBefore());
+            if (breakBefore != Constants.EN_AUTO) {
+                returnList.addFirst(new BreakElement(getAuxiliaryPosition(), 
+                        0, -KnuthElement.INFINITE, breakBefore, context));
+            }
+        }
+
+        //addKnuthElementsForBreakAfter(returnList, context);
+        int breakAfter = BreakUtil.compareBreakClasses(getTable().getBreakAfter(),
+                childLC.getBreakAfter());
+        if (breakAfter != Constants.EN_AUTO) {
+            returnList.add(new BreakElement(getAuxiliaryPosition(), 
+                    0, -KnuthElement.INFINITE, breakAfter, context));
+        }
+
         setFinished(true);
         resetSpaces();
         return returnList;
     }
-    
+
+    /**
+     * Registers the given area, that will be used to render the part of column background
+     * covered by a table-cell. If percentages are used to place the background image, the
+     * final bpd of the (fraction of) table that will be rendered on the current page must
+     * be known. The traits can't then be set when the areas for the cell are created
+     * since at that moment this bpd is yet unknown. So they will instead be set in
+     * TableLM's {@link #addAreas(PositionIterator, LayoutContext)} method.
+     * 
+     * @param column the table-column element from which the cell gets background
+     * informations
+     * @param backgroundArea the block of the cell's dimensions that will hold the column
+     * background
+     * @param xShift additional amount by which the image must be shifted to be correctly
+     * placed (to counterbalance the cell's start border)
+     */
+    void registerColumnBackgroundArea(TableColumn column, Block backgroundArea, int xShift) {
+        addBackgroundArea(backgroundArea);
+        if (columnBackgroundAreas == null) {
+            columnBackgroundAreas = new ArrayList();
+        }
+        columnBackgroundAreas.add(new ColumnBackgroundInfo(column, backgroundArea, xShift));
+    }
+
     /**
      * The table area is a reference area that contains areas for
      * columns, bodies, rows and the contents are in cells.
@@ -336,6 +333,7 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         
         // add column, body then row areas
 
+        // BPD of the table, i.e., height of its content; table's borders and paddings not counted
         int tableHeight = 0;
         //Body childLM;
         LayoutContext lc = new LayoutContext(0);
@@ -348,6 +346,17 @@ public class TableLayoutManager extends BlockStackingLayoutManager
 
         curBlockArea.setBPD(tableHeight);
 
+        if (columnBackgroundAreas != null) {
+            for (Iterator iter = columnBackgroundAreas.iterator(); iter.hasNext();) {
+                ColumnBackgroundInfo b = (ColumnBackgroundInfo) iter.next();
+                TraitSetter.addBackground(b.backgroundArea,
+                        b.column.getCommonBorderPaddingBackground(), this,
+                        b.xShift, -b.backgroundArea.getYOffset(),
+                        b.column.getColumnWidth().getValue(this), tableHeight);
+            }
+            columnBackgroundAreas.clear();
+        }
+
         if (getTable().isSeparateBorderModel()) {
             TraitSetter.addBorders(curBlockArea, 
                     getTable().getCommonBorderPaddingBackground(), 
@@ -417,30 +426,28 @@ public class TableLayoutManager extends BlockStackingLayoutManager
     }
 
     /**
-     * Reset the position of this layout manager.
-     *
-     * @param resetPos the position to reset to
+     * Adds the given area to this layout manager's area, without updating the used bpd.
+     * 
+     * @param background an area
      */
-    public void resetPosition(Position resetPos) {
-        if (resetPos == null) {
-            reset(null);
-        }
+    void addBackgroundArea(Block background) {
+        curBlockArea.addChildArea(background);
     }
 
-    /** @see org.apache.fop.layoutmgr.BlockLevelLayoutManager */
+    /** {@inheritDoc} */
     public int negotiateBPDAdjustment(int adj, KnuthElement lastElement) {
         // TODO Auto-generated method stub
         return 0;
     }
 
-    /** @see org.apache.fop.layoutmgr.BlockLevelLayoutManager */
+    /** {@inheritDoc} */
     public void discardSpace(KnuthGlue spaceGlue) {
         // TODO Auto-generated method stub
         
     }
 
     /**
-     * @see org.apache.fop.layoutmgr.BlockLevelLayoutManager#mustKeepTogether()
+     * {@inheritDoc}
      */
     public boolean mustKeepTogether() {
         //TODO Keeps will have to be more sophisticated sooner or later
@@ -450,7 +457,7 @@ public class TableLayoutManager extends BlockStackingLayoutManager
     }
 
     /**
-     * @see org.apache.fop.layoutmgr.BlockLevelLayoutManager#mustKeepWithPrevious()
+     * {@inheritDoc}
      */
     public boolean mustKeepWithPrevious() {
         return !getTable().getKeepWithPrevious().getWithinPage().isAuto()
@@ -458,7 +465,7 @@ public class TableLayoutManager extends BlockStackingLayoutManager
     }
 
     /**
-     * @see org.apache.fop.layoutmgr.BlockLevelLayoutManager#mustKeepWithNext()
+     * {@inheritDoc}
      */
     public boolean mustKeepWithNext() {
         return !getTable().getKeepWithNext().getWithinPage().isAuto()
@@ -468,7 +475,7 @@ public class TableLayoutManager extends BlockStackingLayoutManager
     // --------- Property Resolution related functions --------- //
 
     /**
-     * @see org.apache.fop.datatypes.PercentBaseContext#getBaseLength(int, FObj)
+     * {@inheritDoc} 
      */
     public int getBaseLength(int lengthBase, FObj fobj) {
         // Special handler for TableColumn width specifications
@@ -492,7 +499,7 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         }
     }
     
-    /** @see org.apache.fop.layoutmgr.ConditionalElementListener */
+    /** {@inheritDoc} */
     public void notifySpace(RelSide side, MinOptMax effectiveLength) {
         if (RelSide.BEFORE == side) {
             if (log.isDebugEnabled()) {
@@ -509,7 +516,7 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         }
     }
 
-    /** @see org.apache.fop.layoutmgr.ConditionalElementListener */
+    /** {@inheritDoc} */
     public void notifyBorder(RelSide side, MinOptMax effectiveLength) {
         if (effectiveLength == null) {
             if (RelSide.BEFORE == side) {
@@ -523,7 +530,7 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         }
     }
 
-    /** @see org.apache.fop.layoutmgr.ConditionalElementListener */
+    /** {@inheritDoc} */
     public void notifyPadding(RelSide side, MinOptMax effectiveLength) {
         if (effectiveLength == null) {
             if (RelSide.BEFORE == side) {
@@ -537,4 +544,4 @@ public class TableLayoutManager extends BlockStackingLayoutManager
         }
     }
 
-}
\ No newline at end of file
+}
