@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFDocument.java 535883 2007-05-07 14:30:23Z jeremias $ */
+/* $Id: PDFDocument.java 611114 2008-01-11 09:04:28Z jeremias $ */
 
 package org.apache.fop.pdf;
 
@@ -24,15 +24,16 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
 import java.io.UnsupportedEncodingException;
+import java.io.Writer;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.text.DateFormat;
 import java.text.SimpleDateFormat;
 import java.util.Collections;
 import java.util.Date;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
-import java.util.Iterator;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
@@ -316,6 +317,22 @@ public class PDFDocument {
         }
     }
 
+    /**
+     * Creates and returns a Writer object wrapping the given OutputStream. The Writer is
+     * buffered to reduce the number of calls to the encoding converter so don't forget
+     * to <code>flush()</code> the Writer after use or before writing directly to the
+     * underlying OutputStream.
+     * @param out the OutputStream to write to
+     * @return the requested Writer
+     */
+    public static Writer getWriterFor(OutputStream out) {
+        try {
+            return new java.io.BufferedWriter(new java.io.OutputStreamWriter(out, ENCODING));
+        } catch (UnsupportedEncodingException uee) {
+            throw new Error("JVM doesn't support " + ENCODING + " encoding!");
+        }
+    }
+    
     /**
      * set the producer of the document
      *
@@ -721,8 +738,20 @@ public class PDFDocument {
      *
      * @param key the image key to look for
      * @return the image or PDFXObject for the key if found
+     * @deprecated Use getXObject instead (so forms are treated in the same way)
      */
-    public PDFXObject getImage(String key) {
+    public PDFImageXObject getImage(String key) {
+        PDFImageXObject xObject = (PDFImageXObject)xObjectsMap.get(key);
+        return xObject;
+    }
+
+    /**
+     * Get an XObject from the image map.
+     *
+     * @param key the XObject key to look for
+     * @return the PDFXObject for the key if found
+     */
+    public PDFXObject getXObject(String key) {
         PDFXObject xObject = (PDFXObject)xObjectsMap.get(key);
         return xObject;
     }
@@ -779,10 +808,10 @@ public class PDFDocument {
      * @param img the PDF image to add
      * @return the PDF XObject that references the PDF image data
      */
-    public PDFXObject addImage(PDFResourceContext res, PDFImage img) {
+    public PDFImageXObject addImage(PDFResourceContext res, PDFImage img) {
         // check if already created
         String key = img.getKey();
-        PDFXObject xObject = (PDFXObject)xObjectsMap.get(key);
+        PDFImageXObject xObject = (PDFImageXObject)xObjectsMap.get(key);
         if (xObject != null) {
             if (res != null) {
                 res.getPDFResources().addXObject(xObject);
@@ -793,7 +822,7 @@ public class PDFDocument {
         // setup image
         img.setup(this);
         // create a new XObject
-        xObject = new PDFXObject(++this.xObjectCount, img);
+        xObject = new PDFImageXObject(++this.xObjectCount, img);
         registerObject(xObject);
         this.resources.addXObject(xObject);
         if (res != null) {
@@ -811,26 +840,35 @@ public class PDFDocument {
      *
      * @param res the PDF resource context to add to, may be null
      * @param cont the PDF Stream contents of the Form XObject
-     * @param formres the PDF Resources for the Form XObject data
+     * @param formres a reference to the PDF Resources for the Form XObject data
      * @param key the key for the object
      * @return the PDF Form XObject that references the PDF data
      */
     public PDFFormXObject addFormXObject(
         PDFResourceContext res,
         PDFStream cont,
-        PDFResources formres,
+        PDFReference formres,
         String key) {
-        PDFFormXObject xObject;
-        xObject =
-            new PDFFormXObject(
+        
+        // check if already created
+        PDFFormXObject xObject = (PDFFormXObject)xObjectsMap.get(key);
+        if (xObject != null) {
+            if (res != null) {
+                res.getPDFResources().addXObject(xObject);
+            }
+            return xObject;
+        }
+        
+        xObject = new PDFFormXObject(
                 ++this.xObjectCount,
                 cont,
-                formres.referencePDF());
+                formres);
         registerObject(xObject);
         this.resources.addXObject(xObject);
         if (res != null) {
             res.getPDFResources().addXObject(xObject);
         }
+        this.xObjectsMap.put(key, xObject);
         return xObject;
     }
 
