@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFFilterList.java 542237 2007-05-28 14:31:24Z jeremias $ */
+/* $Id: PDFFilterList.java 611278 2008-01-11 19:50:53Z jeremias $ */
  
 package org.apache.fop.pdf;
 
@@ -34,6 +34,8 @@ public class PDFFilterList {
     public static final String DEFAULT_FILTER = "default";
     /** Key for the filter used for normal content*/
     public static final String CONTENT_FILTER = "content";
+    /** Key for the filter used for precompressed content */
+    public static final String PRECOMPRESSED_FILTER = "precompressed";
     /** Key for the filter used for images */
     public static final String IMAGE_FILTER = "image";
     /** Key for the filter used for JPEG images */
@@ -48,6 +50,8 @@ public class PDFFilterList {
     private List filters = new java.util.ArrayList();
 
     private boolean ignoreASCIIFilters = false;
+    
+    private boolean disableAllFilters = false;
         
     /**
      * Default constructor.
@@ -67,6 +71,22 @@ public class PDFFilterList {
         this.ignoreASCIIFilters = ignoreASCIIFilters;
     }
 
+    /**
+     * Used to disable all filters.
+     * @param value true if all filters shall be disabled
+     */
+    public void setDisableAllFilters(boolean value) {
+        this.disableAllFilters = value;
+    }
+    
+    /**
+     * Returns true if all filters are disabled.
+     * @return true if all filters are disabled
+     */
+    public boolean isDisableAllFilters() {
+        return this.disableAllFilters;
+    }
+    
     /**
      * Indicates whether the filter list is already initialized.
      * @return true if more there are filters present
@@ -160,6 +180,9 @@ public class PDFFilterList {
             } else if (TIFF_FILTER.equals(type)) {
                 //CCITT-encoded images are already well compressed
                 addFilter(new NullFilter());
+            } else if (PRECOMPRESSED_FILTER.equals(type)) {
+                //precompressed content doesn't need further compression
+                addFilter(new NullFilter());
             } else {
                 // built-in default to flate
                 addFilter(new FlateFilter());
@@ -185,21 +208,7 @@ public class PDFFilterList {
             List names = new java.util.ArrayList();
             List parms = new java.util.ArrayList();
 
-            // run the filters
-            int nonNullParams = 0;
-            for (int count = 0; count < filters.size(); count++) {
-                PDFFilter filter = (PDFFilter)filters.get(count);
-                // place the names in our local vector in reverse order
-                if (filter.getName().length() > 0) {
-                    names.add(0, filter.getName());
-                    if (filter.getDecodeParms() != null) {
-                        parms.add(0, filter.getDecodeParms());
-                        nonNullParams++;
-                    } else {
-                        parms.add(0, null);
-                    }
-                }
-            }
+            int nonNullParams = populateNamesAndParms(names, parms);
 
             // now build up the filter entries for the dictionary
             return buildFilterEntries(names) 
@@ -209,6 +218,47 @@ public class PDFFilterList {
 
     }
 
+    /**
+     * Apply the filters to the data
+     * in the order given and add the /Filter and /DecodeParms
+     * entries to the stream dictionary. If the filters have already
+     * been applied to the data (either externally, or internally)
+     * then the dictionary entries added.
+     * @param dict the PDFDictionary to set the entries on
+     */
+    protected void putFilterDictEntries(PDFDictionary dict) {
+        if (filters != null && filters.size() > 0) {
+            List names = new java.util.ArrayList();
+            List parms = new java.util.ArrayList();
+
+            populateNamesAndParms(names, parms);
+
+            // now build up the filter entries for the dictionary
+            putFilterEntries(dict, names);
+            putDecodeParams(dict, parms);
+        }
+    }
+    
+    private int populateNamesAndParms(List names, List parms) {
+        // run the filters
+        int nonNullParams = 0;
+        for (int count = 0; count < filters.size(); count++) {
+            PDFFilter filter = (PDFFilter)filters.get(count);
+            // place the names in our local vector in reverse order
+            if (filter.getName().length() > 0) {
+                names.add(0, filter.getName());
+                PDFObject param = filter.getDecodeParms(); 
+                if (param != null) {
+                    parms.add(0, param);
+                    nonNullParams++;
+                } else {
+                    parms.add(0, null);
+                }
+            }
+        }
+        return nonNullParams;
+    }
+
     private String buildFilterEntries(List names) {
         int filterCount = 0;
         StringBuffer sb = new StringBuffer(64);
@@ -231,6 +281,23 @@ public class PDFFilterList {
         }
     }
 
+    private void putFilterEntries(PDFDictionary dict, List names) {
+        PDFArray array = new PDFArray(dict);
+        for (int i = 0, c = names.size(); i < c; i++) {
+            final String name = (String)names.get(i);
+            if (name.length() > 0) {
+                array.add(new PDFName(name));
+            }
+        }
+        if (array.length() > 0) {
+            if (array.length() > 1) {
+                dict.put("Filter", array);
+            } else {
+                dict.put("Filter", array.get(0));
+            }
+        }
+    }
+    
     private String buildDecodeParms(List parms) {
         StringBuffer sb = new StringBuffer();
         boolean needParmsEntry = false;
@@ -259,6 +326,26 @@ public class PDFFilterList {
         }
     }
 
+    private void putDecodeParams(PDFDictionary dict, List parms) {
+        boolean needParmsEntry = false;
+        PDFArray array = new PDFArray(dict);
+        for (int i = 0, c = parms.size(); i < c; i++) {
+            Object obj = parms.get(i);
+            if (obj != null) {
+                array.add(obj);
+                needParmsEntry = true;
+            } else {
+                array.add(null);
+            }
+        }
+        if (array.length() > 0 & needParmsEntry) {
+            if (array.length() > 1) {
+                dict.put("DecodeParms", array);
+            } else {
+                dict.put("DecodeParms", array.get(0));
+            }
+        }
+    }
     
     /**
      * Applies all registered filters as necessary. The method returns an 
@@ -269,7 +356,7 @@ public class PDFFilterList {
      */
     public OutputStream applyFilters(OutputStream stream) throws IOException {
         OutputStream out = stream;
-        if (filters != null) {
+        if (filters != null && !isDisableAllFilters()) {
             for (int count = filters.size() - 1; count >= 0; count--) {
                 PDFFilter filter = (PDFFilter)filters.get(count);
                 out = filter.applyFilter(out);
