@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id$ */
+/* $Id: PDFMetadata.java 627367 2008-02-13 12:03:30Z maxberger $ */
  
 package org.apache.fop.pdf;
 
@@ -25,6 +25,8 @@ import java.util.Date;
 
 import javax.xml.transform.TransformerConfigurationException;
 
+import org.xml.sax.SAXException;
+
 import org.apache.xmlgraphics.xmp.Metadata;
 import org.apache.xmlgraphics.xmp.XMPSerializer;
 import org.apache.xmlgraphics.xmp.schemas.DublinCoreAdapter;
@@ -36,8 +38,6 @@ import org.apache.xmlgraphics.xmp.schemas.pdf.AdobePDFSchema;
 import org.apache.xmlgraphics.xmp.schemas.pdf.PDFAAdapter;
 import org.apache.xmlgraphics.xmp.schemas.pdf.PDFAXMPSchema;
 
-import org.xml.sax.SAXException;
-
 /**
  * Special PDFStream for Metadata.
  * @since PDF 1.4
@@ -58,7 +58,7 @@ public class PDFMetadata extends PDFStream {
         this.readOnly = readOnly;
     }
 
-    /** @see org.apache.fop.pdf.AbstractPDFStream#setupFilterList() */
+    /** {@inheritDoc} */
     protected void setupFilterList() {
         if (!getFilterList().isInitialized()) {
             getFilterList().addDefaultFilters(
@@ -78,7 +78,7 @@ public class PDFMetadata extends PDFStream {
     /**
      * overload the base object method so we don't have to copy
      * byte arrays around so much
-     * @see org.apache.fop.pdf.PDFObject#output(OutputStream)
+     * {@inheritDoc}
      */
     protected int output(java.io.OutputStream stream)
                 throws java.io.IOException {
@@ -87,7 +87,7 @@ public class PDFMetadata extends PDFStream {
         return length;
     }
     
-    /** @see org.apache.fop.pdf.AbstractPDFStream#outputRawStreamData(java.io.OutputStream) */
+    /** {@inheritDoc} */
     protected void outputRawStreamData(OutputStream out) throws IOException {
         try {
             XMPSerializer.writeXMPPacket(xmpMetadata, out, this.readOnly);
@@ -100,34 +100,29 @@ public class PDFMetadata extends PDFStream {
         }
     }
     
-    /** @see org.apache.fop.pdf.AbstractPDFStream#buildStreamDict(String) */
-    protected String buildStreamDict(String lengthEntry) {
+    /** {@inheritDoc} */
+    protected void populateStreamDict(Object lengthEntry) {
         final String filterEntry = getFilterList().buildFilterDictEntries();
         if (getDocumentSafely().getProfile().getPDFAMode().isPDFA1LevelB() 
                 && filterEntry != null && filterEntry.length() > 0) {
             throw new PDFConformanceException(
                     "The Filter key is prohibited when PDF/A-1 is active");
         }
-        final StringBuffer sb = new StringBuffer(128);
-        sb.append(getObjectID());
-        sb.append("<< ");
-        sb.append("/Type /Metadata");
-        sb.append("\n/Subtype /XML");
-        sb.append("\n/Length " + lengthEntry);
-        sb.append("\n" + filterEntry);
-        sb.append("\n>>\n");
-        return sb.toString();
+        put("Type", new PDFName("Metadata"));
+        put("Subtype", new PDFName("XML"));
+        super.populateStreamDict(lengthEntry);
     }
-
+    
     /**
      * Creates an XMP document based on the settings on the PDF Document.
      * @param pdfDoc the PDF Document
      * @return the requested XMP metadata
      */
-    public static Metadata createXMPFromUserAgent(PDFDocument pdfDoc) {
+    public static Metadata createXMPFromPDFDocument(PDFDocument pdfDoc) {
         Metadata meta = new Metadata();
         
         PDFInfo info = pdfDoc.getInfo();
+        PDFRoot root = pdfDoc.getRoot();
 
         //Set creation date if not available, yet
         if (info.getCreationDate() == null) {
@@ -151,6 +146,10 @@ public class PDFMetadata extends PDFStream {
             //Subject maps to dc:description["x-default"] as per ISO-19005-1:2005/Cor.1:2007
             dc.setDescription(null, info.getSubject());
         }
+        if (root.getLanguage() != null) {
+            //Note: No check is performed to make sure the value is valid RFC 3066!
+            dc.addLanguage(root.getLanguage());
+        }
         dc.addDate(info.getCreationDate());
 
         //PDF/A identification
