@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFStream.java 533986 2007-05-01 09:41:05Z jeremias $ */
+/* $Id: PDFStream.java 588547 2007-10-26 07:48:14Z jeremias $ */
  
 package org.apache.fop.pdf;
 
@@ -64,7 +64,17 @@ public class PDFStream extends AbstractPDFStream {
         }
 
     }
-
+    
+    /**
+     * Returns an OutputStream that can be used to write to the buffer which is used
+     * to build up the PDF stream.
+     * @return the OutputStream
+     * @throws IOException In case of an I/O problem
+     */
+    public OutputStream getBufferOutputStream() throws IOException {
+        return this.data.getOutputStream();
+    }
+    
     /**
      * Used to set the contents of the PDF stream.
      * @param data the contents as a byte array
@@ -90,21 +100,21 @@ public class PDFStream extends AbstractPDFStream {
     }
 
     /**
-     * @see org.apache.fop.pdf.AbstractPDFStream#getSizeHint()
+     * {@inheritDoc}
      */
     protected int getSizeHint() throws IOException {
         return data.getSize();
     }
 
     /**
-     * @see org.apache.fop.pdf.AbstractPDFStream#outputRawStreamData(OutputStream)
+     * {@inheritDoc}
      */
     protected void outputRawStreamData(OutputStream out) throws IOException {
         data.outputContents(out);
     }
 
     /**
-     * @see org.apache.fop.pdf.PDFObject#output(OutputStream)
+     * {@inheritDoc}
      */
     protected int output(OutputStream stream) throws IOException {
         final int len = super.output(stream);
