@@ -15,13 +15,14 @@
  * limitations under the License.
  */
 
-/* $Id$ */
+/* $Id: AFPSVGHandler.java 631178 2008-02-26 11:07:20Z jeremias $ */
 
 package org.apache.fop.render.afp;
 
 // FOP
 import org.apache.fop.render.AbstractGenericSVGHandler;
 import org.apache.fop.render.Renderer;
+import org.apache.fop.render.RendererContext;
 
 /**
  * AFP XML handler for SVG. Uses Apache Batik for SVG processing.
@@ -30,10 +31,17 @@ import org.apache.fop.render.Renderer;
  */
 public class AFPSVGHandler extends AbstractGenericSVGHandler {
 
-    /** @see org.apache.fop.render.XMLHandler#supportsRenderer(org.apache.fop.render.Renderer) */
+    /** {@inheritDoc} */
     public boolean supportsRenderer(Renderer renderer) {
         return (renderer instanceof AFPRenderer);
     }
     
+    /** {@inheritDoc} */
+    protected void updateRendererContext(RendererContext context) {
+        //Work around a problem in Batik: Gradients cannot be done in ColorSpace.CS_GRAY
+        context.setProperty(AFPRendererContextConstants.AFP_GRAYSCALE,
+                Boolean.FALSE);
+    }
+    
 }
 
