@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AFPConstants.java 426576 2006-07-28 15:44:37Z jeremias $ */
+/* $Id: AFPConstants.java 601446 2007-12-05 17:45:20Z acumiskey $ */
 
 package org.apache.fop.render.afp.modca;
 
@@ -28,11 +28,10 @@ public interface AFPConstants {
     /**
      * The encoding to use to convert to EBCIDIC
      */
-    public String EBCIDIC_ENCODING = "Cp1146";
+    String EBCIDIC_ENCODING = "Cp1146";
 
     /**
      * The encoding to use to convert to ASCII
      */
-    public String ASCII_ENCODING = "Cp1252";
-
+    String ASCII_ENCODING = "Cp1252";
 }
