@@ -15,13 +15,14 @@
  * limitations under the License.
  */
 
-/* $Id$ */
+/* $Id: PCLSVGHandler.java 627367 2008-02-13 12:03:30Z maxberger $ */
 
 package org.apache.fop.render.pcl;
 
 // FOP
 import org.apache.fop.render.AbstractGenericSVGHandler;
 import org.apache.fop.render.Renderer;
+import org.apache.fop.render.RendererContext;
 
 /**
  * PCL XML handler for SVG. Uses Apache Batik for SVG processing.
@@ -31,10 +32,17 @@ import org.apache.fop.render.Renderer;
  */
 public class PCLSVGHandler extends AbstractGenericSVGHandler {
 
-    /** @see org.apache.fop.render.XMLHandler#supportsRenderer(org.apache.fop.render.Renderer) */
+    /** {@inheritDoc} */
     public boolean supportsRenderer(Renderer renderer) {
         return (renderer instanceof PCLRenderer);
     }
     
+    /** {@inheritDoc} */
+    protected void updateRendererContext(RendererContext context) {
+        //Work around a problem in Batik: Gradients cannot be done in ColorSpace.CS_GRAY
+        context.setProperty(PCLRendererContextConstants.PCL_COLOR_CANVAS,
+                Boolean.TRUE);
+    }
+    
 }
 
