@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PSTextPainter.java 542237 2007-05-28 14:31:24Z jeremias $ */
+/* $Id: PSTextPainter.java 603590 2007-12-12 12:24:10Z acumiskey $ */
 
 package org.apache.fop.render.ps;
 
@@ -40,6 +40,7 @@ import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
 import org.apache.xmlgraphics.java2d.ps.PSGraphics2D;
+import org.apache.xmlgraphics.java2d.TextHandler;
 
 import org.apache.batik.dom.svg.SVGOMTextElement;
 import org.apache.batik.gvt.text.Mark;
@@ -66,7 +67,7 @@ import org.apache.fop.fonts.FontTriplet;
  * (todo) use drawString(AttributedCharacterIterator iterator...) for some
  *
  * @author <a href="mailto:keiron@aftexsw.com">Keiron Liddle</a>
- * @version $Id: PSTextPainter.java 542237 2007-05-28 14:31:24Z jeremias $
+ * @version $Id: PSTextPainter.java 603590 2007-12-12 12:24:10Z acumiskey $
  */
 public class PSTextPainter implements TextPainter {
     
@@ -74,7 +75,7 @@ public class PSTextPainter implements TextPainter {
     protected Log log = LogFactory.getLog(PSTextPainter.class);
     
     private NativeTextHandler nativeTextHandler;
-    //private FontInfo fontInfo;
+    private FontInfo fontInfo;
 
     /**
      * Use the stroking text painter to get the bounds and shape.
@@ -89,6 +90,7 @@ public class PSTextPainter implements TextPainter {
      */
     public PSTextPainter(NativeTextHandler nativeTextHandler) {
         this.nativeTextHandler = nativeTextHandler;
+        this.fontInfo = nativeTextHandler.getFontInfo();
     }
 
     /**
@@ -314,6 +316,12 @@ public class PSTextPainter implements TextPainter {
             }
         }
         
+        drawPrimitiveString(g2d, loc, font, txt, tx);
+        loc.setLocation(loc.getX() + (double)advance, loc.getY());
+        return loc;
+    }
+
+    protected void drawPrimitiveString(Graphics2D g2d, Point2D loc, Font font, String txt, float tx) {
         //Finally draw text
         nativeTextHandler.setOverrideFont(font);
         try {
@@ -327,8 +335,6 @@ public class PSTextPainter implements TextPainter {
         } finally {
             nativeTextHandler.setOverrideFont(null);
         }
-        loc.setLocation(loc.getX() + (double)advance, loc.getY());
-        return loc;
     }
 
     private void updateLocationFromACI(
@@ -379,8 +385,6 @@ public class PSTextPainter implements TextPainter {
         String style = getStyle(aci);
         int weight = getWeight(aci);
 
-        boolean found = false;
-        FontInfo fontInfo = nativeTextHandler.getFontInfo();
         String fontFamily = null;
         List gvtFonts = (List) aci.getAttribute(
                       GVTAttributedCharacterIterator.TextAttribute.GVT_FONT_FAMILIES);
