@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfExternalGraphic.java 426576 2006-07-28 15:44:37Z jeremias $ */
+/* $Id: RtfExternalGraphic.java 675150 2008-07-09 12:56:41Z jeremias $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -26,19 +26,17 @@ package org.apache.fop.render.rtf.rtflib.rtfdoc;
  * the FOP project.
  */
 
-import org.apache.commons.io.IOUtils;
-import org.apache.fop.render.rtf.rtflib.tools.ImageConstants;
-import org.apache.fop.render.rtf.rtflib.tools.ImageUtil;
-//import org.apache.fop.render.rtf.rtflib.tools.jpeg.Encoder;
-//import org.apache.fop.render.rtf.rtflib.tools.jpeg.JPEGException;
-
+import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.Writer;
-
-import java.io.File;
-import java.net.URL;
 import java.net.MalformedURLException;
+import java.net.URL;
+
+import org.apache.commons.io.IOUtils;
+
+import org.apache.fop.render.rtf.rtflib.tools.ImageConstants;
+import org.apache.fop.render.rtf.rtflib.tools.ImageUtil;
 
 /**
  * Creates an RTF image from an external graphic file.
@@ -67,7 +65,7 @@ public class RtfExternalGraphic extends RtfElement {
             super(reason);
         }
     }
-    
+
     //////////////////////////////////////////////////
     // Supported Formats
     //////////////////////////////////////////////////
@@ -85,7 +83,7 @@ public class RtfExternalGraphic extends RtfElement {
         public static boolean isFormat(byte[] data) {
             return false;
         }
-        
+
         /**
          * Convert image data if necessary - for example when format is not supported by rtf.
          *
@@ -95,7 +93,7 @@ public class RtfExternalGraphic extends RtfElement {
         public FormatBase convert(FormatBase format, byte[] data) {
             return format;
         }
-        
+
         /**
          * Determine image file format.
          *
@@ -120,7 +118,7 @@ public class RtfExternalGraphic extends RtfElement {
                 return null;
             }
         }
-        
+
         /**
          * Get image type.
          *
@@ -129,7 +127,7 @@ public class RtfExternalGraphic extends RtfElement {
         public int getType() {
             return ImageConstants.I_NOT_SUPPORTED;
         }
-        
+
         /**
          * Get rtf tag.
          *
@@ -139,7 +137,7 @@ public class RtfExternalGraphic extends RtfElement {
             return "";
         }
     }
-    
+
     private static class FormatGIF extends FormatBase {
         public static boolean isFormat(byte[] data) {
             // Indentifier "GIF8" on position 0
@@ -147,12 +145,12 @@ public class RtfExternalGraphic extends RtfElement {
 
             return ImageUtil.compareHexValues(pattern, data, 0, true);
         }
-        
+
         public int getType() {
             return ImageConstants.I_GIF;
         }
     }
-    
+
     private static class FormatEMF extends FormatBase {
         public static boolean isFormat(byte[] data) {
             // No offical Indentifier known
@@ -160,28 +158,28 @@ public class RtfExternalGraphic extends RtfElement {
 
             return ImageUtil.compareHexValues(pattern, data, 0, true);
         }
-        
+
         public int getType() {
             return ImageConstants.I_EMF;
         }
-        
+
         public String getRtfTag() {
             return "emfblip";
         }
     }
-    
+
     private  static class FormatBMP extends FormatBase {
         public static boolean isFormat(byte[] data) {
             byte [] pattern = new byte [] {(byte) 0x42, (byte) 0x4D};
 
             return ImageUtil.compareHexValues(pattern, data, 0, true);
         }
-        
+
         public int getType() {
             return ImageConstants.I_BMP;
         }
     }
-    
+
     private static class FormatJPG extends FormatBase {
         public static boolean isFormat(byte[] data) {
             // Indentifier "0xFFD8" on position 0
@@ -189,16 +187,16 @@ public class RtfExternalGraphic extends RtfElement {
 
             return ImageUtil.compareHexValues(pattern, data, 0, true);
         }
-      
+
         public int getType() {
             return ImageConstants.I_JPG;
         }
-        
+
         public String getRtfTag() {
             return "jpegblip";
         }
     }
-    
+
     private static class FormatPNG extends FormatBase {
         public static boolean isFormat(byte[] data) {
             // Indentifier "PNG" on position 1
@@ -206,16 +204,16 @@ public class RtfExternalGraphic extends RtfElement {
 
             return ImageUtil.compareHexValues(pattern, data, 1, true);
         }
-        
+
         public int getType() {
             return ImageConstants.I_PNG;
         }
-        
+
         public String getRtfTag() {
             return "pngblip";
         }
     }
-    
+
     //////////////////////////////////////////////////
     // @@ Members
     //////////////////////////////////////////////////
@@ -271,6 +269,9 @@ public class RtfExternalGraphic extends RtfElement {
      */
     protected boolean scaleUniform = false;
 
+    /** cropping on left/top/right/bottom edges for \piccrop*N */
+    private int[] cropValues = new int[4];
+
     /**
      * Graphic compression rate
      */
@@ -342,13 +343,13 @@ public class RtfExternalGraphic extends RtfElement {
         }
 
 
-        if (url == null) {
-            throw new ExternalGraphicException("The attribute 'url' of "
-                    + "<fo:external-graphic> is null.");
+        if (url == null && imagedata == null) {
+            throw new ExternalGraphicException(
+                    "No image data is available (neither URL, nor in-memory)");
         }
 
         String linkToRoot = System.getProperty("jfor_link_to_root");
-        if (linkToRoot != null) {
+        if (url != null && linkToRoot != null) {
             writer.write("{\\field {\\* \\fldinst { INCLUDEPICTURE \"");
             writer.write(linkToRoot);
             File urlFile = new File(url.getFile());
@@ -372,7 +373,7 @@ public class RtfExternalGraphic extends RtfElement {
                 throw new ExternalGraphicException("The attribute 'src' of "
                         + "<fo:external-graphic> has a invalid value: '"
                         + url + "' (" + e + ")");
-            }            
+            }
         }
 
         if (imagedata == null) {
@@ -380,13 +381,13 @@ public class RtfExternalGraphic extends RtfElement {
         }
 
         // Determine image file format
-        String file = url.getFile ();
+        String file = (url != null ? url.getFile() : "<unknown>");
         imageformat = FormatBase.determineFormat(imagedata);
         if (imageformat != null) {
             imageformat = imageformat.convert(imageformat, imagedata);
         }
-        
-        if (imageformat == null 
+
+        if (imageformat == null
                 || imageformat.getType() == ImageConstants.I_NOT_SUPPORTED
                 || "".equals(imageformat.getRtfTag())) {
             throw new ExternalGraphicException("The tag <fo:external-graphic> "
@@ -408,6 +409,7 @@ public class RtfExternalGraphic extends RtfElement {
 
         computeImageSize();
         writeSizeInfo();
+        writeAttributes(getRtfAttributes(), null);
 
         for (int i = 0; i < imagedata.length; i++) {
             int iData = imagedata [i];
@@ -467,17 +469,17 @@ public class RtfExternalGraphic extends RtfElement {
             }
         } else if (imageformat.getType() == ImageConstants.I_EMF) {
             int i = 0;
-            
+
             i = ImageUtil.getIntFromByteArray(imagedata, 151, 4, false);
             if (i != 0 ) {
-                width = i; 
+                width = i;
             }
-            
+
             i = ImageUtil.getIntFromByteArray(imagedata, 155, 4, false);
             if (i != 0 ) {
                 height = i;
             }
-            
+
         }
     }
 
@@ -521,6 +523,19 @@ public class RtfExternalGraphic extends RtfElement {
                 writeControlWord("picscaley" + widthDesired * 100 / width);
             }
         }
+
+        if (this.cropValues[0] != 0) {
+            writeOneAttribute("piccropl", new Integer(this.cropValues[0]));
+        }
+        if (this.cropValues[1] != 0) {
+            writeOneAttribute("piccropt", new Integer(this.cropValues[1]));
+        }
+        if (this.cropValues[2] != 0) {
+            writeOneAttribute("piccropr", new Integer(this.cropValues[2]));
+        }
+        if (this.cropValues[3] != 0) {
+            writeOneAttribute("piccropb", new Integer(this.cropValues[3]));
+        }
     }
 
     //////////////////////////////////////////////////
@@ -547,6 +562,24 @@ public class RtfExternalGraphic extends RtfElement {
         this.perCentW = ImageUtil.isPercent(theWidth);
     }
 
+    /**
+     * Sets the desired width of the image.
+     * @param twips The desired image width (in twips)
+     */
+    public void setWidthTwips(int twips) {
+        this.widthDesired = twips;
+        this.perCentW = false;
+    }
+
+    /**
+     * Sets the desired height of the image.
+     * @param twips The desired image height (in twips)
+     */
+    public void setHeightTwips(int twips) {
+        this.heightDesired = twips;
+        this.perCentH = false;
+    }
+
     /**
      * Sets the flag whether the image size shall be adjusted.
      *
@@ -555,15 +588,40 @@ public class RtfExternalGraphic extends RtfElement {
      * false   no adjustment
      */
     public void setScaling(String value) {
-        if (value.equalsIgnoreCase("uniform")) {
-            this.scaleUniform = true;
-        }
+        setUniformScaling("uniform".equalsIgnoreCase(value));
     }
-    
+
+    /**
+     * Sets the flag whether the image size shall be adjusted.
+     *
+     * @param uniform
+     *                true    image width or height shall be adjusted automatically\n
+     *                false   no adjustment
+     */
+    public void setUniformScaling(boolean uniform) {
+        this.scaleUniform = uniform;
+    }
+
+    /**
+     * Sets cropping values for all four edges for the \piccrop*N commands.
+     * A positive value crops toward the center of the picture;
+     * a negative value crops away from the center, adding a space border around the picture
+     * @param left left cropping value (in twips)
+     * @param top top cropping value (in twips)
+     * @param right right cropping value (in twips)
+     * @param bottom bottom cropping value (in twips)
+     */
+    public void setCropping(int left, int top, int right, int bottom) {
+        this.cropValues[0] = left;
+        this.cropValues[1] = top;
+        this.cropValues[2] = right;
+        this.cropValues[3] = bottom;
+    }
+
     /**
      * Sets the binary imagedata of the image.
      *
-     * @param imagedata Binary imagedata as read from file.
+     * @param data  binary imagedata as read from file.
      * @throws IOException On error
      */
     public void setImageData(byte[] data) throws IOException {
