@@ -15,12 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: TXTRenderer.java 542237 2007-05-28 14:31:24Z jeremias $ */
+/* $Id: TXTRenderer.java 612815 2008-01-17 13:37:04Z jeremias $ */
  
 package org.apache.fop.render.txt;
 
 import java.awt.Color;
 import java.awt.Point;
+import java.awt.geom.AffineTransform;
 import java.awt.geom.Rectangle2D;
 import java.io.IOException;
 import java.io.OutputStream;
@@ -109,7 +110,7 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
     public TXTRenderer() {
     }
 
-    /** @see org.apache.fop.render.AbstractRenderer#getMimeType() */
+    /** {@inheritDoc} */
     public String getMimeType() {
         return "text/plain";
     }
@@ -195,7 +196,7 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.Renderer#renderPage(PageViewport)
+     * {@inheritDoc}
      */
     public void renderPage(PageViewport page) throws IOException, FOPException {
         if (firstPage) {
@@ -282,7 +283,7 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.Renderer#startRenderer(java.io.OutputStream)
+     * {@inheritDoc}
      */
     public void startRenderer(OutputStream os) throws IOException {
         log.info("Rendering areas to TEXT.");
@@ -293,7 +294,7 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.Renderer#stopRenderer()
+     * {@inheritDoc}
      */
     public void stopRenderer() throws IOException {
         log.info("writing out TEXT");
@@ -303,7 +304,7 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
 
     /**
      * Does nothing.
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * {@inheritDoc}
      */
     protected void restoreStateStackAfterBreakOut(List breakOutList) {
     }
@@ -311,7 +312,7 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
     /**
      * Does nothing.
      * @return null
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * {@inheritDoc}
      */
     protected List breakOutOfStateStack() {
         return null;
@@ -319,63 +320,65 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
 
     /**
      * Does nothing.
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * {@inheritDoc}
      */
     protected void saveGraphicsState() {
+        currentState.push(new CTM());
     }
 
     /**
      * Does nothing.
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * {@inheritDoc}
      */
     protected void restoreGraphicsState() {
+        currentState.pop();
     }
 
     /**
      * Does nothing.
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * {@inheritDoc}
      */
     protected void beginTextObject() {
     }
 
     /**
      * Does nothing.
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * {@inheritDoc}
      */
     protected void endTextObject() {
     }
 
     /**
      * Does nothing.
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * {@inheritDoc}
      */
     protected void clip() {
     }
 
     /**
      * Does nothing.
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * {@inheritDoc}
      */
     protected void clipRect(float x, float y, float width, float height) {
     }
 
     /**
      * Does nothing. 
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * {@inheritDoc}
      */
     protected void moveTo(float x, float y) {
     }
 
     /**
      * Does nothing. 
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * {@inheritDoc}
      */
     protected void lineTo(float x, float y) {
     }
 
     /**
      * Does nothing.
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * {@inheritDoc}
      */
     protected void closePath() {
     }
@@ -401,7 +404,7 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
     
     /**
      * Fills a rectangular area with the current filling char.
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * {@inheritDoc}
      */
     protected void fillRect(float x, float y, float width, float height) {
         fillRect(bm.getStartX(), bm.getStartY(), bm.getWidth(), bm.getHeight(),
@@ -410,7 +413,7 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
     
     /**
      * Changes current filling char.
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * {@inheritDoc}
      */
     protected void updateColor(Color col, boolean fill) {
         if (col == null) {
@@ -438,17 +441,16 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
         }
     }
 
-    /**
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer#drawImage(
-     *          java.lang.String, java.awt.geom.Rectangle2D, java.util.Map)
-     */
+    /** {@inheritDoc} */
     protected void drawImage(String url, Rectangle2D pos, Map foreignAttributes) {
         //No images are painted here
     }
     
     /**
      * Fills image rectangle with a <code>IMAGE_CHAR</code>.
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * 
+     * @param   image   the base image
+     * @param   pos     the position of the image
      */
     public void renderImage(Image image, Rectangle2D pos) {
         int x1 = Helper.ceilPosition(currentIPPosition, CHAR_WIDTH);
@@ -488,7 +490,7 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer
+     * {@inheritDoc}
      */
     protected void drawBorderLine(float x1, float y1, float x2, float y2,
             boolean horz, boolean startOrBefore, int style, Color col) {
@@ -540,7 +542,7 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractPathOrientedRenderer 
+     * {@inheritDoc} 
      */
     protected void drawBackAndBorders(Area area, float startx, float starty,
             float width, float height) {
@@ -553,16 +555,22 @@ public class TXTRenderer extends AbstractPathOrientedRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#startVParea(CTM, Rectangle2D)
+     * {@inheritDoc}
      */
     protected void startVParea(CTM ctm, Rectangle2D clippingRect) {
         currentState.push(ctm);
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#endVParea()
+     * {@inheritDoc}
      */
     protected void endVParea() {
         currentState.pop();
     }
+    
+    /** {@inheritDoc} */
+    protected void concatenateTransformationMatrix(AffineTransform at) {
+        currentState.push(new CTM(ptToMpt(at)));
+    }
+    
 }
