@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: XMLRenderer.java 542237 2007-05-28 14:31:24Z jeremias $ */
+/* $Id: XMLRenderer.java 651302 2008-04-24 16:08:53Z jeremias $ */
 
 package org.apache.fop.render.xml;
 
@@ -41,7 +41,8 @@ import org.xml.sax.SAXException;
 import org.xml.sax.ext.LexicalHandler;
 import org.xml.sax.helpers.AttributesImpl;
 
-import org.apache.fop.util.QName;
+import org.apache.xmlgraphics.util.XMLizable;
+
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.apps.MimeConstants;
@@ -51,21 +52,23 @@ import org.apache.fop.area.BeforeFloat;
 import org.apache.fop.area.Block;
 import org.apache.fop.area.BlockViewport;
 import org.apache.fop.area.BodyRegion;
+import org.apache.fop.area.BookmarkData;
 import org.apache.fop.area.CTM;
+import org.apache.fop.area.DestinationData;
 import org.apache.fop.area.Footnote;
 import org.apache.fop.area.LineArea;
 import org.apache.fop.area.MainReference;
 import org.apache.fop.area.NormalFlow;
 import org.apache.fop.area.OffDocumentExtensionAttachment;
 import org.apache.fop.area.OffDocumentItem;
-import org.apache.fop.area.BookmarkData;
+import org.apache.fop.area.PageSequence;
 import org.apache.fop.area.PageViewport;
 import org.apache.fop.area.RegionReference;
 import org.apache.fop.area.RegionViewport;
 import org.apache.fop.area.Span;
 import org.apache.fop.area.Trait;
-import org.apache.fop.area.Trait.InternalLink;
 import org.apache.fop.area.Trait.Background;
+import org.apache.fop.area.Trait.InternalLink;
 import org.apache.fop.area.inline.Container;
 import org.apache.fop.area.inline.ForeignObject;
 import org.apache.fop.area.inline.Image;
@@ -87,14 +90,14 @@ import org.apache.fop.render.Renderer;
 import org.apache.fop.render.RendererContext;
 import org.apache.fop.render.XMLHandler;
 import org.apache.fop.util.ColorUtil;
-import org.apache.fop.util.XMLizable;
+import org.apache.fop.util.QName;
 
 /**
  * Renderer that renders areas to XML for debugging purposes.
  * This creates an xml that contains the information of the area
  * tree. It does not output any state or derived information.
- * The output can be used to build a new area tree (@see AreaTreeBuilder)
- * which can be rendered to any renderer.
+ * The output can be used to build a new area tree which can be 
+ * rendered to any renderer.
  */
 public class XMLRenderer extends PrintRenderer {
 
@@ -137,7 +140,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.Renderer#setUserAgent(FOUserAgent)
+     * {@inheritDoc}
      */
     public void setUserAgent(FOUserAgent agent) {
         super.setUserAgent(agent);
@@ -159,7 +162,7 @@ public class XMLRenderer extends PrintRenderer {
         this.mimic = renderer;
     }
 
-    /** @see org.apache.fop.render.PrintRenderer#setupFontInfo(org.apache.fop.fonts.FontInfo) */
+    /** {@inheritDoc} */
     public void setupFontInfo(FontInfo inFontInfo) {
         if (mimic != null) {
             mimic.setupFontInfo(inFontInfo);
@@ -415,10 +418,12 @@ public class XMLRenderer extends PrintRenderer {
         }
     }
 
-    /** @see org.apache.fop.render.AbstractRenderer#processOffDocumentItem(OffDocumentItem) */
+    /** {@inheritDoc} */
     public void processOffDocumentItem(OffDocumentItem oDI) {
         if (oDI instanceof BookmarkData) {
             renderBookmarkTree((BookmarkData) oDI);
+        } else if (oDI instanceof DestinationData) {
+            renderDestination((DestinationData) oDI);
         } else if (oDI instanceof OffDocumentExtensionAttachment) {
             ExtensionAttachment attachment = ((OffDocumentExtensionAttachment)oDI).getAttachment();
             if (extensionAttachments == null) {
@@ -466,8 +471,23 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.Renderer#startRenderer(OutputStream)
+     * Renders a DestinationData object (named destination)
+     * @param destination the destination object
      */
+    protected void renderDestination(DestinationData destination) {
+        if (destination.getWhenToProcess() == OffDocumentItem.END_OF_DOC) {
+            endPageSequence();
+        }
+        atts.clear();
+        PageViewport pv = destination.getPageViewport();
+        String pvKey = pv == null ? null : pv.getKey();
+        addAttribute("internal-link",
+                InternalLink.makeXMLAttribute(pvKey, destination.getIDRef()));
+        startElement("destination", atts);
+        endElement("destination");
+    }
+
+    /** {@inheritDoc} */
     public void startRenderer(OutputStream outputStream)
                 throws IOException {
         log.debug("Rendering areas to Area Tree XML");
@@ -499,7 +519,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.Renderer#stopRenderer()
+     * {@inheritDoc}
      */
     public void stopRenderer() throws IOException {
         endPageSequence();
@@ -516,7 +536,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.Renderer#renderPage(PageViewport)
+     * {@inheritDoc}
      */
     public void renderPage(PageViewport page) throws IOException, FOPException {
         atts.clear();
@@ -524,7 +544,9 @@ public class XMLRenderer extends PrintRenderer {
         addAttribute("key", page.getKey());
         addAttribute("nr", page.getPageNumber());
         addAttribute("formatted-nr", page.getPageNumberString());
-        addAttribute("simple-page-master-name", page.getSimplePageMasterName());
+        if (page.getSimplePageMasterName() != null) {
+            addAttribute("simple-page-master-name", page.getSimplePageMasterName());
+        }
         if (page.isBlank()) {
             addAttribute("blank", "true");
         }
@@ -564,14 +586,20 @@ public class XMLRenderer extends PrintRenderer {
         handleExtensionAttachments(page.getExtensionAttachments());
     }
 
-    /**
-     * @see org.apache.fop.render.Renderer#startPageSequence(LineArea)
-     */
-    public void startPageSequence(LineArea seqTitle) {
+    /** {@inheritDoc} */
+    public void startPageSequence(PageSequence pageSequence) {
         handleDocumentExtensionAttachments();
         endPageSequence();  // move this before handleDocumentExtensionAttachments() ?
         startedSequence = true;
-        startElement("pageSequence");
+        atts.clear();
+        if (pageSequence.getLanguage() != null) {
+            addAttribute("language", pageSequence.getLanguage());
+        }
+        if (pageSequence.getCountry() != null) {
+            addAttribute("country", pageSequence.getCountry());
+        }
+        startElement("pageSequence", atts);
+        LineArea seqTitle = pageSequence.getTitle();
         if (seqTitle != null) {
             startElement("title");
             List children = seqTitle.getInlineAreas();
@@ -596,7 +624,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderRegionViewport(RegionViewport)
+     * {@inheritDoc}
      */
     protected void renderRegionViewport(RegionViewport port) {
         if (port != null) {
@@ -644,18 +672,18 @@ public class XMLRenderer extends PrintRenderer {
         }
     }
 
-    /** @see org.apache.fop.render.AbstractRenderer */
+    /** {@inheritDoc} */
     protected void startVParea(CTM ctm, Rectangle2D clippingRect) {
         //only necessary for graphical output
     }
 
-    /** @see org.apache.fop.render.AbstractRenderer#endVParea() */
+    /** {@inheritDoc} */
     protected void endVParea() {
         //only necessary for graphical output
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderInlineAreaBackAndBorders(
+     * {@inheritDoc}
      *          org.apache.fop.area.inline.InlineArea)
      */
     protected void renderInlineAreaBackAndBorders(InlineArea area) {
@@ -663,7 +691,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderBeforeFloat(BeforeFloat)
+     * {@inheritDoc}
      */
     protected void renderBeforeFloat(BeforeFloat bf) {
         startElement("beforeFloat");
@@ -672,7 +700,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderFootnote(Footnote)
+     * {@inheritDoc}
      */
     protected void renderFootnote(Footnote footnote) {
         startElement("footnote");
@@ -681,7 +709,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderMainReference(MainReference)
+     * {@inheritDoc}
      */
     protected void renderMainReference(MainReference mr) {
         atts.clear();
@@ -714,7 +742,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderFlow(NormalFlow)
+     * {@inheritDoc}
      */
     protected void renderFlow(NormalFlow flow) {
         // the normal flow reference area contains stacked blocks
@@ -726,9 +754,17 @@ public class XMLRenderer extends PrintRenderer {
         endElement("flow");
     }
 
-    /**
-     * @see org.apache.fop.render.AbstractRenderer#renderBlock(Block)
-     */
+    /** {@inheritDoc} */
+    protected void renderReferenceArea(Block block) {
+        handleBlockTraits(block);
+
+        List children = block.getChildAreas();
+        if (children != null) {
+            renderBlocks(block, children);
+        }
+    }
+    
+    /** {@inheritDoc} */
     protected void renderBlock(Block block) {
         atts.clear();
         addAreaAttributes(block);
@@ -775,7 +811,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderLineArea(LineArea)
+     * {@inheritDoc}
      */
     protected void renderLineArea(LineArea line) {
         atts.clear();
@@ -787,7 +823,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderInlineArea(InlineArea)
+     * {@inheritDoc}
      */
     protected void renderInlineArea(InlineArea inlineArea) {
         atts.clear();
@@ -805,7 +841,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderViewport(Viewport)
+     * {@inheritDoc}
      */
     protected void renderViewport(Viewport viewport) {
         atts.clear();
@@ -822,7 +858,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer
+     * {@inheritDoc}
      */
     public void renderImage(Image image, Rectangle2D pos) {
         atts.clear();
@@ -835,7 +871,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderContainer(Container)
+     * {@inheritDoc}
      */
     public void renderContainer(Container cont) {
         startElement("container");
@@ -863,7 +899,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderInlineSpace(Space)
+     * {@inheritDoc}
      */
     protected void renderInlineSpace(Space space) {
         atts.clear();
@@ -875,7 +911,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderText(TextArea)
+     * {@inheritDoc}
      */
     protected void renderText(TextArea text) {
         atts.clear();
@@ -895,7 +931,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderWord(WordArea)
+     * {@inheritDoc}
      */
     protected void renderWord(WordArea word) {
         atts.clear();
@@ -922,7 +958,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderSpace(SpaceArea)
+     * {@inheritDoc}
      */
     protected void renderSpace(SpaceArea space) {
         atts.clear();
@@ -937,7 +973,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderInlineParent(InlineParent)
+     * {@inheritDoc}
      */
     protected void renderInlineParent(InlineParent ip) {
         atts.clear();
@@ -950,7 +986,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderInlineBlockParent(InlineBlockParent)
+     * {@inheritDoc}
      */
     protected void renderInlineBlockParent(InlineBlockParent ibp) {
         atts.clear();
@@ -963,7 +999,7 @@ public class XMLRenderer extends PrintRenderer {
     }
 
     /**
-     * @see org.apache.fop.render.AbstractRenderer#renderLeader(Leader)
+     * {@inheritDoc}
      */
     protected void renderLeader(Leader area) {
         atts.clear();
@@ -977,7 +1013,7 @@ public class XMLRenderer extends PrintRenderer {
         super.renderLeader(area);
     }
 
-    /** @see org.apache.fop.render.AbstractRenderer#getMimeType() */
+    /** {@inheritDoc} */
     public String getMimeType() {
         return XML_MIME_TYPE;
     }
