@@ -15,16 +15,26 @@
  * limitations under the License.
  */
 
-/* $Id: PDFTranscoder.java 542237 2007-05-28 14:31:24Z jeremias $ */
+/* $Id: PDFTranscoder.java 611278 2008-01-11 19:50:53Z jeremias $ */
 
 package org.apache.fop.svg;
 
 import java.awt.Color;
+import java.io.BufferedOutputStream;
 import java.io.IOException;
+import java.io.OutputStream;
+import java.io.InputStream;
+
+import javax.xml.transform.Source;
+import javax.xml.transform.stream.StreamSource;
+
+import org.w3c.dom.Document;
+import org.w3c.dom.svg.SVGLength;
 
 import org.apache.avalon.framework.configuration.Configurable;
 import org.apache.avalon.framework.configuration.Configuration;
 import org.apache.avalon.framework.configuration.ConfigurationException;
+import org.apache.avalon.framework.configuration.DefaultConfiguration;
 import org.apache.batik.bridge.BridgeContext;
 import org.apache.batik.bridge.UnitProcessor;
 import org.apache.batik.bridge.UserAgent;
@@ -33,12 +43,17 @@ import org.apache.batik.transcoder.TranscoderException;
 import org.apache.batik.transcoder.TranscoderOutput;
 import org.apache.batik.transcoder.TranscodingHints;
 import org.apache.batik.transcoder.image.ImageTranscoder;
+import org.apache.batik.transcoder.keys.BooleanKey;
 import org.apache.batik.transcoder.keys.FloatKey;
+import org.apache.batik.util.ParsedURL;
+
+import org.apache.xmlgraphics.image.loader.ImageContext;
+import org.apache.xmlgraphics.image.loader.ImageManager;
+import org.apache.xmlgraphics.image.loader.ImageSessionContext;
+import org.apache.xmlgraphics.image.loader.impl.AbstractImageSessionContext;
+
 import org.apache.fop.Version;
 import org.apache.fop.fonts.FontInfo;
-import org.apache.fop.fonts.FontSetup;
-import org.w3c.dom.Document;
-import org.w3c.dom.svg.SVGLength;
 
 /**
  * This class enables to transcode an input to a pdf document.
@@ -63,9 +78,15 @@ import org.w3c.dom.svg.SVGLength;
  * <tt>KEY_USER_STYLESHEET_URI</tt> to fix the URI of a user
  * stylesheet, and <tt>KEY_PIXEL_TO_MM</tt> to specify the pixel to
  * millimeter conversion factor.
+ * 
+ * <p><tt>KEY_AUTO_FONTS</tt> to disable the auto-detection of fonts installed in the system.
+ * The PDF Transcoder cannot use AWT's font subsystem and that's why the fonts have to be
+ * configured differently. By default, font auto-detection is enabled to match the behaviour
+ * of the other transcoders, but this may be associated with a price in the form of a small
+ * performance penalty. If font auto-detection is not desired, it can be disable using this key.
  *
  * @author <a href="mailto:keiron@aftexsw.com">Keiron Liddle</a>
- * @version $Id: PDFTranscoder.java 542237 2007-05-28 14:31:24Z jeremias $
+ * @version $Id: PDFTranscoder.java 611278 2008-01-11 19:50:53Z jeremias $
  */
 public class PDFTranscoder extends AbstractFOPTranscoder
         implements Configurable {
@@ -76,13 +97,23 @@ public class PDFTranscoder extends AbstractFOPTranscoder
      */
     public static final TranscodingHints.Key KEY_DEVICE_RESOLUTION = new FloatKey();
 
+    /**
+     * The key is used to specify whether the available fonts should be automatically
+     * detected. The alternative is to configure the transcoder manually using a configuration
+     * file. 
+     */
+    public static final TranscodingHints.Key KEY_AUTO_FONTS = new BooleanKey();
+
     private Configuration cfg = null;
     
     /** Graphics2D instance that is used to paint to */
     protected PDFDocumentGraphics2D graphics = null;
 
+    private ImageManager imageManager;
+    private ImageSessionContext imageSessionContext;
+    
     /**
-     * Constructs a new <tt>ImageTranscoder</tt>.
+     * Constructs a new <tt>PDFTranscoder</tt>.
      */
     public PDFTranscoder() {
         super();
@@ -90,7 +121,7 @@ public class PDFTranscoder extends AbstractFOPTranscoder
     }
 
     /**
-     * @see org.apache.fop.svg.AbstractFOPTranscoder#createUserAgent()
+     * {@inheritDoc}
      */
     protected UserAgent createUserAgent() {
         return new AbstractFOPTranscoder.FOPTranscoderUserAgent() {
@@ -102,9 +133,7 @@ public class PDFTranscoder extends AbstractFOPTranscoder
         };
     }
     
-    /**
-     * @see org.apache.avalon.framework.configuration.Configurable#configure(Configuration)
-     */
+    /** {@inheritDoc} */
     public void configure(Configuration cfg) throws ConfigurationException {
         this.cfg = cfg;
     }
@@ -121,16 +150,40 @@ public class PDFTranscoder extends AbstractFOPTranscoder
                              TranscoderOutput output) 
         throws TranscoderException {
 
-        graphics = new PDFDocumentGraphics2D();
+        graphics = new PDFDocumentGraphics2D(isTextStroked());
         graphics.getPDFDocument().getInfo().setProducer("Apache FOP Version " 
                 + Version.getVersion() 
                 + ": PDF Transcoder for Batik");
+        if (hints.containsKey(KEY_DEVICE_RESOLUTION)) {
+            graphics.setDeviceDPI(((Float)hints.get(KEY_DEVICE_RESOLUTION)).floatValue());
+        }
+        
+        setupImageInfrastructure(uri);
         
         try {
-            if (this.cfg != null) {
+            Configuration effCfg = this.cfg; 
+            if (effCfg == null) {
+                //By default, enable font auto-detection if no cfg is given
+                boolean autoFonts = true;
+                if (hints.containsKey(KEY_AUTO_FONTS)) {
+                    autoFonts = ((Boolean)hints.get(KEY_AUTO_FONTS)).booleanValue();
+                }
+                if (autoFonts) {
+                    DefaultConfiguration c = new DefaultConfiguration("pdf-transcoder");
+                    DefaultConfiguration fonts = new DefaultConfiguration("fonts");
+                    c.addChild(fonts);
+                    DefaultConfiguration autodetect = new DefaultConfiguration("auto-detect");
+                    fonts.addChild(autodetect);
+                    effCfg = c;
+                }
+            }
+            
+            if (effCfg != null) {
                 PDFDocumentGraphics2DConfigurator configurator
                         = new PDFDocumentGraphics2DConfigurator();
-                configurator.configure(graphics, this.cfg);
+                configurator.configure(graphics, effCfg);
+            } else {
+                graphics.setupDefaultFontInfo();
             }
         } catch (Exception e) {
             throw new TranscoderException(
@@ -161,10 +214,11 @@ public class PDFTranscoder extends AbstractFOPTranscoder
         //int h = (int)(height + 0.5);
 
         try {
-            if (hints.containsKey(KEY_DEVICE_RESOLUTION)) {
-                graphics.setDeviceDPI(((Float)hints.get(KEY_DEVICE_RESOLUTION)).floatValue());
+            OutputStream out = output.getOutputStream();
+            if (!(out instanceof BufferedOutputStream)) {
+                out = new BufferedOutputStream(out);
             }
-            graphics.setupDocument(output.getOutputStream(), w, h);
+            graphics.setupDocument(out, w, h);
             graphics.setSVGDimension(width, height);
 
             if (hints.containsKey(ImageTranscoder.KEY_BACKGROUND_COLOR)) {
@@ -188,10 +242,54 @@ public class PDFTranscoder extends AbstractFOPTranscoder
         }
     }
 
-    /** @see org.apache.batik.transcoder.SVGAbstractTranscoder#createBridgeContext() */
-    protected BridgeContext createBridgeContext() {
-        BridgeContext ctx = new PDFBridgeContext(userAgent, graphics.getFontInfo());
-        return ctx;
+    private void setupImageInfrastructure(final String baseURI) {
+        final ImageContext imageContext = new ImageContext() {
+            public float getSourceResolution() {
+                return 25.4f / userAgent.getPixelUnitToMillimeter();
+            }
+        };
+        this.imageManager = new ImageManager(imageContext);
+        this.imageSessionContext = new AbstractImageSessionContext() {
+
+            public ImageContext getParentContext() {
+                return imageContext;
+            }
+
+            public float getTargetResolution() {
+                return graphics.getDeviceDPI();
+            }
+
+            public Source resolveURI(String uri) {
+                System.out.println("resolve " + uri);
+                try {
+                    ParsedURL url = new ParsedURL(baseURI, uri);
+                    InputStream in = url.openStream();
+                    StreamSource source = new StreamSource(in, url.toString());
+                    return source;
+                } catch (IOException ioe) {
+                    userAgent.displayError(ioe);
+                    return null;
+                }
+            }
+            
+        };
     }
 
+    /** {@inheritDoc} */
+    protected BridgeContext createBridgeContext() {
+        //For compatibility with Batik 1.6
+        return createBridgeContext("1.x");
+    }
+
+    /** {@inheritDoc} */
+    public BridgeContext createBridgeContext(String version) {
+        FontInfo fontInfo = graphics.getFontInfo();
+        if (isTextStroked()) {
+            fontInfo = null;
+        }
+        BridgeContext ctx = new PDFBridgeContext(userAgent, fontInfo,
+                this.imageManager, this.imageSessionContext);
+        return ctx;
+    }
+    
 }
