@@ -15,29 +15,29 @@
  * limitations under the License.
  */
 
-/* $Id: CloseBlockerOutputStream.java 563951 2007-08-08 17:37:33Z vhennebert $ */
+/* $Id: CloseBlockerOutputStream.java 647537 2008-04-13 07:36:00Z jeremias $ */
  
 package org.apache.fop.util;
 
-import java.io.FilterOutputStream;
 import java.io.IOException;
 import java.io.OutputStream;
 
+import org.apache.commons.io.output.ProxyOutputStream;
+
 /**
  * This is a decorator to block calls to close() to the underlying stream.
  */
-public class CloseBlockerOutputStream extends FilterOutputStream {
+public class CloseBlockerOutputStream extends ProxyOutputStream {
 
     /**
-     * @see java.io.FilterOutputStream#FilterOutputStream(OutputStream)
+     * Main constructor.
+     * @param out the underlying stream
      */
     public CloseBlockerOutputStream(OutputStream out) {
         super(out);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void close() throws IOException {
         try {
             flush();
