@@ -15,40 +15,34 @@
  * limitations under the License.
  */
 
-/* $Id: BatchDiffer.java 512686 2007-02-28 10:03:38Z jeremias $ */
+/* $Id: BatchDiffer.java 612816 2008-01-17 13:43:33Z jeremias $ */
 
 package org.apache.fop.visual;
 
 import java.awt.image.BufferedImage;
-import java.awt.image.RenderedImage;
 import java.io.File;
-import java.io.FileOutputStream;
 import java.io.IOException;
-import java.io.OutputStream;
 import java.util.Collection;
 import java.util.Iterator;
 
 import javax.xml.transform.TransformerConfigurationException;
 import javax.xml.transform.stream.StreamSource;
 
+import org.xml.sax.SAXException;
+
 import org.apache.avalon.framework.configuration.Configuration;
 import org.apache.avalon.framework.configuration.ConfigurationException;
 import org.apache.avalon.framework.configuration.DefaultConfigurationBuilder;
 import org.apache.avalon.framework.container.ContainerUtil;
-
 import org.apache.commons.io.FileUtils;
-import org.apache.commons.io.IOUtils;
 import org.apache.commons.io.filefilter.IOFileFilter;
 import org.apache.commons.io.filefilter.SuffixFileFilter;
-
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
-import org.apache.fop.layoutengine.LayoutEngineTestSuite;
-import org.apache.xmlgraphics.image.writer.ImageWriter;
-import org.apache.xmlgraphics.image.writer.ImageWriterRegistry;
+import org.apache.xmlgraphics.image.writer.ImageWriterUtil;
 
-import org.xml.sax.SAXException;
+import org.apache.fop.layoutengine.LayoutEngineTestSuite;
 
 /**
  * This class is used to visually diff bitmap images created through various sources.
@@ -113,22 +107,6 @@ public class BatchDiffer {
                 + " for the batch run.");
     }
 
-    /**
-     * Saves a BufferedImage as a PNG file.
-     * @param bitmap the bitmap to encode
-     * @param outputFile the target file
-     * @throws IOException in case of an I/O problem
-     */
-    public static void saveAsPNG(RenderedImage bitmap, File outputFile) throws IOException {
-        OutputStream out = new FileOutputStream(outputFile);
-        try {
-            ImageWriter writer = ImageWriterRegistry.getInstance().getWriterFor("image/png");
-            writer.writeImage(bitmap, out);
-        } finally {
-            IOUtils.closeQuietly(out);
-        }
-    }
-    
     /**
      * Runs the batch.
      * @param cfgFile configuration file to use
@@ -169,7 +147,7 @@ public class BatchDiffer {
             if (!srcDir.exists()) {
                 throw new RuntimeException("source-directory does not exist: " + srcDir);
             }
-            File targetDir = new File(cfg.getChild("target-directory").getValue());
+            final File targetDir = new File(cfg.getChild("target-directory").getValue());
             targetDir.mkdirs();
             if (!targetDir.exists()) {
                 throw new RuntimeException("target-directory is invalid: " + targetDir);
@@ -177,10 +155,9 @@ public class BatchDiffer {
             context.setTargetDir(targetDir);
             
             boolean stopOnException = cfg.getChild("stop-on-exception").getValueAsBoolean(true);
-            boolean createDiffs = cfg.getChild("create-diffs").getValueAsBoolean(true);
+            final boolean createDiffs = cfg.getChild("create-diffs").getValueAsBoolean(true);
             
             //RUN!
-            BufferedImage[] bitmaps = new BufferedImage[producers.length];
             
             IOFileFilter filter = new SuffixFileFilter(new String[] {".xml", ".fo"});
             //Same filtering as in layout engine tests
@@ -192,50 +169,47 @@ public class BatchDiffer {
             Collection files = FileUtils.listFiles(srcDir, filter, null);
             Iterator i = files.iterator();
             while (i.hasNext()) {
+                final File f = (File)i.next();
                 try {
-                    File f = (File)i.next();
                     log.info("---=== " + f + " ===---");
                     long[] times = new long[producers.length];
+                    final BufferedImage[] bitmaps = new BufferedImage[producers.length];
                     for (int j = 0; j < producers.length; j++) {
                         times[j] = System.currentTimeMillis();
                         bitmaps[j] = producers[j].produce(f, context);
                         times[j] = System.currentTimeMillis() - times[j];
                     }
-                    StringBuffer sb = new StringBuffer("Bitmap production times: ");
-                    for (int j = 0; j < producers.length; j++) {
-                        if (j > 0) {
-                            sb.append(", ");
+                    if (log.isDebugEnabled()) {
+                        StringBuffer sb = new StringBuffer("Bitmap production times: ");
+                        for (int j = 0; j < producers.length; j++) {
+                            if (j > 0) {
+                                sb.append(", ");
+                            }
+                            sb.append(times[j]).append("ms");
                         }
-                        sb.append(times[j]).append("ms");
+                        log.debug(sb.toString());
                     }
-                    log.info(sb.toString());
                     //Create combined image
                     if (bitmaps[0] == null) {
-                        throw new RuntimeException("First producer didn't return a bitmap."
-                                + " Cannot continue.");
+                        throw new RuntimeException("First producer didn't return a bitmap for " 
+                                + f + ". Cannot continue.");
                     }
-                    BufferedImage combined = BitmapComparator.buildCompareImage(bitmaps);
                     
-                    //Save combined bitmap as PNG file
-                    File outputFile = new File(targetDir, f.getName() + "._combined.png");
-                    saveAsPNG(combined, outputFile);
-
-                    if (createDiffs) {
-                        for (int k = 1; k < bitmaps.length; k++) {
-                            BufferedImage diff = BitmapComparator.buildDiffImage(
-                                    bitmaps[0], bitmaps[k]);
-                            outputFile = new File(targetDir, f.getName() + "._diff" + k + ".png");
-                            saveAsPNG(diff, outputFile);
+                    Runnable runnable = new Runnable() {
+                        public void run() {
+                            try {
+                                saveBitmaps(targetDir, f, createDiffs, bitmaps);
+                            } catch (IOException e) {
+                                log.error("IO error while saving bitmaps: " + e.getMessage());
+                            }
                         }
-                    }
-                    //Release memory as soon as possible. These images are huge!
-                    for (int k = 0; k < bitmaps.length; k++) {
-                        bitmaps[k] = null;
-                    }
+                    };
+                    //This speeds it up a little on multi-core CPUs (very cheap, I know)
+                    new Thread(runnable).start();
                 } catch (RuntimeException e) {
-                    System.out.println("Catching RE: " + e.getMessage());
+                    log.error("Catching RE on file " + f + ": " + e.getMessage());
                     if (stopOnException) {
-                        System.out.println("rethrowing...");
+                        log.error("rethrowing...");
                         throw e;
                     }
                 }
@@ -253,6 +227,24 @@ public class BatchDiffer {
         }
     }
     
+    private void saveBitmaps(File targetDir, File srcFile, boolean createDiffs,
+            BufferedImage[] bitmaps) throws IOException {
+        BufferedImage combined = BitmapComparator.buildCompareImage(bitmaps);
+        
+        //Save combined bitmap as PNG file
+        File outputFile = new File(targetDir, srcFile.getName() + "._combined.png");
+        ImageWriterUtil.saveAsPNG(combined, outputFile);
+
+        if (createDiffs) {
+            for (int k = 1; k < bitmaps.length; k++) {
+                BufferedImage diff = BitmapComparator.buildDiffImage(
+                        bitmaps[0], bitmaps[k]);
+                outputFile = new File(targetDir, srcFile.getName() + "._diff" + k + ".png");
+                ImageWriterUtil.saveAsPNG(diff, outputFile);
+            }
+        }
+    }
+    
     private BitmapProducer[] getProducers(Configuration cfg) {
         Configuration[] children = cfg.getChildren("producer");
         BitmapProducer[] producers = new BitmapProducer[children.length];
@@ -293,7 +285,7 @@ public class BatchDiffer {
             
             System.out.println("Regular exit...");
         } catch (Exception e) {
-            System.out.println("Exception caugth...");
+            System.out.println("Exception caught...");
             e.printStackTrace();
         }
     }
